@@ -27,13 +27,16 @@ package org.jmol.adapter.readers.cifpdb;
 import org.jmol.adapter.smarter.*;
 
 
+
 import org.jmol.api.JmolAdapter;
+import org.jmol.constant.EnumStructure;
 import org.jmol.util.Logger;
 import org.jmol.util.TextFormat;
 
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import javax.vecmath.Matrix4f;
 
@@ -63,12 +66,12 @@ public class PdbReader extends AtomSetCollectionReader {
   private int lineLength;
   // index into atoms array + 1
   // so that 0 can be used for the null value
-  private final Hashtable htFormul = new Hashtable();
-  private Hashtable htHetero = null;
-  private Hashtable htSites = null;
+  private final Map<String, Map<String, Boolean>> htFormul = new Hashtable<String, Map<String, Boolean>>();
+  private Map<String, String> htHetero = null;
+  private Map<String, Map<String, Object>> htSites = null;
   private String currentGroup3;
   private int currentResno = Integer.MIN_VALUE;
-  private Hashtable htElementsInCurrentGroup;
+  private Map<String, Boolean> htElementsInCurrentGroup;
   private int maxSerial;
   private int[] chainAtomCounts;
   private int nUNK;
@@ -96,15 +99,21 @@ public class PdbReader extends AtomSetCollectionReader {
    "REMARK  " + //17
    "HEADER  " + //18
    "COMPND  " + //19
-   "SOURCE  ";  //20
+   "SOURCE  " + //20
+   "TITLE   ";  //21
 
  private int serial = 0;
  private StringBuffer pdbHeader;
  private int configurationPtr = Integer.MIN_VALUE;
- 
+ private boolean applySymmetry;
+
+ @Override
  protected void initializeReader() throws Exception {
-   atomSetCollection.setIsPDB();
+   setIsPDB();
    pdbHeader = (getHeader ? new StringBuffer() : null);
+   applySymmetry = !checkFilter("NOSYMMETRY");
+
+   
    if (checkFilter("CONF ")) {
      configurationPtr = parseInt(filter, filter.indexOf("CONF ") + 5);
      sbIgnored = new StringBuffer();
@@ -112,14 +121,16 @@ public class PdbReader extends AtomSetCollectionReader {
    }
  }
 
- protected boolean checkLine() throws Exception {
+  @Override
+  protected boolean checkLine() throws Exception {
     int ptOption = ((lineLength = line.length()) < 6 ? -1 : lineOptions
         .indexOf(line.substring(0, 6))) >> 3;
     boolean isAtom = (ptOption == 0 || ptOption == 1);
     boolean isModel = (ptOption == 2);
     if (isAtom)
       serial = parseInt(line, 6, 11);
-    boolean isNewModel = ((isTrajectory || isSequential) && !isMultiModel && isAtom && serial == 1);
+    boolean isNewModel = ((isTrajectory || isSequential) && !isMultiModel
+        && isAtom && serial == 1);
     if (getHeader) {
       if (isAtom || isModel)
         getHeader = false;
@@ -131,7 +142,6 @@ public class PdbReader extends AtomSetCollectionReader {
       getHeader = false;
       // PDB is different -- targets actual model number
       int modelNo = (isNewModel ? modelNumber + 1 : getModelNumber());
-      // System.out.println(modelNo);
       modelNumber = (bsModels == null ? modelNo : modelNumber + 1);
       if (!doGetModel(modelNumber))
         return checkLastModel();
@@ -140,7 +150,7 @@ public class PdbReader extends AtomSetCollectionReader {
         applySymmetryAndSetTrajectory();
       // supposedly MODEL is only for NMR
       model(modelNo);
-      if (!isAtom)
+      if (stateScriptVersionInt < 120000 || !isAtom) // Jmol 12.0.RC24 fixed this bug, but for earlier scripts we need to unfix it.
         return true;
     }
     /*
@@ -218,10 +228,14 @@ public class PdbReader extends AtomSetCollectionReader {
     case 20:
       compndSource(true);
       return true;
+    case 21:
+      title();
+      return true;
     }
     return true;
   }
 
+  @Override
   protected void finalizeReader() throws Exception {
     checkNotPDB();
     atomSetCollection.connectAll(maxSerial);
@@ -229,15 +243,16 @@ public class PdbReader extends AtomSetCollectionReader {
         && atomSetCollection.getAtomCount() > 0) {
       atomSetCollection.setAtomSetAuxiliaryInfo("biomolecules", biomolecules);
       setBiomoleculeAtomCounts();
-      if (biomts != null && !checkFilter("NOSYMMETRY")) {
+      if (biomts != null && applySymmetry) {
         atomSetCollection.applySymmetry(biomts, notionalUnitCell, applySymmetryToBonds, filter);
       }
     }
     super.finalizeReader();
     if (vCompnds != null)
       atomSetCollection.setAtomSetCollectionAuxiliaryInfo("compoundSource", vCompnds);
-    if (htSites != null)// && atomSetCollection.getAtomSetCount() == 1)
+    if (htSites != null) { // && atomSetCollection.getAtomSetCount() == 1)
       addSites(htSites);
+    }
     if (pdbHeader != null)
       atomSetCollection.setAtomSetCollectionAuxiliaryInfo("fileHeader",
           pdbHeader.toString());
@@ -247,18 +262,10 @@ public class PdbReader extends AtomSetCollectionReader {
     }
   }
   
-  public void applySymmetryAndSetTrajectory() throws Exception {
-    // This speeds up calculation, because no crosschecking
-    // No special-position atoms in mmCIF files, because there will
-    // be no center of symmetry, no rotation-inversions, 
-    // no atom-centered rotation axes, and no mirror or glide planes. 
-    atomSetCollection.setCheckSpecial(false);
-    super.applySymmetryAndSetTrajectory();
-  }
-
   private void header() {
     if (lineLength < 8)
       return;
+    appendLoadNote(line.substring(7).trim());
     if (lineLength >= 66)
       atomSetCollection.setCollectionName(line.substring(62, 66));
     if (lineLength > 50)
@@ -266,32 +273,25 @@ public class PdbReader extends AtomSetCollectionReader {
     atomSetCollection.setAtomSetCollectionAuxiliaryInfo("CLASSIFICATION", line.substring(7).trim());
   }
 
-  private List vCompnds;
-  private Hashtable currentCompnd;
-  private String currentKey;
-  private Hashtable htMolIds;
-  private boolean resetKey = true;
-  
-  String compnd = null;
-  private void compndOld() {
-    if (compnd == null)
-      compnd = "";
-    else
-      compnd += " ";
-    String s = line;
-    if (lineLength > 62)
-      s = s.substring(0, 62);
-    compnd += s.substring(10).trim();
-    atomSetCollection.setAtomSetCollectionAuxiliaryInfo("COMPND", compnd);
+  private void title() {
+    if (lineLength < 10)
+      return;
+    appendLoadNote(line.substring(10).trim());
   }
 
+  private List<Map<String, String>> vCompnds;
+  private Map<String, String> currentCompnd;
+  private String currentKey;
+  private Map<String, Map<String, String>> htMolIds;
+  private boolean resetKey = true;
+  
   private void compndSource(boolean isSource) {
     if (vCompnds == null) {
       if (isSource)
         return;
-      vCompnds = new ArrayList();
-      htMolIds = new Hashtable();
-      currentCompnd = new Hashtable();
+      vCompnds = new ArrayList<Map<String,String>>();
+      htMolIds = new Hashtable<String, Map<String,String>>();
+      currentCompnd = new Hashtable<String, String>();
       currentCompnd.put("select", "(*)");
       currentKey = "MOLECULE";
       htMolIds.put("", currentCompnd);
@@ -299,7 +299,7 @@ public class PdbReader extends AtomSetCollectionReader {
     if (isSource && resetKey) {
       resetKey = false;
       currentKey = "SOURCE";
-      currentCompnd = (Hashtable) htMolIds.get("");
+      currentCompnd = htMolIds.get("");
     }
     line = line.substring(10, Math.min(lineLength, 72)).trim();
     int pt = line.indexOf(":");
@@ -311,17 +311,17 @@ public class PdbReader extends AtomSetCollectionReader {
       if (value == null)
         return;
       if (isSource) {
-        currentCompnd = (Hashtable) htMolIds.remove(value);
+        currentCompnd = htMolIds.remove(value);
         return;
       }
-      currentCompnd = new Hashtable();
+      currentCompnd = new Hashtable<String, String>();
       vCompnds.add(currentCompnd);
       htMolIds.put(value, currentCompnd);
     }
     if (currentCompnd == null)
       return;
     if (value == null) {
-      value = (String) currentCompnd.get(currentKey);
+      value = currentCompnd.get(currentKey);
       if (value == null)
         value = "";
       value += key;
@@ -339,16 +339,31 @@ public class PdbReader extends AtomSetCollectionReader {
               .simpleReplace(value, ", ", ",:"), " ", "") + ")");
   }
 
+  private String compnd = null;
+  private void compndOld() {
+    if (compnd == null)
+      compnd = "";
+    else
+      compnd += " ";
+    String s = line;
+    if (lineLength > 62)
+      s = s.substring(0, 62);
+    compnd += s.substring(10).trim();
+    atomSetCollection.setAtomSetCollectionAuxiliaryInfo("COMPND", compnd);
+  }
+
+
+  @SuppressWarnings("unchecked")
   private void setBiomoleculeAtomCounts() {
     for (int i = biomolecules.size(); --i >= 0;) {
-      Hashtable biomolecule = (Hashtable) (biomolecules.get(i));
+      Map<String, Object> biomolecule = biomolecules.get(i);
       String chain = (String) biomolecule.get("chains");
-      int nTransforms = ((List) biomolecule.get("biomts")).size();
+      int nTransforms = ((List<Matrix4f>) biomolecule.get("biomts")).size();
       int nAtoms = 0;
       for (int j = chain.length() - 1; --j >= 0;)
         if (chain.charAt(j) == ':')
           nAtoms += chainAtomCounts[chain.charAt(j + 1)];
-      biomolecule.put("atomCount", new Integer(nAtoms * nTransforms));
+      biomolecule.put("atomCount", Integer.valueOf(nAtoms * nTransforms));
     }
   }
 
@@ -385,18 +400,18 @@ REMARK 350   BIOMT3   3  0.000000  0.000000  1.000000        0.00000
 
 */
  
-  private List biomolecules;
-  private List biomts;
+  private List<Map<String, Object>> biomolecules;
+  private List<Matrix4f> biomts;
   
   private void remark350() throws Exception {
-    List biomts = null;
-    biomolecules = new ArrayList();
+    List<Matrix4f> biomts = null;
+    biomolecules = new ArrayList<Map<String,Object>>();
     chainAtomCounts = new int[255];
     String title = "";
     String chainlist = "";
     int iMolecule = 0;
     boolean needLine = true;
-    Hashtable info = null;
+    Map<String, Object> info = null;
     int nBiomt = 0;
     Matrix4f mIdent = new Matrix4f();
     mIdent.setIdentity();
@@ -412,11 +427,11 @@ REMARK 350   BIOMT3   3  0.000000  0.000000  1.000000        0.00000
           if (nBiomt > 0)
             Logger.info("biomolecule " + iMolecule + ": number of transforms: "
                 + nBiomt);
-          info = new Hashtable();
-          biomts = new ArrayList();
+          info = new Hashtable<String, Object>();
+          biomts = new ArrayList<Matrix4f>();
           iMolecule = parseInt(line.substring(line.indexOf(":") + 1));
           title = line.trim();
-          info.put("molecule", new Integer(iMolecule));
+          info.put("molecule", Integer.valueOf(iMolecule));
           info.put("title", title);
           info.put("chains", "");
           info.put("biomts", biomts);
@@ -441,8 +456,6 @@ REMARK 350   BIOMT3   3  0.000000  0.000000  1.000000        0.00000
             Logger.info("filter set to \"" + filter + "\"");
             this.biomts = biomts;
           }
-          if (info == null)
-            return; //bad file format
           info.put("chains", chainlist);
           continue;
         }
@@ -542,10 +555,11 @@ REMARK 290 REMARK: NULL
     atom.chainID = ch;
     atom.sequenceNumber = parseInt(line, 22, 26);
     atom.insertionCode = JmolAdapter.canonizeInsertionCode(line.charAt(26));
-
+    atom.isHetero = line.startsWith("HETATM");
+    atom.elementSymbol = deduceElementSymbol(atom.isHetero);
     if (!filterAtom(atom, iAtom++))
       return;
-
+    
     atom.atomSerial = serial;
     if (serial > maxSerial)
       maxSerial = serial;
@@ -558,14 +572,11 @@ REMARK 290 REMARK: NULL
     } else if (!atom.group3.equals(currentGroup3) || atom.sequenceNumber != currentResno) {
       currentGroup3 = atom.group3;
       currentResno = atom.sequenceNumber;
-      htElementsInCurrentGroup = (Hashtable) htFormul.get(atom.group3);
+      htElementsInCurrentGroup = htFormul.get(atom.group3);
       nRes++;
       if (atom.group3.equals("UNK"))
         nUNK++;
     }
-    boolean isHetero = line.startsWith("HETATM");
-    atom.isHetero = isHetero;
-    atom.elementSymbol = deduceElementSymbol(isHetero);
     //calculate the charge from cols 79 & 80 (1-based): 2+, 3-, etc
     int charge = 0;
     if (lineLength >= 80) {
@@ -584,17 +595,12 @@ REMARK 290 REMARK: NULL
       }
     }
     atom.formalCharge = charge;
-    float partialCharge = readPartialCharge();
-    if (partialCharge != Float.MAX_VALUE)
-      atom.partialCharge = partialCharge;
 
     setAtomCoord(atom, parseFloat(line, 30, 38), parseFloat(line, 38, 46),
         parseFloat(line, 46, 54));
 
-    atom.radius = readRadius();
-    atom.bfactor = readBFactor();
-    atom.occupancy = readOccupancy();
-    
+    setAdditionalAtomParameters(atom);
+
     lastAtomData = line.substring(6, 26);
     lastAtomIndex = atomSetCollection.getAtomCount();
     if (haveMappedSerials)
@@ -604,7 +610,7 @@ REMARK 290 REMARK: NULL
     if (atomCount++ == 0)
       atomSetCollection.setAtomSetAuxiliaryInfo("isPDB", Boolean.TRUE);
     // note that values are +1 in this serial map
-    if (isHetero) {
+    if (atom.isHetero) {
       if (htHetero != null) {
         atomSetCollection.setAtomSetAuxiliaryInfo("hetNames", htHetero);
         htHetero = null;
@@ -616,8 +622,9 @@ REMARK 290 REMARK: NULL
   private char lastInsertion;
   private char lastAltLoc;
   private int conformationIndex;
-  StringBuffer sbIgnored, sbSelected;
+  private StringBuffer sbIgnored, sbSelected;
 
+  @Override
   protected boolean filterAtom(Atom atom, int iAtom) {
     if (!super.filterAtom(atom, iAtom))
       return false;
@@ -650,35 +657,50 @@ REMARK 290 REMARK: NULL
     }
     return true;
   }
-  
-  protected int readOccupancy() {
+
+  /**
+   * adaptable via subclassing
+   * 
+   * @param atom
+   */
+  protected void setAdditionalAtomParameters(Atom atom) {
 
     /****************************************************************
      * read the occupancy from cols 55-60 (1-based)
      * should be in the range 0.00 - 1.00
      ****************************************************************/
-    int occupancy = 100;
     float floatOccupancy = parseFloat(line, 54, 60);
-    if (!Float.isNaN(floatOccupancy))
-      occupancy = (int) (floatOccupancy * 100);
-    return occupancy;
-  }
-  
-  protected float readBFactor() {
+    atom.occupancy = (Float.isNaN(floatOccupancy) ? 100 : (int) (floatOccupancy * 100));
+
     /****************************************************************
      * read the bfactor from cols 61-66 (1-based)
      ****************************************************************/
-    return parseFloat(line, 60, 66);
+    atom.bfactor = parseFloat(line, 60, 66);
+
   }
-  
-  protected float readPartialCharge() {
-    return Float.MAX_VALUE; 
-  }
-  
-  protected float readRadius() {
-    return Float.NaN; 
-  }
-  
+
+  /**
+   * The problem here stems from the fact that developers have not fully 
+   * understood the PDB specifications -- and that those have changed. 
+   * The actual rules are as follows (using 1-based numbering:
+   * 
+   * 1) Chemical symbols may be in columns 77 and 78 for total disambiguity.
+   * 2) Only valid chemical symbols should be in columns 13 and 14
+   *    These are the first two characters of a four-character field.
+   * 3) Four-character atom names for hydrogen necessarily start in 
+   *    column 13, so when that is the case, if the four-letter 
+   *    name starts with "H" then it is hydrogen regardless of what
+   *    letter comes next. For example, "HG3 " is mercury (and should
+   *    be in a HETATM record, not an ATOM record, anyway), but "HG33"
+   *    is hydrogen, presumably.
+   *    
+   *    This leave open the ambiguity of a four-letter H name in a 
+   *    heteroatom set where the symbol is really H, not Hg or Ha, or Ho or Hf, etc.
+   *     
+   * 
+   * @param isHetero
+   * @return           an atom symbol
+   */
   private String deduceElementSymbol(boolean isHetero) {
     if (lineLength >= 78) {
       char ch76 = line.charAt(76);
@@ -690,26 +712,50 @@ REMARK 290 REMARK: NULL
     }
     char ch12 = line.charAt(12);
     char ch13 = line.charAt(13);
+    // PDB atom symbols are supposed to be in these two characters
+    // But they could be right-aligned or left-aligned
     if ((htElementsInCurrentGroup == null ||
          htElementsInCurrentGroup.get(line.substring(12, 14)) != null) &&
         Atom.isValidElementSymbolNoCaseSecondChar(ch12, ch13))
       return (isHetero || ch12 != 'H' ? "" + ch12 + ch13 : "H");
+    // not a known two-letter code
+    if (ch12 == 'H') // added check for PQR files "HD22" for example
+      return "H";
+    // check for " NZ" for example
     if ((htElementsInCurrentGroup == null ||
          htElementsInCurrentGroup.get("" + ch13) != null) &&
         Atom.isValidElementSymbol(ch13))
       return "" + ch13;
-    if ((htElementsInCurrentGroup == null ||
+    // check for misplaced "O   " for example
+    if (ch12 != ' ' && (htElementsInCurrentGroup == null ||
          htElementsInCurrentGroup.get("" + ch12) != null) &&
         Atom.isValidElementSymbol(ch12))
       return "" + ch12;
+    // could be GLX or ASX;
+    // probably a bad file. But we will make ONE MORE ATTEMPT
+    // and read columns 14/15 instead of 12/13. What the heck!
+    char ch14 = line.charAt(14);
+    if (ch12 == ' ' && ch13 != 'X' && (htElementsInCurrentGroup == null ||
+        htElementsInCurrentGroup.get(line.substring(13, 15)) != null) &&
+        Atom.isValidElementSymbolNoCaseSecondChar(ch13, ch14))
+     return  "" + ch13 + ch14;
     return "Xx";
   }
 
-  private StringBuffer sbConect;
+  private StringBuffer sbConect, sb;
+  private int lastSourceSerial = Integer.MIN_VALUE;
+  private int lastTargetSerial = Integer.MIN_VALUE;
+  
+  private boolean haveDoubleBonds;
+  
   private void conect() {
     // adapted for improper non-crossreferenced files such as 1W7R
-    if (sbConect == null)
+    if (sbConect == null) {
       sbConect = new StringBuffer();
+      sb = new StringBuffer();
+    } else {
+      sb.setLength(0);
+    }
     int sourceSerial = -1;
     sourceSerial = parseInt(line, 6, 11);
     if (sourceSerial < 0)
@@ -721,8 +767,13 @@ REMARK 290 REMARK: NULL
           offsetEnd) : -1);
       if (targetSerial < 0)
         continue;
-      int i1;
+      boolean isDoubleBond = (sourceSerial == lastSourceSerial && targetSerial == lastTargetSerial);
+      if (isDoubleBond)
+        haveDoubleBonds = true;
+      lastSourceSerial = sourceSerial;
+      lastTargetSerial = targetSerial;
       boolean isSwapped = (targetSerial < sourceSerial);
+      int i1;
       if (isSwapped) {
         i1 = targetSerial;
         targetSerial = sourceSerial;
@@ -730,35 +781,46 @@ REMARK 290 REMARK: NULL
         i1 = sourceSerial;
       }
       String st = ";" + i1 + " " + targetSerial + ";";
-      if (sbConect.indexOf(st) >= 0)
+      if (sbConect.indexOf(st) >= 0 && !isDoubleBond)
         continue;
-      sbConect.append(st);
+      // check for previous double
+      if (haveDoubleBonds) {
+        String st1 = "--" + st;
+        if (sbConect.indexOf(st1) >= 0)
+          continue;
+        sbConect.append(st);
+        sb.append(st1);
+      } else {
+        sbConect.append(st);
+      }
       atomSetCollection.addConnection(new int[] { i1, targetSerial,
           i < 4 ? 1 : JmolAdapter.ORDER_HBOND });
     }
+    sbConect.append(sb);
   }
 
   /*
           1         2         3
-0123456789012345678901234567890123456
-HELIX    1  H1 ILE      7  LEU     18
-HELIX    2  H2 PRO     19  PRO     19
-HELIX    3  H3 GLU     23  TYR     29
-HELIX    4  H4 THR     30  THR     30
-SHEET    1  S1 2 THR     2  CYS     4
-SHEET    2  S2 2 CYS    32  ILE    35
-SHEET    3  S3 2 THR    39  PRO    41
-TURN     1  T1 GLY    42  TYR    44
+  0123456789012345678901234567890123456
+  HELIX    1  H1 ILE      7  LEU     18
+  HELIX    2  H2 PRO     19  PRO     19
+  HELIX    3  H3 GLU     23  TYR     29
+  HELIX    4  H4 THR     30  THR     30
+  SHEET    1  S1 2 THR     2  CYS     4
+  SHEET    2  S2 2 CYS    32  ILE    35
+  SHEET    3  S3 2 THR    39  PRO    41
+  TURN     1  T1 GLY    42  TYR    44
 
-HELIX     1 H1 ILE A    7  PRO A   19
-HELIX     2 H2 GLU A   23  THR A   30
-SHEET     1 S1 0 CYS A   3  CYS A   4
-SHEET     2 S2 0 CYS A  32  ILE A  35
+  HELIX     1 H1 ILE A    7  PRO A   19
+  HELIX     2 H2 GLU A   23  THR A   30
+  SHEET     1 S1 0 CYS A   3  CYS A   4
+  SHEET     2 S2 0 CYS A  32  ILE A  35
+
+  HELIX  113 113 ASN H  307  ARG H  327  1                                  21    
+  SHEET    1   A 6 ASP A  77  HIS A  80  0                                        
+  SHEET    2   A 6 GLU A  47  ILE A  51  1  N  ILE A  48   O  ASP A  77           
+  SHEET    3   A 6 ARG A  22  ILE A  26  1  N  VAL A  23   O  GLU A  47           
 
-HELIX  113 113 ASN H  307  ARG H  327  1                                  21    
-SHEET    1   A 6 ASP A  77  HIS A  80  0                                        
-SHEET    2   A 6 GLU A  47  ILE A  51  1  N  ILE A  48   O  ASP A  77           
-SHEET    3   A 6 ARG A  22  ILE A  26  1  N  VAL A  23   O  GLU A  47           
 
 TYPE OF HELIX CLASS NUMBER (COLUMNS 39 - 40)
 --------------------------------------------------------------
@@ -776,30 +838,30 @@ Polyproline 10
    */
   
   private void structure() {
-    int structureType = 0;
-    int substructureType = 0;
+    EnumStructure structureType = EnumStructure.NONE;
+    EnumStructure substructureType = EnumStructure.NONE;
     int startChainIDIndex;
     int startIndex;
     int endChainIDIndex;
     int endIndex;
     int strandCount = 0;
     if (line.startsWith("HELIX ")) {
-      structureType = Structure.PROTEIN_STRUCTURE_HELIX;
+      structureType = EnumStructure.HELIX;
       startChainIDIndex = 19;
       startIndex = 21;
       endChainIDIndex = 31;
       endIndex = 33;
       if (line.length() >= 40)
-        substructureType = Structure.getHelixType(parseInt(line.substring(38, 40)));
+      substructureType = Structure.getHelixType(parseInt(line.substring(38, 40)));
     } else if (line.startsWith("SHEET ")) {
-      structureType = Structure.PROTEIN_STRUCTURE_SHEET;
+      structureType = EnumStructure.SHEET;
       startChainIDIndex = 21;
       startIndex = 22;
       endChainIDIndex = 32;
       endIndex = 33;
       strandCount = parseInt(line.substring(14, 16));
     } else if (line.startsWith("TURN  ")) {
-      structureType = Structure.PROTEIN_STRUCTURE_TURN;
+      structureType = EnumStructure.TURN;
       startChainIDIndex = 19;
       startIndex = 20;
       endChainIDIndex = 30;
@@ -825,7 +887,7 @@ Polyproline 10
     // this should probably call Structure.validateAndAllocate
     // in order to check validity of parameters
     // model number set to -1 here to indicate ALL MODELS
-    if (substructureType == 0)
+    if (substructureType == EnumStructure.NONE)
       substructureType = structureType;
     Structure structure = new Structure(-1, structureType, substructureType,
         structureID, serialID, strandCount, startChainID, startSequenceNumber,
@@ -858,16 +920,21 @@ Polyproline 10
      * number right after the word MODEL :-(
      ****************************************************************/
     checkNotPDB();
-    sbConect = null;
     haveMappedSerials = false;
+    sbConect = null;
     atomSetCollection.newAtomSet();
     atomSetCollection.setAtomSetAuxiliaryInfo("isPDB", Boolean.TRUE);
     atomSetCollection.setAtomSetNumber(modelNumber);
   }
 
   private void checkNotPDB() {
-    if (nRes > 0 && nUNK == nRes)
-      atomSetCollection.setAtomSetAuxiliaryInfo("isPDB", Boolean.FALSE);
+    boolean isPDB = (nRes == 0 || nUNK != nRes);
+    // This speeds up calculation, because no crosschecking
+    // No special-position atoms in mmCIF files, because there will
+    // be no center of symmetry, no rotation-inversions, 
+    // no atom-centered rotation axes, and no mirror or glide planes. 
+    atomSetCollection.setCheckSpecial(!isPDB);
+    atomSetCollection.setAtomSetAuxiliaryInfo("isPDB", isPDB ? Boolean.TRUE : Boolean.FALSE);
     nUNK = nRes = 0;
     currentGroup3 = null;
   }
@@ -909,9 +976,9 @@ Polyproline 10
         return; // invalid formula;
       formula = parseTrimmed(formula, ichLeftParen + 1, ichRightParen);
     }
-    Hashtable htElementsInGroup = (Hashtable)htFormul.get(groupName);
+    Map<String, Boolean> htElementsInGroup = htFormul.get(groupName);
     if (htElementsInGroup == null)
-      htFormul.put(groupName, htElementsInGroup = new Hashtable());
+      htFormul.put(groupName, htElementsInGroup = new Hashtable<String, Boolean>());
     // now, look for atom names in the formula
     next[0] = 0;
     String elementWithCount;
@@ -928,29 +995,34 @@ Polyproline 10
   }
   
   private void het() {
-    if (line.length() < 30)
+    if (line.length() < 30) {
       return;
-    if (htHetero == null)
-      htHetero = new Hashtable();
+    }
+    if (htHetero == null) {
+      htHetero = new Hashtable<String, String>();
+    }
     String groupName = parseToken(line, 7, 10);
-    if (htHetero.containsKey(groupName))
+    if (htHetero.containsKey(groupName)) {
       return;
+    }
     String hetName = parseTrimmed(line, 30, 70);
     htHetero.put(groupName, hetName);
   }
   
   private void hetnam() {
-    if (htHetero == null)
-      htHetero = new Hashtable();
+    if (htHetero == null) {
+      htHetero = new Hashtable<String, String>();
+    }
     String groupName = parseToken(line, 11, 14);
     String hetName = parseTrimmed(line, 15, 70);
     if (groupName == null) {
       Logger.error("ERROR: HETNAM record does not contain a group name: " + line);
       return;
     }
-    String htName = (String) htHetero.get(groupName);
-    if (htName != null)
+    String htName = htHetero.get(groupName);
+    if (htName != null) {
       hetName = htName + hetName;
+    }
     htHetero.put(groupName, hetName);
     //Logger.debug("hetero: "+groupName+" "+hetName);
   }
@@ -1067,16 +1139,17 @@ COLUMNS       DATA TYPE         FIELD            DEFINITION
    */
   
   private void site() {
-    if (htSites == null)
-      htSites = new Hashtable();
+    if (htSites == null) {
+      htSites = new Hashtable<String, Map<String, Object>>();
+    }
     //int seqNum = parseInt(line, 7, 10);
     int nResidues = parseInt(line, 15, 17);
     String siteID = parseTrimmed(line, 11, 14);
-    Hashtable htSite = (Hashtable) htSites.get(siteID);
+    Map<String, Object> htSite = htSites.get(siteID);
     if (htSite == null) {
-      htSite = new Hashtable();
+      htSite = new Hashtable<String, Object>();
       //htSite.put("seqNum", "site_" + seqNum);
-      htSite.put("nResidues", new Integer(nResidues));
+      htSite.put("nResidues", Integer.valueOf(nResidues));
       htSite.put("groups", "");
       htSites.put(siteID, htSite);
     }
