@@ -27,14 +27,17 @@ package org.jmol.adapter.readers.quantum;
 import org.jmol.adapter.smarter.*;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import javax.vecmath.Vector3f;
 
 import org.jmol.api.JmolAdapter;
+import org.jmol.util.ArrayUtil;
 import org.jmol.util.Logger;
+import org.jmol.util.Parser;
 
 /**
  * Reader for Gaussian 94/98/03/09 output files.
@@ -99,6 +102,7 @@ public class GaussianReader extends MOReader {
    * 
    **/
 
+  @Override
   protected boolean checkLine() throws Exception {
     if (line.startsWith(" Step number")) {
       equivalentAtomSets = 0;
@@ -168,13 +172,12 @@ public class GaussianReader extends MOReader {
       readBasis();
       return true;
     }
-    if (line.indexOf("Molecular Orbital Coefficients") >= 0) {
+    if (line.indexOf("Molecular Orbital Coefficients") >= 0 
+        || line.indexOf("Natural Orbital Coefficients") >= 0) {
       if (!filterMO())
         return true;
       readMolecularOrbitals();
-      if (Logger.debugging) {
-        Logger.debug(orbitals.size() + " molecular orbitals read");
-      }
+      Logger.info(orbitals.size() + " molecular orbitals read");
       return true;
     }
     if (line.startsWith(" Normal termination of Gaussian")) {
@@ -274,7 +277,7 @@ public class GaussianReader extends MOReader {
     atomSetCollection.setAtomSetEnergy(energyString, parseFloat(energyString));
 //  atomSetCollection.setAtomSetName("Last read atomset.");
     String path = getTokens()[0]; // path = type of orientation
-    discardLines(4);
+    readLines(4);
     String tokens[];
     while (readLine() != null &&
         !line.startsWith(" --")) {
@@ -285,11 +288,11 @@ public class GaussianReader extends MOReader {
       if (atom.elementNumber < 0)
         atom.elementNumber = 0; // dummy atoms have -1 -> 0
       int offset = tokens.length-3;
-      atom.x = parseFloat(tokens[offset]);
-      atom.y = parseFloat(tokens[++offset]);
-      atom.z = parseFloat(tokens[++offset]);
+      setAtomCoord(atom, parseFloat(tokens[offset]), 
+          parseFloat(tokens[++offset]), 
+          parseFloat(tokens[++offset]));
     }
-    atomSetCollection.setAtomSetProperty(SmarterJmolAdapter.PATH_KEY,
+    atomSetCollection.setAtomSetModelProperty(SmarterJmolAdapter.PATH_KEY,
         "Calculation "+calculationNumber+
         (scanPoint>=0?(SmarterJmolAdapter.PATH_SEPARATOR+"Scan Point "+scanPoint):"")+
         SmarterJmolAdapter.PATH_SEPARATOR+path);
@@ -319,8 +322,8 @@ public class GaussianReader extends MOReader {
    */
 
   private void readBasis() throws Exception {
-    shells = new ArrayList();
-    List gdata = new ArrayList();
+    shells = new ArrayList<int[]>();
+    List<String[]> gdata = new ArrayList<String[]>();
     int atomCount = 0;
     gaussianCount = 0;
     shellCount = 0;
@@ -348,22 +351,21 @@ public class GaussianReader extends MOReader {
       slater[3] = nGaussians;
       shells.add(slater);
       gaussianCount += nGaussians;
-      for (int i = 0; i < nGaussians; i++)
+      for (int i = 0; i < nGaussians; i++) {
         gdata.add(getTokens(readLine()));
+      }
     }
     if (atomCount == 0)
       atomCount = 1;
     gaussians = new float[gaussianCount][];
     for (int i = 0; i < gaussianCount; i++) {
-      tokens = (String[]) gdata.get(i);
+      tokens = gdata.get(i);
       gaussians[i] = new float[tokens.length];
       for (int j = 0; j < tokens.length; j++)
         gaussians[i][j] = parseFloat(tokens[j]);
     }
-    if (Logger.debugging) {
-      Logger.debug(shellCount + " slater shells read");
-      Logger.debug(gaussianCount + " gaussian primitives read");
-    }
+    Logger.info(shellCount + " slater shells read");
+    Logger.info(gaussianCount + " gaussian primitives read");
   }
   
   /*
@@ -379,34 +381,57 @@ public class GaussianReader extends MOReader {
    5        2PZ        -0.00134  -0.09475   0.00000   0.55774   0.00000
    6        3S          0.00415   0.43535   0.00000   0.32546   0.00000
 
-but:
+  but:
+
+  105        4S        -47.27845  63.29565-100.44035   1.98362 -51.35328
+
+  also G09  # B3LYP 6-31g sp gfprint pop(full,NO) 
+
+     Natural Orbital Coefficients:
+                           1         2         3         4         5
+     Eigenvalues --     2.00000   2.00000   2.00000   2.00000   2.00000
+   1 1   C  1S          0.03807   0.23941   0.96961   0.01811  -0.04011
+   2        2S         -0.00048   0.00095   0.01728   0.01316  -0.02849
 
- 105        4S        -47.27845  63.29565-100.44035   1.98362 -51.35328
 
    */
   private void readMolecularOrbitals() throws Exception {
     if (shells == null)
       return;
-    Hashtable[] mos = new Hashtable[5];
-    List[] data = new ArrayList[5];
+    Map<String, Object>[] mos = ArrayUtil.createArrayOfHashtable(5);
+    List<String>[] data = ArrayUtil.createArrayOfArrayList(5);
     int nThisLine = 0;
-    while (readLine() != null
-        && line.toUpperCase().indexOf("DENS") < 0) {
+    boolean isNOtype = line.contains("Natural"); //gfprint pop(full,NO)
+    while (readLine() != null && line.toUpperCase().indexOf("DENS") < 0) {
       String[] tokens;
       if (line.indexOf("                    ") == 0) {
         addMOData(nThisLine, data, mos);
-        tokens = getTokens(readLine());
-        nThisLine = tokens.length;
-        for (int i = 0; i < nThisLine; i++) {
-          mos[i] = new Hashtable();
-          data[i] = new ArrayList();
-          String sym = tokens[i];
-          mos[i].put("symmetry", sym);
-          if (sym.indexOf("--O") >= 0)
-            mos[i].put("occupancy", new Float(2));
-          else if (sym.indexOf("--V") >= 0)
-            mos[i].put("occupancy", new Float(0));
+        if (isNOtype) {
+          tokens = getTokens(line);
+          nThisLine = tokens.length;
+          tokens = getTokens(readLine());
+        } else {
+          tokens = getTokens(readLine());
+          nThisLine = tokens.length;
         }
+        for (int i = 0; i < nThisLine; i++) {
+          mos[i] = new Hashtable<String, Object>();
+          data[i] = new ArrayList<String>();
+          String sym;
+          if (isNOtype) {
+            mos[i]
+                .put("occupancy", new Float(Parser.parseFloat(tokens[i + 2])));
+          } else {
+            sym = tokens[i];
+            mos[i].put("symmetry", sym);
+            if (sym.indexOf("O") >= 0)
+              mos[i].put("occupancy", new Float(2));
+            else if (sym.indexOf("V") >= 0)
+              mos[i].put("occupancy", new Float(0));
+          }
+        }
+        if (isNOtype)
+          continue;
         line = readLine().substring(21);
         tokens = getTokens();
         if (tokens.length != nThisLine)
@@ -414,18 +439,20 @@ but:
         for (int i = 0; i < nThisLine; i++)
           mos[i].put("energy", new Float(tokens[i]));
         continue;
-      } else if (line.length() < 21 || (line.charAt(11) != ' ' 
-                                        && ! Character.isDigit(line.charAt(11)) 
-                                        ) ) {
+      } else if (line.length() < 21
+          || (line.charAt(5) != ' ' && !Character.isDigit(line.charAt(5)))) {
         continue;
       }
       try {
-        String type = line.substring(13, 18).trim();
-        if (!isQuantumBasisSupported(type.charAt(0)) && "XYZ".indexOf(type.charAt(0)) >= 0)
+        tokens = getTokens();
+        String type = tokens[tokens.length - nThisLine - 1].substring(1);
+        if (!isQuantumBasisSupported(type.charAt(0))
+            && "XYZ".indexOf(type.charAt(0)) >= 0)
           type = (type.length() == 2 ? "D" : "F") + type;
         if (!isQuantumBasisSupported(type.charAt(0)))
           continue;
-        tokens = getStrings(line.substring(21), nThisLine, 10);
+        tokens = getStrings(line.substring(line.length() - 10 * nThisLine),
+            nThisLine, 10);
         for (int i = 0; i < nThisLine; i++)
           data[i].add(tokens[i]);
       } catch (Exception e) {
@@ -513,11 +540,11 @@ but:
         atomSetCollection.cloneLastAtomSet();
         // set the properties
         atomSetCollection.setAtomSetFrequency("Calculation " + calculationNumber, symmetries[i], frequencies[i], null);
-        atomSetCollection.setAtomSetProperty("ReducedMass",
+        atomSetCollection.setAtomSetModelProperty("ReducedMass",
             red_masses[i]+" AMU");
-        atomSetCollection.setAtomSetProperty("ForceConstant",
+        atomSetCollection.setAtomSetModelProperty("ForceConstant",
             frc_consts[i]+" mDyne/A");
-        atomSetCollection.setAtomSetProperty("IRIntensity",
+        atomSetCollection.setAtomSetModelProperty("IRIntensity",
             intensities[i]+" KM/Mole");
       }
       discardLinesUntilContains(" AN ");
@@ -556,7 +583,7 @@ but:
   // TODO this really should set the charges for the last nOrientations read
   // being careful about the dummy atoms...
   void readPartialCharges() throws Exception {
-    discardLines(1);
+    readLine();
     int atomCount = atomSetCollection.getAtomCount();
     int i0 = atomSetCollection.getLastAtomSetAtomIndex();
     Atom[] atoms = atomSetCollection.getAtoms();
