@@ -24,14 +24,17 @@
 
 package org.jmol.adapter.readers.quantum;
 
+
 import org.jmol.adapter.smarter.*;
 import org.jmol.api.JmolAdapter;
+import org.jmol.util.ArrayUtil;
 import org.jmol.util.Logger;
 
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 /**
  * A reader for Q-Chem 2.1 and 3.2
@@ -70,8 +73,10 @@ public class QchemReader extends MOReader {
   private MOInfo[] betas = null;
   private int nBasis = 0;          // # of basis according to qchem
 
+  
+  @Override
   protected void initializeReader() {
-    energyUnits = "au";    
+    energyUnits = "au";   
   }
   
   /**
@@ -79,6 +84,7 @@ public class QchemReader extends MOReader {
    * @throws Exception
    * 
    */
+  @Override
   protected boolean checkLine() throws Exception {
     if (line.indexOf("Standard Nuclear Orientation") >= 0) {
       readAtoms();
@@ -142,7 +148,7 @@ public class QchemReader extends MOReader {
   private void readAtoms() throws Exception {
     atomSetCollection.newAtomSet();
     setMOData(true);
-    discardLines(2);
+    readLines(2);
     String[] tokens;
     while (readLine() != null && !line.startsWith(" --")) {
       tokens = getTokens();
@@ -159,8 +165,8 @@ public class QchemReader extends MOReader {
         continue;
       Atom atom = atomSetCollection.addNewAtom();
       atom.elementSymbol = symbol;
-      atom.set(x, y, z);
-      atomSetCollection.setAtomSetProperty(SmarterJmolAdapter.PATH_KEY,
+      setAtomCoord(atom, x, y, z);
+      atomSetCollection.setAtomSetModelProperty(SmarterJmolAdapter.PATH_KEY,
           "Calculation "+calculationNumber);
    }
   }
@@ -204,7 +210,7 @@ public class QchemReader extends MOReader {
   }
 
   private void readPartialCharges() throws Exception {
-    discardLines(3);
+    readLines(3);
     Atom[] atoms = atomSetCollection.getAtoms();
     int atomCount = atomSetCollection.getLastAtomSetAtomCount();
     for (int i = 0; i < atomCount && readLine() != null; ++i)
@@ -263,13 +269,13 @@ $end
 
   private void readBasis() throws Exception {
     // initialize the 'global' variables
-    moData = new Hashtable();
+    moData = new Hashtable<String, Object>();
     int atomCount = 0;
     int shellCount = 0;
     int gaussianCount = 0;
     // local variables
-    shells = new ArrayList();
-    List gdata = new ArrayList();
+    shells = new ArrayList<int[]>();
+    List<String[]> gdata = new ArrayList<String[]>();
     String[] tokens;
 
     discardLinesUntilStartsWith("$basis");
@@ -290,13 +296,14 @@ $end
       slater[3] = nGaussians;
       shells.add(slater);
       gaussianCount += nGaussians;
-      for (int i = 0; i < nGaussians; i++)
-        gdata.add(getTokens(readLine()));     
+      for (int i = 0; i < nGaussians; i++) {
+        gdata.add(getTokens(readLine()));
+      }
     }
     // now rearrange the gaussians (direct copy from GaussianReader)
     gaussians = new float[gaussianCount][];
     for (int i = 0; i < gaussianCount; i++) {
-      tokens = (String[]) gdata.get(i);
+      tokens = gdata.get(i);
       gaussians[i] = new float[tokens.length];
       for (int j = 0; j < tokens.length; j++)
         gaussians[i][j] = parseFloat(tokens[j]);
@@ -443,6 +450,7 @@ $end
     MOInfo[] moInfos;
     int ne = 0; // number of electrons for a particular series of orbitals
     boolean readBetas = false;
+
     discardLinesUntilStartsWith(" Alpha");
     String[] tokens = getTokens(line); // initialize tokens for later as well
     moInfos = alphas;
@@ -554,7 +562,6 @@ $end
   private static String FC_LIST = CANONICAL_FC_LIST;
   private static String FS_LIST = "F4    F5    F3    F6    F2    F7    F1";
   
- 
   private void readQchemMolecularOrbitals() throws Exception {
 
     /* 
@@ -579,53 +586,49 @@ $end
     }
     boolean isOK = true;
     if (dList.length() > 0) {
-      if (dSpherical)
+      if (dSpherical) 
         isOK = getDFMap(dList, JmolAdapter.SHELL_D_SPHERICAL, DS_LIST, 2);
       else
         isOK = getDFMap(dList, JmolAdapter.SHELL_D_CARTESIAN, DC_LIST, 3);
       if (!isOK) {
-        Logger
-            .error("atomic orbital order is unrecognized -- skipping reading of MOs. dList="
-                + dList);
+        Logger.error("atomic orbital order is unrecognized -- skipping reading of MOs. dList=" + dList);
         shells = null;
       }
     }
     if (fList.length() > 0) {
-      if (fSpherical)
+      if (fSpherical) 
         isOK = getDFMap(fList, JmolAdapter.SHELL_F_SPHERICAL, FS_LIST, 2);
       else
         isOK = getDFMap(fList, JmolAdapter.SHELL_F_CARTESIAN, FC_LIST, 3);
       if (!isOK) {
-        Logger
-            .error("atomic orbital order is unrecognized -- skipping reading of MOs. fList="
-                + fList);
+        Logger.error("atomic orbital order is unrecognized -- skipping reading of MOs. fList=" + fList);
         shells = null;
       }
     }
     setMOData(shells == null);
     shells = null; // clears mo data upon next model loading
   }
-  
+
   String dList = "";
   String fList = "";
   boolean dSpherical = false;
   boolean fSpherical = false;
   
   private int readMOs(boolean restricted, MOInfo[] moInfos) throws Exception {
-    Hashtable[] mos = new Hashtable[6];  // max 6 MO's per line
-    float[][] mocoef = new float[6][];   // coefficients for each MO
-    int[] moid = new int[6];             // mo numbers
+    Map<String, Object>[] mos = ArrayUtil.createArrayOfHashtable(6); // max 6 MO's per line
+    float[][] mocoef = new float[6][]; // coefficients for each MO
+    int[] moid = new int[6]; // mo numbers
     String[] tokens, energy;
     int nMOs = 0;
-    
+
     while (readLine().length() > 2) {
       tokens = getTokens(line);
-      int nMO = tokens.length;    // number of MO columns
+      int nMO = tokens.length; // number of MO columns
       energy = getTokens(readLine().substring(13));
       for (int i = 0; i < nMO; i++) {
-        moid[i] = parseInt(tokens[i])-1;
+        moid[i] = parseInt(tokens[i]) - 1;
         mocoef[i] = new float[nBasis];
-        mos[i] = new Hashtable();
+        mos[i] = new Hashtable<String, Object>();
       }
       for (int i = 0, pt = 0; i < nBasis; i++) {
         tokens = getTokens(readLine());
@@ -635,10 +638,11 @@ $end
         case 'd':
           s = s.substring(s.length() - 3).toUpperCase();
           if (s.startsWith("D ")) {
-             if (!dFixed)
-               fixSlaterTypes(JmolAdapter.SHELL_D_CARTESIAN, JmolAdapter.SHELL_D_SPHERICAL);
-             s = "D" + s.charAt(2);
-             dSpherical = true;
+            if (!dFixed)
+              fixSlaterTypes(JmolAdapter.SHELL_D_CARTESIAN,
+                  JmolAdapter.SHELL_D_SPHERICAL);
+            s = "D" + s.charAt(2);
+            dSpherical = true;
           }
           if (dList.indexOf(s) < 0)
             dList += s + " ";
@@ -647,10 +651,11 @@ $end
         case 'f':
           s = s.substring(s.length() - 3).toUpperCase();
           if (s.startsWith("F ")) {
-             if (!fFixed)
-               fixSlaterTypes(JmolAdapter.SHELL_F_CARTESIAN, JmolAdapter.SHELL_F_SPHERICAL);
-             s = "F" + s.charAt(2);
-             fSpherical = true;
+            if (!fFixed)
+              fixSlaterTypes(JmolAdapter.SHELL_F_CARTESIAN,
+                  JmolAdapter.SHELL_F_SPHERICAL);
+            s = "F" + s.charAt(2);
+            fSpherical = true;
           }
           if (fList.indexOf(s) < 0)
             fList += s + " ";
@@ -661,18 +666,19 @@ $end
             continue;
           break;
         }
-        for (int j = tokens.length-nMO, k=0; k < nMO; j++, k++)
+        for (int j = tokens.length - nMO, k = 0; k < nMO; j++, k++)
           mocoef[k][pt] = parseFloat(tokens[j]);
         pt++;
       }
       // we have all the info we need 
-      for (int i = 0; i < nMO; i++ ) {
+      for (int i = 0; i < nMO; i++) {
         MOInfo moInfo = moInfos[moid[i]];
         mos[i].put("energy", new Float(energy[i]));
-        mos[i].put("coefficients",mocoef[i]);
+        mos[i].put("coefficients", mocoef[i]);
         String label = alphaBeta;
         int ne = moInfo.ne;
-        if (restricted) ne = alphas[moid[i]].ne + betas[moid[i]].ne;
+        if (restricted)
+          ne = alphas[moid[i]].ne + betas[moid[i]].ne;
         mos[i].put("occupancy", new Float(ne));
         switch (ne) {
         case 2:
