@@ -43,6 +43,8 @@ import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
 
 /**
+ * 
+ * A reader of OUT and OUTP files for CRYSTAL
  * 
  * http://www.crystal.unito.it/
  * 
@@ -76,12 +78,12 @@ import javax.vecmath.Vector3f;
  *          to load just the input deck exactly as indicated, use
  * 
  *          load "xxx.out" FILTER "input"
- *          
- *    now allows reading of frequencies and atomic values with conventional
- *    as long as this is not an optimization. 
- *    
- *  
- *          
+ * 
+ *          now allows reading of frequencies and atomic values with
+ *          conventional as long as this is not an optimization.
+ * 
+ * 
+ * 
  * 
  */
 
@@ -98,29 +100,34 @@ public class CrystalReader extends AtomSetCollectionReader {
   private boolean isLongMode;
   private boolean getLastConventional;
   private boolean havePrimitiveMapping;
-  
+  private boolean isProperties;
+
   private int atomCount;
   private int atomIndexLast;
   private int[] atomFrag;
   private int[] primitiveToIndex;
   private float[] nuclearCharges;
-  private List vInputCoords;
+  private List<String> vInputCoords;
 
   private Double energy;
   private Point3f ptOriginShift = new Point3f();
   private Matrix3f primitiveToCryst;
   private Vector3f[] directLatticeVectors;
-  
+  private String spaceGroupName;
+
+  @Override
   protected void initializeReader() throws Exception {
     doProcessLines = false;
     inputOnly = checkFilter("INPUT");
-    addVibrations &= !inputOnly;
     isPrimitive = !inputOnly && !checkFilter("CONV");
+    addVibrations &= !inputOnly && isPrimitive; //
     getLastConventional = (!isPrimitive && desiredModelNumber == 0);
     setFractionalCoordinates(readHeader());
   }
 
+  @Override
   protected boolean checkLine() throws Exception {
+
     if (line.startsWith(" LATTICE PARAMETER")) {
       boolean isConvLattice = (line.indexOf("- CONVENTIONAL") >= 0);
       if (isConvLattice) {
@@ -133,10 +140,8 @@ public class CrystalReader extends AtomSetCollectionReader {
           return true; // just for properties
         // no input coordinates -- continue;
       }
-      if (isPrimitive) {
-        readCellParams(true);
-      } else {
-        readCellParams(true);
+      readCellParams(true);
+      if (!isPrimitive) {
         discardLinesUntilContains(" TRANSFORMATION");
         readTransformationMatrix();
         discardLinesUntilContains(" CRYSTALLOGRAPHIC");
@@ -172,7 +177,7 @@ public class CrystalReader extends AtomSetCollectionReader {
       if (line.startsWith(" SHIFT OF THE ORIGIN"))
         return readShift();
       if (line.startsWith(" INPUT COORDINATES")) {
-        readCrystallographicCoords();
+        readCrystallographicCoords(); // note, these will not be the full set of atoms, so we IGNORE VIBRATIONS
         if (inputOnly)
           continuing = false;
         return true;
@@ -180,7 +185,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     }
 
     if (line
-        .startsWith(" DIRECT LATTICE VECTORS CARTESIAN COMPONENTS (ANGSTROM)"))
+        .startsWith(" DIRECT LATTICE VECTOR"))
       return setDirect();
 
     if (line.indexOf("DIMENSIONALITY OF THE SYSTEM") >= 0) {
@@ -204,18 +209,19 @@ public class CrystalReader extends AtomSetCollectionReader {
       isSlab = false;
       return true;
     }
-    
-    if(line.indexOf("* CLUSTER CALCULATION") >= 0){
-      isMolecular =true;
+
+    if (line.indexOf("* CLUSTER CALCULATION") >= 0) {
+      isMolecular = true;
       isSlab = false;
-      isPolymer =false;
+      isPolymer = false;
       return true;
     }
 
-    if (isPrimitive && line.startsWith(" ATOMS IN THE ASYMMETRIC UNIT")) {
+    if ((isPrimitive && line.startsWith(" ATOMS IN THE ASYMMETRIC UNIT"))
+        || isProperties && line.startsWith("   ATOM N.AT.")) {
       if (!doGetModel(++modelNumber))
         return checkLastModel();
-      return readFractionalCoords();
+      return readAtoms();
     }
 
     if (!doProcessLines)
@@ -242,11 +248,13 @@ public class CrystalReader extends AtomSetCollectionReader {
     if (line.startsWith(" TOTAL ATOMIC CHARGES"))
       return readTotalAtomicCharges();
 
-    if (addVibrations && line.contains(isVersion3 ? "EIGENVALUES (EV) OF THE MASS": "EIGENVALUES (EIGV) OF THE MASS") 
-        || line.indexOf("LONGITUDINAL OPTICAL (LO)") >= 0) {
+    if (addVibrations
+        && line.contains(isVersion3 ? "EIGENVALUES (EV) OF THE MASS"
+            : "EIGENVALUES (EIGV) OF THE MASS")
+            || line.indexOf("LONGITUDINAL OPTICAL (LO)") >= 0) {
       if (vInputCoords != null)
         processInputCoords();
-     isLongMode = (line.indexOf("LONGITUDINAL OPTICAL (LO)") >= 0);
+      isLongMode = (line.indexOf("LONGITUDINAL OPTICAL (LO)") >= 0);
       return readFrequencies();
     }
 
@@ -259,9 +267,23 @@ public class CrystalReader extends AtomSetCollectionReader {
     if (line.startsWith(" TOTAL ATOMIC SPINS  :"))
       return readMagneticMoments();
 
+    if (!isProperties)
+      return true;
+    
+    /// From here on we are considering only keywords of properties output files
+
+    if (line.startsWith(" DEFINITION OF TRACELESS"))
+      return getPropertyTensors();
+   
+    if (line.startsWith(" MULTIPOLE ANALYSIS BY ATOMS")) {
+      appendLoadNote("Multipole Analysis");
+      return true;
+    }
+    
     return true;
   }
 
+  @Override
   protected void finalizeReader() throws Exception {
     if (vInputCoords != null)
       processInputCoords();
@@ -276,15 +298,20 @@ public class CrystalReader extends AtomSetCollectionReader {
    0.290663292155E+01   0.000000000000E+00   0.460469095849E+01
   -0.145331646077E+01   0.251721794953E+01   0.460469095849E+01
   -0.145331646077E+01  -0.251721794953E+01   0.460469095849E+01
+  
+  or
+  
+   DIRECT LATTICE VECTOR COMPONENTS (BOHR)
+        11.12550    0.00000    0.00000
+         0.00000   10.45091    0.00000
+         0.00000    0.00000    8.90375
+
+
    */
 
   private boolean setDirect() throws Exception {
-    readLine();
-    directLatticeVectors = new Vector3f[3];
-    directLatticeVectors[0] = getPoint3f(null, 0);
-    directLatticeVectors[1] = getPoint3f(null, 0);
-    directLatticeVectors[2] = getPoint3f(null, 0);
-
+    boolean isBohr = (line.indexOf("(BOHR") >= 0);
+    directLatticeVectors = read3Vectors(isBohr);
     if (Logger.debugging) {
       addJmolScript("draw va vector {0 0 0} "
           + Escape.escape(directLatticeVectors[0]) + " color red");
@@ -315,19 +342,10 @@ public class CrystalReader extends AtomSetCollectionReader {
       mp.getColumn(1, b);
     }
     matUnitCellOrientation = Quaternion.getQuaternionFrame(new Point3f(), a, b)
-        .getMatrix();
+    .getMatrix();
     Logger.info("oriented unit cell is in model "
         + atomSetCollection.getAtomSetCount());
-    return true;
-  }
-
-  private Vector3f getPoint3f(float[] f, int pt) throws Exception {
-    if (f == null) {
-      f = new float[3];
-      fillFloatArray(f, null, 0);
-      return new Vector3f(f[0], f[1], f[2]);
-    }
-    return new Vector3f(f[pt++], f[pt++], f[pt]);
+    return !isProperties;
   }
 
   /*
@@ -338,12 +356,9 @@ public class CrystalReader extends AtomSetCollectionReader {
    */
 
   private void readTransformationMatrix() throws Exception {
-    float[] f = new float[9];
-    fillFloatArray(f, null, 0);
-    primitiveToCryst = new Matrix3f(f);
+    primitiveToCryst = new Matrix3f(fillFloatArray(null, 0, new float[9]));
   }
 
-
   private boolean readShift() {
     //  SHIFT OF THE ORIGIN                  :    3/4    1/4      0
     String[] tokens = getTokens();
@@ -378,7 +393,7 @@ public class CrystalReader extends AtomSetCollectionReader {
       else
         break;
       if (atomSetCollection.getAtomCount() > 0)
-        atomSetCollection.setAtomSetAuxiliaryProperty(key, tokens[2]);
+        atomSetCollection.setAtomSetModelProperty(key, tokens[2]);
       readLine();
     }
     return true;
@@ -394,9 +409,9 @@ public class CrystalReader extends AtomSetCollectionReader {
       line = TextFormat.simpleReplace(line, "DENSITY", "DENSITY ");
     }
     String densityPrim = tokens[10];
-    atomSetCollection.setAtomSetAuxiliaryProperty("volumePrimitive", TextFormat
-        .formatDecimal(parseFloat(volumePrim), 3));
-    atomSetCollection.setAtomSetAuxiliaryProperty("densityPrimitive",
+    atomSetCollection.setAtomSetModelProperty("volumePrimitive",
+        TextFormat.formatDecimal(parseFloat(volumePrim), 3));
+    atomSetCollection.setAtomSetModelProperty("densityPrimitive",
         TextFormat.formatDecimal(parseFloat(densityPrim), 3));
     return true;
   }
@@ -429,7 +444,7 @@ public class CrystalReader extends AtomSetCollectionReader {
   }
 
   private void setData(String name, String data, int pt, int dp)
-      throws Exception {
+  throws Exception {
     if (vInputCoords != null)
       processInputCoords();
     String[] s = new String[atomCount];
@@ -440,21 +455,31 @@ public class CrystalReader extends AtomSetCollectionReader {
         s[j++] = tokens[pt];
     }
     data = TextFormat.join(s, '\n', 0);
-    atomSetCollection.setAtomSetAuxiliaryProperty(name, data);
+    atomSetCollection.setAtomSetAtomProperty(name, data);
   }
 
   private boolean readHeader() throws Exception {
     discardLinesUntilContains("*                                CRYSTAL");
+    
     isVersion3 = (line.indexOf("CRYSTAL03") >= 0);
     discardLinesUntilContains("EEEEEEEEEE");
-    atomSetCollection.setCollectionName(readLine().trim()
-        + (desiredModelNumber == 0 ? " (optimized)" : ""));
-    readLine();
+    String name;
+    if (readLine().length() == 0) {
+      name = readLines(2).trim();
+    } else {
+      name = line.trim();
+      readLine();
+    }
     String type = readLine().trim();
     /*
      * This is when the initial geometry is read from an external file GEOMETRY
      * INPUT FROM EXTERNAL FILE (FORTRAN UNIT 34)
      */
+    int pt = type.indexOf("- PROPERTIES"); 
+    if (pt >= 0) {
+      isProperties = true;
+      type = type.substring(0, pt).trim();
+    }
     if (type.indexOf("EXTERNAL FILE") >= 0) {
       type = readLine().trim();
       isPolymer = (type.equals("1D - POLYMER"));
@@ -463,6 +488,8 @@ public class CrystalReader extends AtomSetCollectionReader {
       isPolymer = (type.equals("POLYMER CALCULATION"));
       isSlab = (type.equals("SLAB CALCULATION"));
     }
+    atomSetCollection.setCollectionName(name
+        + (!isProperties && desiredModelNumber == 0 ? " (optimized)" : ""));
     atomSetCollection.setAtomSetCollectionAuxiliaryInfo("symmetryType", type);
     if ((isPolymer || isSlab) && !isPrimitive) {
       Logger.error("Cannot use FILTER \"conventional\" with POLYMER or SLAB");
@@ -475,52 +502,55 @@ public class CrystalReader extends AtomSetCollectionReader {
       isMolecular = doProcessLines = true;
       readLine();
       atomSetCollection.setAtomSetCollectionAuxiliaryInfo(
-          "molecularCalculationPointGroup", line.substring(
-              line.indexOf(" OR ") + 4).trim());
+          "molecularCalculationPointGroup",
+          line.substring(line.indexOf(" OR ") + 4).trim());
       return false;
     }
+    spaceGroupName = "P1";
     if (!isPrimitive) {
-      discardLines(5);
-      setSpaceGroupName(line.indexOf(":") >= 0 ? line.substring(
-          line.indexOf(":") + 1).trim() : "P1");
+      readLines(5);
+      pt = line.indexOf(":"); 
+      if (pt >= 0)
+        spaceGroupName =  line.substring(pt + 1).trim();
     }
-    return true;
+    doApplySymmetry = isProperties;
+    return !isProperties;
   }
 
   private void readCellParams(boolean isNewSet) throws Exception {
-    directLatticeVectors = null;
+    float f = (line.indexOf("(BOHR") >= 0 ? ANGSTROMS_PER_BOHR : 1);
     if (isNewSet)
       newAtomSet();
     if (isPolymer && !isPrimitive) {
-      float a = parseFloat(line.substring(line.indexOf("CELL") + 4));
-      setUnitCell(a, -1, -1, 90, 90, 90);
+      setUnitCell(parseFloat(line.substring(line.indexOf("CELL") + 4)) * f, -1, -1, 90, 90, 90);
     } else {
       discardLinesUntilContains("GAMMA");
       String[] tokens = getTokens(readLine());
       if (isSlab) {
         if (isPrimitive)
-          setUnitCell(parseFloat(tokens[0]), parseFloat(tokens[1]), -1,
+          setUnitCell(parseFloat(tokens[0]) * f, parseFloat(tokens[1]) * f, -1,
               parseFloat(tokens[3]), parseFloat(tokens[4]),
               parseFloat(tokens[5]));
         else
-          setUnitCell(parseFloat(tokens[0]), parseFloat(tokens[1]), -1, 90, 90,
+          setUnitCell(parseFloat(tokens[0]) * f, parseFloat(tokens[1]) * f, -1, 90, 90,
               parseFloat(tokens[2]));
       } else if (isPolymer) {
-        setUnitCell(parseFloat(tokens[0]), -1, -1, parseFloat(tokens[3]),
+        setUnitCell(parseFloat(tokens[0]) * f, -1, -1, parseFloat(tokens[3]),
             parseFloat(tokens[4]), parseFloat(tokens[5]));
       } else {
-        setUnitCell(parseFloat(tokens[0]), parseFloat(tokens[1]),
-            parseFloat(tokens[2]), parseFloat(tokens[3]),
+        setUnitCell(parseFloat(tokens[0]) * f, parseFloat(tokens[1]) * f,
+            parseFloat(tokens[2]) * f, parseFloat(tokens[3]),
             parseFloat(tokens[4]), parseFloat(tokens[5]));
       }
     }
   }
 
   /**
-   * create arrays that maps primitive atoms to conventional atoms
-   * in a 1:1 fashion. Creates:
+   * create arrays that maps primitive atoms to conventional atoms in a 1:1
+   * fashion. Creates:
+   * 
+   * int[] primitiveToIndex -- points to model-based atomIndex
    * 
-   *  int[] primitiveToIndex -- points to model-based atomIndex
    * @return TRUE
    * 
    * @throws Exception
@@ -536,7 +566,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     for (int i = 0; i < n; i++)
       indexToPrimitive[i] = -1;
 
-    discardLines(3);
+    readLines(3);
     while (readLine() != null && line.indexOf(" NOT IRREDUCIBLE") >= 0) {
       // example HA_BULK_PBE_FREQ.OUT
       // we remove unnecessary atoms. This is important, because
@@ -548,7 +578,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     }
 
     // COORDINATES OF THE EQUIVALENT ATOMS
-    discardLines(3);
+    readLines(3);
     int iPrim = 0;
     int nPrim = 0;
     while (readLine() != null && line.indexOf("NUMBER") < 0) {
@@ -567,13 +597,13 @@ public class CrystalReader extends AtomSetCollectionReader {
         if (bsInputAtomsIgnore.get(i))
           vInputCoords.remove(i);
     atomCount = vInputCoords.size();
-    Logger.info(nPrim + " primitive atoms and " + atomCount + " conventionalAtoms");
+    Logger.info(nPrim + " primitive atoms and " + atomCount
+        + " conventionalAtoms");
     primitiveToIndex = new int[nPrim];
     for (int i = 0; i < nPrim; i++)
       primitiveToIndex[i] = -1;
     for (int i = atomCount; --i >= 0;) {
-      iPrim = indexToPrimitive[parseInt(((String) vInputCoords.get(i))
-          .substring(0, 4)) - 1];
+      iPrim = indexToPrimitive[parseInt(vInputCoords.get(i).substring(0, 4)) - 1];
       if (iPrim >= 0)
         primitiveToIndex[iPrim] = i;
     }
@@ -581,41 +611,71 @@ public class CrystalReader extends AtomSetCollectionReader {
   }
 
   /*
-   * ATOMS IN THE ASYMMETRIC UNIT 30 - ATOMS IN THE UNIT CELL: 30 ATOM X/A Y/B
-   * Z(ANGSTROM)
-   * ***************************************************************** 1 T 26 FE
-   * 3.332220233571E-01 1.664350001467E-01 5.975038441891E+00 2 T 8 O
-   * -3.289334452690E-01 1.544678332212E-01 5.601153565811E+00
+  ATOMS IN THE ASYMMETRIC UNIT    2 - ATOMS IN THE UNIT CELL:    4
+     ATOM              X/A                 Y/B                 Z/C    
+  *******************************************************************************
+   1 T 282 PB    0.000000000000E+00  5.000000000000E-01  2.385000000000E-01
+   2 F 282 PB    5.000000000000E-01  0.000000000000E+00 -2.385000000000E-01
+   3 T   8 O     0.000000000000E+00  0.000000000000E+00  0.000000000000E+00
+   4 F   8 O     5.000000000000E-01  5.000000000000E-01  0.000000000000E+00
+   
+   or
+   
+      ATOM N.AT.  SHELL    X(A)      Y(A)      Z(A)      EXAD       N.ELECT.
+  *******************************************************************************
+    1  282 PB    4     1.331    -0.077     1.178   1.934E-01       2.891
+    2  282 PB    4    -1.331     0.077    -1.178   1.934E-01       2.891
+    3  282 PB    4     1.331    -2.688    -1.178   1.934E-01       2.891
+    4  282 PB    4    -1.331     2.688     1.178   1.934E-01       2.891
+    5    8 O     5    -0.786     0.522     1.178   6.500E-01       9.109
+    6    8 O     5     0.786    -0.522    -1.178   6.500E-01       9.109
+    7    8 O     5    -0.786     2.243    -1.178   6.500E-01       9.109
+    8    8 O     5     0.786    -2.243     1.178   6.500E-01       9.109
+
    */
-  private boolean readFractionalCoords() throws Exception {
+  private boolean readAtoms() throws Exception {
     if (isMolecular)
       newAtomSet();
-    readLine();
-    readLine();
+    while (readLine() != null && line.indexOf("*") < 0) {
+      if (line.indexOf("X(ANGSTROM") >= 0) {
+        // fullerene from slab has this.
+        setFractionalCoordinates(false);
+        isMolecular = true;
+      }
+    }
     int i = atomIndexLast;
-    boolean doNormalizePrimitive = isPrimitive && !isMolecular && !isPolymer
-        && !isSlab && (!doApplySymmetry || latticeCells[2] != 0);
+    // I turned off normalization -- proper way to do this is to 
+    // add the "packed" keyword. As it was, it was impossible to
+    // load the file with its original coordinates, which in many
+    // cases are VERY interesting and far better (in my opinion!)
+    
+    boolean doNormalizePrimitive = false;// && isPrimitive && !isMolecular && !isPolymer && !isSlab && (!doApplySymmetry || latticeCells[2] != 0);
     atomIndexLast = atomSetCollection.getAtomCount();
-    while (readLine() != null && line.length() > 0 && line.indexOf("=") < 0) {
+
+    while (readLine() != null && line.length() > 0 && line.indexOf(isPrimitive ? "*" : "=") < 0) {
       Atom atom = atomSetCollection.addNewAtom();
       String[] tokens = getTokens();
-      int atomicNumber = getAtomicNumber(tokens[2]);
-      atom.atomName = getAtomName(tokens[3]);
-      float x = parseFloat(tokens[4]);
-      float y = parseFloat(tokens[5]);
-      float z = parseFloat(tokens[6]);
+      int pt = (isProperties ? 1 : 2);
+      atom.elementSymbol = getElementSymbol(getAtomicNumber(tokens[pt++]));
+      atom.atomName = getAtomName(tokens[pt++]);
+      if (isProperties)
+        pt++; // skip SHELL
+      float x = parseFloat(tokens[pt++]);
+      float y = parseFloat(tokens[pt++]);
+      float z = parseFloat(tokens[pt]);
       if (haveCharges)
         atom.partialCharge = atomSetCollection.getAtom(i++).partialCharge;
-      // note: this normalization is unique to this reader -- all other
-      //       readers operate through symmetry application
-      if (x < 0 && (isPolymer || isSlab || doNormalizePrimitive))
-        x += 1;
-      if (y < 0 && (isSlab || doNormalizePrimitive))
-        y += 1;
-      if (z < 0 && doNormalizePrimitive)
-        z += 1;
+      if (iHaveFractionalCoordinates && !isProperties) {
+        // note: this normalization is unique to this reader -- all other
+        //       readers operate through symmetry application
+        if (x < 0 && (isPolymer || isSlab || doNormalizePrimitive))
+          x += 1;
+        if (y < 0 && (isSlab || doNormalizePrimitive))
+          y += 1;
+        if (z < 0 && doNormalizePrimitive)
+          z += 1;
+      }
       setAtomCoord(atom, x, y, z);
-      atom.elementSymbol = getElementSymbol(atomicNumber);
     }
     atomCount = atomSetCollection.getAtomCount() - atomIndexLast;
     return true;
@@ -625,7 +685,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     String atomName = s;
     if (atomName.length() > 1 && Character.isLetter(atomName.charAt(1)))
       atomName = atomName.substring(0, 1)
-          + Character.toLowerCase(atomName.charAt(1)) + atomName.substring(2);
+      + Character.toLowerCase(atomName.charAt(1)) + atomName.substring(2);
     return atomName;
   }
 
@@ -654,7 +714,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     // we only store them, because we may want to delete some
     readLine();
     readLine();
-    vInputCoords = new ArrayList();
+    vInputCoords = new ArrayList<String>();
     while (readLine() != null && line.length() > 0)
       vInputCoords.add(line);
   }
@@ -664,7 +724,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     atomCount = vInputCoords.size();
     for (int i = 0; i < atomCount; i++) {
       Atom atom = atomSetCollection.addNewAtom();
-      String[] tokens = getTokens((String) vInputCoords.get(i));
+      String[] tokens = getTokens(vInputCoords.get(i));
       int atomicNumber, offset;
       if (tokens.length == 7) {
         atomicNumber = getAtomicNumber(tokens[2]);
@@ -691,10 +751,12 @@ public class CrystalReader extends AtomSetCollectionReader {
   }
 
   private void newAtomSet() throws Exception {
-    if (atomSetCollection.getAtomCount() == 0)
-      return;
-    applySymmetryAndSetTrajectory();
-    atomSetCollection.newAtomSet();
+    if (atomSetCollection.getAtomCount() > 0) {
+      applySymmetryAndSetTrajectory();
+      atomSetCollection.newAtomSet();
+    }
+    if (spaceGroupName != null)
+      setSpaceGroupName(spaceGroupName);
   }
 
   private void readEnergy() {
@@ -722,7 +784,7 @@ public class CrystalReader extends AtomSetCollectionReader {
     if (haveCharges || atomSetCollection.getAtomCount() == 0)
       return true;
     haveCharges = true;
-    discardLines(3);
+    readLines(3);
     Atom[] atoms = atomSetCollection.getAtoms();
     int i0 = atomSetCollection.getLastAtomSetAtomIndex();
     int iPrim = 0;
@@ -731,7 +793,7 @@ public class CrystalReader extends AtomSetCollectionReader {
         int iConv = getAtomIndexFromPrimitiveIndex(iPrim);
         if (iConv >= 0)
           atoms[i0 + iConv].partialCharge = parseFloat(line.substring(9, 11))
-              - parseFloat(line.substring(12, 18));
+          - parseFloat(line.substring(12, 18));
         iPrim++;
       }
     return true;
@@ -783,9 +845,10 @@ public class CrystalReader extends AtomSetCollectionReader {
     String Sfrag = "";
     while (readLine() != null && line.indexOf("(") >= 0)
       Sfrag += line;
-    Sfrag = TextFormat.simpleReplace(Sfrag, "(", " (");
+    Sfrag = TextFormat.simpleReplace(Sfrag, "(", " ");
+    Sfrag = TextFormat.simpleReplace(Sfrag, ")", " ");
     String[] tokens = getTokens(Sfrag);
-    for (int i = 0, pos = 0; i < numAtomsFrag; i++, pos += 5)
+    for (int i = 0, pos = 0; i < numAtomsFrag; i++, pos += 3)
       atomFrag[i] = getAtomIndexFromPrimitiveIndex(parseInt(tokens[pos]) - 1);
 
     Arrays.sort(atomFrag); // the frequency module needs these sorted
@@ -819,22 +882,21 @@ public class CrystalReader extends AtomSetCollectionReader {
      41-  43    0.9004E-06    208.2551    6.2433 (F1U)    244.7   10.821   0.3244
    */
   private boolean readFrequencies() throws Exception {
-    
+
     energy = null; // don't set energy for these models
     discardLinesUntilContains("MODES");
     // This line is always there
-    boolean haveIntensities = (line.indexOf("INTENS") >= 0);  
+    boolean haveIntensities = (line.indexOf("INTENS") >= 0);
     readLine();
-    List vData = new ArrayList();
+    List<String[]> vData = new ArrayList<String[]>();
     int freqAtomCount = atomCount;
     while (readLine() != null && line.length() > 0) {
       int i0 = parseInt(line.substring(1, 5));
       int i1 = parseInt(line.substring(6, 10));
       String irrep = (isLongMode ? line.substring(48, 51) : line.substring(49,
-       52)).trim();
-      String intens = (!haveIntensities ? "not available" : 
-        (isLongMode ? line.substring(53, 61) 
-            : line.substring(59, 69).replace(')', ' ')).trim());
+          52)).trim();
+      String intens = (!haveIntensities ? "not available" : (isLongMode ? line
+          .substring(53, 61) : line.substring(59, 69).replace(')', ' ')).trim());
 
       // not all crystal calculations include intensities values
       // this feature is activated when the keyword INTENS is on the input
@@ -864,11 +926,12 @@ public class CrystalReader extends AtomSetCollectionReader {
       int iAtom0 = 0;
       int nData = vData.size();
       for (int i = 0; i < frequencyCount; i++) {
-        tokens = (String[]) vData.get(vibrationNumber % nData);
+        tokens = vData.get(vibrationNumber % nData);
         ignore[i] = (!doGetVibration(++vibrationNumber) || tokens == null);
         if (ignore[i])
           continue;
-        lastAtomCount = cloneLastAtomSet(atomCount);
+        applySymmetryAndSetTrajectory();
+        lastAtomCount = cloneLastAtomSet(atomCount, null);
         if (i == 0)
           iAtom0 = atomSetCollection.getLastAtomSetAtomIndex();
         setFreqValue(frequencies[i], tokens);
@@ -883,16 +946,82 @@ public class CrystalReader extends AtomSetCollectionReader {
 
   private void setFreqValue(float freq, String[] data) {
     String activity = "IR: " + data[2] + ", Ram.: " + data[3];
-    atomSetCollection.setAtomSetFrequency(null, activity, "" + freq,
-        null);
-    atomSetCollection.setAtomSetProperty("IRintensity", data[1] + " km/Mole");
-    atomSetCollection.setAtomSetProperty("vibrationalSymmetry", data[0]);
-    atomSetCollection.setAtomSetProperty("IRactivity", data[2]);
-    atomSetCollection.setAtomSetProperty("Ramanactivity", data[3]);
+    atomSetCollection.setAtomSetFrequency(null, activity, "" + freq, null);
+    atomSetCollection.setAtomSetModelProperty("IRintensity", data[1] + " km/Mole");
+    atomSetCollection.setAtomSetModelProperty("vibrationalSymmetry", data[0]);
+    atomSetCollection.setAtomSetModelProperty("IRactivity", data[2]);
+    atomSetCollection.setAtomSetModelProperty("Ramanactivity", data[3]);
     atomSetCollection.setAtomSetName((isLongMode ? "LO " : "") + data[0] + " "
         + TextFormat.formatDecimal(freq, 2) + " cm-1 ("
         + TextFormat.formatDecimal(Float.parseFloat(data[1]), 0)
         + " km/Mole), " + activity);
   }
 
+  /*
+  DEFINITION OF TRACELESS QUADRUPOLE TENSORS:
+
+  (3XX-RR)/2=(2,2)/4-(2,0)/2
+  (3YY-RR)/2=-(2,2)/4-(2,0)/2
+  (3ZZ-RR)/2=(2,0)
+  3XY/2=(2,-2)/4     3XZ/2=(2,1)/2     3YZ/2=(2,-1)/2
+
+  *** ATOM N.     1 (Z=282) PB
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  6.265724E-01 BB -2.651563E-01 CC -3.614161E-01
+
+  *** ATOM N.     2 (Z=282) PB
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  6.265724E-01 BB -2.651563E-01 CC -3.614161E-01
+
+  *** ATOM N.     3 (Z=282) PB
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  6.265724E-01 BB -2.651563E-01 CC -3.614161E-01
+
+  *** ATOM N.     4 (Z=282) PB
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  6.265724E-01 BB -2.651563E-01 CC -3.614161E-01
+
+  *** ATOM N.     5 (Z=  8) O 
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  3.258167E-01 BB -1.529525E-01 CC -1.728642E-01
+
+  *** ATOM N.     6 (Z=  8) O 
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  3.258167E-01 BB -1.529525E-01 CC -1.728642E-01
+
+  *** ATOM N.     7 (Z=  8) O 
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  3.258167E-01 BB -1.529525E-01 CC -1.728642E-01
+
+  *** ATOM N.     8 (Z=  8) O 
+
+  TENSOR IN PRINCIPAL AXIS SYSTEM
+  AA  3.258167E-01 BB -1.529525E-01 CC -1.728642E-01
+  TTTTTTTTTTTTTTTTTTTTTTTTTTTTTT POLI        TELAPSE        0.05 TCPU        0.04
+    */
+   private boolean getPropertyTensors() throws Exception {
+     readLines(6);
+     Atom[] atoms = atomSetCollection.getAtoms();
+     while (readLine() != null  && line.startsWith(" *** ATOM")) {
+       String[] tokens = getTokens();
+       int index = parseInt(tokens[3]) - 1;
+       Atom atom = atoms[index];
+       tokens = getTokens(readLines(3));
+       atom.ellipsoid = symmetry.getEllipsoid(directLatticeVectors, 
+           parseFloat(tokens[1]), 
+           parseFloat(tokens[3]), 
+           parseFloat(tokens[5]));
+       readLine();
+     }
+     return true;
+   }
+
+
 }
