@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-03-16 20:37:57 +0100 (mer., 16 mars 2011) $
- * $Revision: 15290 $
+ * $Date: 2012-03-11 16:08:21 +0100 (dim., 11 mars 2012) $
+ * $Revision: 16878 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -26,10 +26,11 @@ package org.jmol.adapter.smarter;
 
 import java.io.BufferedReader;
 import java.io.InputStream;
-import java.util.Hashtable;
-import java.util.StringTokenizer;
-import java.util.List;
 import java.util.ArrayList;
+import java.util.Hashtable;
+import java.util.List;
+import java.util.Map;
+import java.util.StringTokenizer;
 
 import netscape.javascript.JSObject;
 
@@ -44,14 +45,14 @@ public class Resolver {
 
   private final static String classBase = "org.jmol.adapter.readers.";
   private final static String[] readerSets = new String[] {
-    "cifpdb.", ";Cif;Pdb;Pqr;",
-    "molxyz.", ";Mol;Xyz;",
-    "more.", ";Gromacs;MdCrd;MdTop;Mol2;",
+    "cifpdb.", ";Cif;Pdb;",
+    "molxyz.", ";Mol3D;Mol;Xyz;",
+    "more.", ";BinaryDcd;Gromacs;MdCrd;MdTop;Mol2;Pqr;P2n;",
     "quantum.", ";Adf;Csf;Dgrid;GamessUK;GamessUS;Gaussian;GausianWfn;Jaguar;" +
                  "Molden;MopacGraphf;GenNBO;NWChem;Odyssey;Psi;Qchem;Spartan;SpartanSmol;" +
                  "WebMO;",
     "simple.", ";Alchemy;Ampac;Cube;FoldingXyz;GhemicalMM;HyperChem;Jme;Mopac;", 
-    "xtal.", ";Aims;Castep;Crystal;Shelx;Wien2k;"
+    "xtal.", ";Aims;Castep;Crystal;Dmol;Espresso;Gulp;Shelx;Siesta;VaspOutcar;Wien2k;"
   };
   
   public final static String getReaderClassBase(String type) {
@@ -186,14 +187,13 @@ public class Resolver {
     StringBuffer data = new StringBuffer();
     data.append("Zip File Directory: ").append("\n").append(
         Escape.escape(zipDirectory, true)).append("\n");
-    Hashtable fileData = new Hashtable();
+    Map<String, String> fileData = new Hashtable<String, String>();
     ZipUtil.getAllData(is, new String[] {}, "",
         "Molecule", fileData);
     String prefix = "|";
-    String outputData = (String) fileData.get(prefix + "output");
+    String outputData = fileData.get(prefix + "output");
     if (outputData == null)
-      outputData = (String) fileData.get((prefix = "|" + zipDirectory[1])
-          + "output");
+      outputData = fileData.get((prefix = "|" + zipDirectory[1]) + "output");
     data.append(outputData);
     String[] files = getSpartanFileList(prefix, getSpartanDirs(outputData));
     for (int i = 2; i < files.length; i++) {
@@ -219,21 +219,21 @@ public class Resolver {
    * @throws Exception
    */
   static Object getAtomCollectionReader(String fullName, String type,
-                        BufferedReader bufferedReader, Hashtable htParams,
-                        int ptFile) throws Exception {
+                                        BufferedReader bufferedReader,
+                                        Map<String, Object> htParams, int ptFile)
+      throws Exception {
     AtomSetCollectionReader atomSetCollectionReader = null;
     String readerName;
-    fullName = fullName.replace('\\','/');
+    fullName = fullName.replace('\\', '/');
     String errMsg = null;
     if (type != null) {
       readerName = getReaderFromType(type);
       if (readerName == null)
-        errMsg =  "unrecognized file format type " + type;
-      else 
+        errMsg = "unrecognized file format type " + type;
+      else
         Logger.info("The Resolver assumes " + readerName);
     } else {
-      readerName = determineAtomSetCollectionReader(
-          bufferedReader, true);
+      readerName = determineAtomSetCollectionReader(bufferedReader, true);
       if (readerName.charAt(0) == '\n') {
         type = (String) htParams.get("defaultType");
         if (type != null) {
@@ -243,12 +243,12 @@ public class Resolver {
             readerName = type;
         }
       }
-      if (readerName.charAt(0) == '\n') 
+      if (readerName.charAt(0) == '\n')
         errMsg = "unrecognized file format for file " + fullName + "\n"
             + readerName;
       else if (readerName.equals("spt"))
         errMsg = "NOTE: file recognized as a script file: " + fullName + "\n";
-      else
+      else if (!fullName.equals("ligand"))
         Logger.info("The Resolver thinks " + readerName);
     }
     if (errMsg != null) {
@@ -256,14 +256,14 @@ public class Resolver {
       return errMsg;
     }
     if (htParams == null)
-      htParams = new Hashtable();
-    htParams.put("ptFile", new Integer(ptFile));
+      htParams = new Hashtable<String, Object>();
+    htParams.put("ptFile", Integer.valueOf(ptFile));
     if (ptFile <= 0)
       htParams.put("readerName", readerName);
     if (readerName.indexOf("Xml") == 0)
       readerName = "Xml";
     String className = null;
-    Class atomSetCollectionReaderClass;
+    Class<?> atomSetCollectionReaderClass;
     String err = null;
     try {
       try {
@@ -295,9 +295,9 @@ public class Resolver {
    * @return an AtomSetCollection or a String error
    * @throws Exception
    */
-  static Object DOMResolve(Object DOMNode, Hashtable htParams) throws Exception {
+  static Object DOMResolve(Object DOMNode, Map<String, Object> htParams) throws Exception {
     String className = null;
-    Class atomSetCollectionReaderClass;
+    Class<?> atomSetCollectionReaderClass;
     AtomSetCollectionReader atomSetCollectionReader; 
     String atomSetCollectionReaderName = getXmlType((JSObject) DOMNode);
     if (Logger.debugging) {
@@ -361,7 +361,7 @@ public class Resolver {
         || outputFileData.startsWith("FILE NOT FOUND")
         || outputFileData.indexOf("<html") >= 0)
       return new String[] { "M0001" };
-    List v = new ArrayList();
+    List<String> v = new ArrayList<String>();
     String token;
     String lasttoken = "";
     try {
@@ -399,7 +399,7 @@ public class Resolver {
     }
     String[] dirs = new String[v.size()];
     for (int i = 0; i < v.size(); i++)
-      dirs[i] = (String) v.get(i);
+      dirs[i] = v.get(i);
     return dirs;
   }
   
@@ -439,24 +439,11 @@ public class Resolver {
    */
   private static String determineAtomSetCollectionReader(BufferedReader bufferedReader, boolean returnLines)
       throws Exception {
-    String[] lines = new String[16];
     LimitedLineReader llr = new LimitedLineReader(bufferedReader, 16384);
-    int nLines = 0;
-    for (int i = 0; i < lines.length; ++i) {
-      lines[i] = llr.readLineWithNewline();
-      if (lines[i].length() > 0)
-        nLines++;
-    }
-
-    String readerName = checkSpecial(nLines, lines, false);
     
-    if (readerName != null)
-      return readerName;
-
-    // run these loops forward ... easier for people to understand
-    //file starts with added 4/26 to ensure no issue with NWChem files
+    // first check just the first 10 bytes
     
-    String leader = llr.getHeader(LEADER_CHAR_MAX);
+    String leader = llr.getHeader(LEADER_CHAR_MAX).trim();
 
     for (int i = 0; i < fileStartsWithRecords.length; ++i) {
       String[] recordTags = fileStartsWithRecords[i];
@@ -466,6 +453,61 @@ public class Resolver {
           return recordTags[0];
       }
     }
+
+    // PNG or BCD-encoded JPG or JPEG
+    if (leader.indexOf("PNG") == 1 || leader.indexOf("JPG") == 1
+        || leader.indexOf("JFIF") == 6)
+      return "spt"; // presume embedded script --- allows dragging into Jmol
+
+    // now allow identification in first 16 lines
+    // excluding those starting with "#"
+    
+    String[] lines = new String[16];
+    int nLines = 0;
+    for (int i = 0; i < lines.length; ++i) {
+      lines[i] = llr.readLineWithNewline();
+      if (lines[i].length() > 0)
+        nLines++;
+    }
+
+    String readerName;
+    if ((readerName = checkSpecial(nLines, lines, false)) != null)
+      return readerName;
+
+    if ((readerName = checkLineStarts(lines)) != null)
+      return readerName;
+
+    if ((readerName = checkHeaderContains(llr.getHeader(0))) != null)
+      return readerName;
+
+    if ((readerName = checkSpecial(nLines, lines, true)) != null)
+      return readerName;
+    
+    return (returnLines ? "\n" + lines[0] + "\n" + lines[1] + "\n" + lines[2] + "\n" : null);
+  }
+
+  private static String checkHeaderContains(String header) throws Exception {
+    for (int i = 0; i < headerContainsRecords.length; ++i) {
+      String[] recordTags = headerContainsRecords[i];
+      for (int j = 1; j < recordTags.length; ++j) {
+        String recordTag = recordTags[j];
+        if (header.indexOf(recordTag) < 0)
+          continue;
+        String type = recordTags[0];
+        // for XML check for an error message from a server -- certainly not XML
+        // but new CML format includes xmlns:xhtml="http://www.w3.org/1999/xhtml" in <cml> tag.
+        return (!type.equals("Xml") ? type 
+            : header.indexOf("<!DOCTYPE HTML PUBLIC") < 0
+              && header.indexOf("XHTML") < 0 
+              && (header.indexOf("xhtml") < 0 || header.indexOf("<cml") >= 0) 
+              ? getXmlType(header) 
+            : null);
+      }
+    }
+    return null;
+  }
+
+  private static String checkLineStarts(String[] lines) {
     for (int i = 0; i < lineStartsWithRecords.length; ++i) {
       String[] recordTags = lineStartsWithRecords[i];
       for (int j = 1; j < recordTags.length; ++j) {
@@ -476,43 +518,7 @@ public class Resolver {
         }
       }
     }
-
-    if (lines[0].indexOf("PNG") == 1 || lines[0].indexOf("JPG") == 1
-        || lines[0].indexOf("JFIF") == 6)
-      return "spt"; // presume embedded script --- allows dragging into Jmol
-    for (int i = 0; i < lines.length; ++i)
-      if (lines[i].indexOf("# Jmol state") >= 0)
-        return "spt";
-
-    //check for spt as a last resort, but ahead of XML, just in case
-    //a web file is being returned with an XML header.
-    
-    String header = llr.getHeader(0);
-    String type = null;
-    for (int i = 0; i < containsRecords.length; ++i) {
-      String[] recordTags = containsRecords[i];
-      for (int j = 1; j < recordTags.length; ++j) {
-        String recordTag = recordTags[j];
-        if (header.indexOf(recordTag) != -1) {
-          type = recordTags[0];
-          if (type.equals("Xml")) {
-            // check for an error message from a server -- certainly not XML
-            // but new CML format includes xmlns:xhtml="http://www.w3.org/1999/xhtml" in <cml> tag.
-            if (header.indexOf("XHTML") >= 0 || header.indexOf("xhtml") >= 0 && header.indexOf("<cml") < 0)
-              break; 
-            type = getXmlType(header);
-          }
-          return type;
-        }
-      }
-    }
-    
-    readerName = checkSpecial(nLines, lines, true);
-    
-    if (readerName != null)
-      return readerName;
-    
-    return (returnLines ? "\n" + lines[0] + "\n" + lines[1] + "\n" + lines[2] + "\n" : null);
+    return null;
   }
 
   private static String getXmlType(String header) throws Exception  {
@@ -541,13 +547,16 @@ public class Resolver {
     if (header.indexOf(">vasp") >= 0) {
       return specialTags[SPECIAL_VASP_XML][0];
     }
+    if (header.indexOf("<GEOMETRY_INFO>") >= 0) {
+      return specialTags[SPECIAL_QE_XML][0];
+    }
     
     return specialTags[SPECIAL_CML_XML][0] + "(unidentified)";
   }
 
   private final static int SPECIAL_JME                = 0;
   private final static int SPECIAL_MOPACGRAPHF        = 1;
-//  private final static int SPECIAL_V3000              = 2;
+  //private final static int SPECIAL_MOL3D              = 2; // only by MOL3D::
   private final static int SPECIAL_ODYSSEY            = 3;
   private final static int SPECIAL_MOL                = 4;
   private final static int SPECIAL_XYZ                = 5;
@@ -570,19 +579,20 @@ public class Resolver {
   public final static int SPECIAL_ODYSSEY_XML = 19;
   public final static int SPECIAL_XSD_XML     = 20;
   public final static int SPECIAL_VASP_XML    = 21; 
-  
-  public final static int SPECIAL_ARGUS_DOM   = 22;
-  public final static int SPECIAL_CML_DOM     = 23;
-  public final static int SPECIAL_CHEM3D_DOM  = 24;
-  public final static int SPECIAL_MOLPRO_DOM  = 25;
-  public final static int SPECIAL_ODYSSEY_DOM = 26;
-  public final static int SPECIAL_XSD_DOM     = 27; // not implemented
-  public final static int SPECIAL_VASP_DOM    = 28; 
+  public final static int SPECIAL_QE_XML      = 22; 
+ 
+  public final static int SPECIAL_ARGUS_DOM   = 23;
+  public final static int SPECIAL_CML_DOM     = 24;
+  public final static int SPECIAL_CHEM3D_DOM  = 25;
+  public final static int SPECIAL_MOLPRO_DOM  = 26;
+  public final static int SPECIAL_ODYSSEY_DOM = 27;
+  public final static int SPECIAL_XSD_DOM     = 28; // not implemented
+  public final static int SPECIAL_VASP_DOM    = 29; 
   
   public final static String[][] specialTags = {
     { "Jme" },
     { "MopacGraphf" },
-    { "V3000" },
+    { "Mol3D" },
     { "Odyssey" },
     { "Mol" },
     
@@ -595,6 +605,7 @@ public class Resolver {
     { "Castep" },
     { "Aims" },  
     { "Crystal" },  
+
     { "Gromacs" },
     { "GenNBO" },
     
@@ -605,6 +616,7 @@ public class Resolver {
     { "XmlOdyssey" },
     { "XmlXsd" },
     { "XmlVasp" },
+    { "XmlQE" },
 
     { "XmlArgus(DOM)" }, //19
     { "XmlCml(DOM)" },
@@ -618,89 +630,68 @@ public class Resolver {
 
   };
 
-  private final static String checkSpecial(int nLines, String[] lines, boolean isEnd) {
+  private final static String getReaderFromType(String type) {
+    type = type.toLowerCase();
+    String base = null;
+    if ((base = checkType(specialTags, type)) != null)
+      return base;
+    if ((base = checkType(fileStartsWithRecords, type)) != null)
+      return base;
+    if ((base = checkType(lineStartsWithRecords, type)) != null)
+      return base;
+    return checkType(headerContainsRecords, type);
+  }
+  
+  private final static String checkType(String[][] typeTags, String type) {
+    for (int i = 0; i < typeTags.length; ++i)
+      if (typeTags[i][0].toLowerCase().equals(type))
+        return typeTags[i][0];
+    return null;
+  }
+  
+  ////////////////////////////////////////////////////////////////
+  // file types that need special treatment
+  ////////////////////////////////////////////////////////////////
+
+  private final static String checkSpecial(int nLines, String[] lines,
+                                           boolean isEnd) {
     // the order here is CRITICAL
+    
     if (isEnd) {
       if (checkGromacs(lines))
         return specialTags[SPECIAL_GROMACS][0];
       if (checkCrystal(lines))
         return specialTags[SPECIAL_CRYSTAL][0];
+      if (checkCastep(lines))
+        return specialTags[SPECIAL_CASTEP][0];
+    } else {
+      if (nLines == 1 && lines[0].length() > 0
+          && Character.isDigit(lines[0].charAt(0)))
+        return specialTags[SPECIAL_JME][0]; //only one line, and that line starts with a number 
+      if (checkMopacGraphf(lines))
+        return specialTags[SPECIAL_MOPACGRAPHF][0]; //must be prior to checkFoldingXyz and checkMol
+      if (checkOdyssey(lines))
+        return specialTags[SPECIAL_ODYSSEY][0];
+      if (checkMol(lines))
+        return specialTags[SPECIAL_MOL][0];
+      if (checkXyz(lines))
+        return specialTags[SPECIAL_XYZ][0];
+      if (checkAlchemy(lines[0]))
+        return specialTags[SPECIAL_ALCHEMY][0];
+      if (checkFoldingXyz(lines))
+        return specialTags[SPECIAL_FOLDINGXYZ][0];
+      if (checkCube(lines))
+        return specialTags[SPECIAL_CUBE][0];
+      if (checkWien2k(lines))
+        return specialTags[SPECIAL_WIEN][0];
+      if (checkAims(lines))
+        return specialTags[SPECIAL_AIMS][0];
+      if (checkGenNBO(lines))
+        return specialTags[SPECIAL_GENNBO][0];
     }
-    if (nLines == 1 && lines[0].length() > 0
-        && Character.isDigit(lines[0].charAt(0)))
-      return specialTags[SPECIAL_JME][0]; //only one line, and that line starts with a number 
-    if (checkMopacGraphf(lines))
-      return specialTags[SPECIAL_MOPACGRAPHF][0]; //must be prior to checkFoldingXyz and checkMol
-    if (checkOdyssey(lines))
-      return specialTags[SPECIAL_ODYSSEY][0];
-    if (checkMol(lines))
-      return specialTags[SPECIAL_MOL][0];
-    if (checkXyz(lines))
-      return specialTags[SPECIAL_XYZ][0];
-    if (checkAlchemy(lines[0]))
-      return specialTags[SPECIAL_ALCHEMY][0];
-    if (checkFoldingXyz(lines))
-      return specialTags[SPECIAL_FOLDINGXYZ][0];
-    if (checkCube(lines))
-      return specialTags[SPECIAL_CUBE][0];
-    if (checkWien2k(lines))
-      return specialTags[SPECIAL_WIEN][0];
-    if (checkCastep(lines))
-      return specialTags[SPECIAL_CASTEP][0];
-    if (checkAims(lines))
-      return specialTags[SPECIAL_AIMS][0];
-    if (checkGenNBO(lines))
-      return specialTags[SPECIAL_GENNBO][0];
     return null;
   }
   
-  private static boolean checkGenNBO(String[] lines) {
-    // .31-.41 file or .nbo file
-    return (lines[1].startsWith(" Basis set information needed for plotting orbitals")
-      || lines[1].indexOf("s in the AO basis:") >= 0
-      || lines[2].indexOf(" N A T U R A L   A T O M I C   O R B I T A L") >= 0);
-  }
-  
-  private static boolean checkGromacs(String[] lines) {
-    if (Parser.parseInt(lines[1]) == Integer.MIN_VALUE)
-      return false;
-    int len = -1;
-    for (int i = 2; i < 16 && len != 0; i++)
-      if ((len = lines[i].length()) != 69 && len != 45 && len != 0)
-        return false;
-    return true;
-  }
-
-  private static boolean checkCrystal(String[] lines) {
-    String s = lines[1].trim();
-    if (s.equals("SLAB") ||s.equals("MOLECULE")
-        || s.equals("CRYSTAL")
-        || s.equals("POLYMER") || (s = lines[3]).equals("SLAB")
-        || s.equals("MOLECULE") || s.equals("POLYMER"))
-      return true;
-    for (int i = 0; i < lines.length; i++) {
-      if (lines[i].trim().equals("OPTGEOM"))
-        return true;
-    }
-    return false;
-  }
-
-  private static boolean checkWien2k(String[] lines) {
-    return (lines[2].startsWith("MODE OF CALC=") 
-        || lines[2].startsWith("             RELA")
-        || lines[2].startsWith("             NREL"));
-  }
-
-  private static boolean checkCastep(String[] lines) {
-    for ( int i = 0; i<lines.length; i++ ) {
-      if ( lines[i].toUpperCase().startsWith("%BLOCK LATTICE_ABC") ) return true;
-      if ( lines[i].toUpperCase().startsWith("%BLOCK LATTICE_CART") ) return true;
-      if ( lines[i].toUpperCase().startsWith("%BLOCK POSITIONS_FRAC") ) return true;
-      if ( lines[i].toUpperCase().startsWith("%BLOCK POSITIONS_ABS") ) return true;
-    }
-    return false;
-  }
-
   private static boolean checkAims(String[] lines) {
 
     // use same tokenizing mechanism as in AimsReader.java to also recognize
@@ -724,76 +715,6 @@ public class Resolver {
     return false;
   }
 
-
-  private final static String getReaderFromType(String type) {
-    type = type.toLowerCase();
-    String base = null;
-    if ((base = checkType(specialTags, type)) != null)
-      return base;
-    if ((base = checkType(fileStartsWithRecords, type)) != null)
-      return base;
-    if ((base = checkType(lineStartsWithRecords, type)) != null)
-      return base;
-    return checkType(containsRecords, type);
-  }
-  
-  private final static String checkType(String[][] typeTags, String type) {
-    for (int i = 0; i < typeTags.length; ++i)
-      if (typeTags[i][0].toLowerCase().equals(type))
-        return typeTags[i][0];
-    return null;
-  }
-  
-  ////////////////////////////////////////////////////////////////
-  // file types that need special treatment
-  ////////////////////////////////////////////////////////////////
-
-  private static boolean checkOdyssey(String[] lines) {
-    int i;
-    for (i = 0; i < lines.length; i++)
-      if (!lines[i].startsWith("C ") && lines[i].length() != 0)
-        break;
-    if (i >= lines.length 
-        || lines[i].charAt(0) != ' ' 
-        || (i = i + 2) + 1 >= lines.length)
-      return false;
-    try {
-      // distinguishing between Spartan input and MOL file
-      // MOL files have aaabbb.... on the data line
-      // SPIN files have cc s on that line (c = charge; s = spin)
-      // so the typical MOL file, with more parameters, will fail getting the spin
-      int spin = Integer.parseInt(lines[i].substring(2).trim());
-      int charge = Integer.parseInt(lines[i].substring(0, 2).trim());
-      // and if it does not, then we get the next lines of info
-      int atom1 = Integer.parseInt(lines[++i].substring(0, 2).trim());
-      if (spin < 0 || spin > 5 || atom1 <= 0 || charge > 5)
-        return false;
-      // hard to believe we would get here for a MOL file
-      float[] atomline = new float[5];
-      AtomSetCollectionReader.getTokensFloat(lines[i], atomline, 5);
-      return !Float.isNaN(atomline[1]) && !Float.isNaN(atomline[2]) && !Float.isNaN(atomline[3]) && Float.isNaN(atomline[4]);
-    } catch (Exception e) {
-    }
-    return false;
-  }
-  
-  private static boolean checkMol(String[] lines) {
-    String line4trimmed = ("X" + lines[3]).trim().toUpperCase();
-    if (line4trimmed.length() < 7 || line4trimmed.indexOf(".") >= 0)
-      return false;
-    if (line4trimmed.endsWith("V2000") || line4trimmed.endsWith("V3000"))
-      return true;
-    try {
-      int n1 = Integer.parseInt(lines[3].substring(0, 3).trim());
-      int n2 = Integer.parseInt(lines[3].substring(3, 6).trim());
-      return (n1 > 0 && n2 >= 0 && lines[0].indexOf("@<TRIPOS>") != 0
-          && lines[1].indexOf("@<TRIPOS>") != 0 
-          && lines[2].indexOf("@<TRIPOS>") != 0);
-    } catch (NumberFormatException nfe) {
-    }
-    return false;
-  }
-
   private static boolean checkAlchemy(String line) {
     /*
     11 ATOMS,    12 BONDS,     0 CHARGES
@@ -809,9 +730,48 @@ public class Resolver {
     return false;
   }
 
-  private static boolean checkXyz(String[] lines) {
+  private static boolean checkCastep(String[] lines) {
+    for ( int i = 0; i<lines.length; i++ ) {
+      if (lines[i].indexOf("Frequencies in         cm-1") == 1
+          || lines[i].contains("CASTEP")
+          || lines[i].toUpperCase().startsWith("%BLOCK LATTICE_ABC")
+          || lines[i].toUpperCase().startsWith("%BLOCK LATTICE_CART")
+          || lines[i].toUpperCase().startsWith("%BLOCK POSITIONS_FRAC")
+          || lines[i].toUpperCase().startsWith("%BLOCK POSITIONS_ABS") 
+          || lines[i].contains("<-- E")) return true;
+    }
+    return false;
+  }
+
+  private static boolean checkCrystal(String[] lines) {
+    String s = lines[1].trim();
+    if (s.equals("SLAB") ||s.equals("MOLECULE")
+        || s.equals("CRYSTAL")
+        || s.equals("POLYMER") || (s = lines[3]).equals("SLAB")
+        || s.equals("MOLECULE") || s.equals("POLYMER"))
+      return true;
+    for (int i = 0; i < lines.length; i++) {
+      if (lines[i].trim().equals("OPTGEOM"))
+        return true;
+    }
+    return false;
+  }
+  
+  private static boolean checkCube(String[] lines) {
     try {
-      Integer.parseInt(lines[0].trim());
+      StringTokenizer tokens2 = new StringTokenizer(lines[2]);
+      if (tokens2 == null || tokens2.countTokens() != 4)
+        return false;
+      Integer.parseInt(tokens2.nextToken());
+      for (int i = 3; --i >= 0; )
+        new Float(tokens2.nextToken());
+      StringTokenizer tokens3 = new StringTokenizer(lines[3]);
+      if (tokens3 == null || tokens3.countTokens() != 4)
+        return false;
+      Integer.parseInt(tokens3.nextToken());
+      for (int i = 3; --i >= 0; )
+        if ((new Float(tokens3.nextToken())).floatValue() < 0)
+          return false;
       return true;
     } catch (NumberFormatException nfe) {
     }
@@ -848,6 +808,40 @@ public class Resolver {
     return true;
   }
   
+  private static boolean checkGenNBO(String[] lines) {
+    // .31-.41 file or .nbo file
+    return (lines[1].startsWith(" Basis set information needed for plotting orbitals")
+      || lines[1].indexOf("s in the AO basis:") >= 0
+      || lines[2].indexOf(" N A T U R A L   A T O M I C   O R B I T A L") >= 0);
+  }
+  
+  private static boolean checkGromacs(String[] lines) {
+    if (Parser.parseInt(lines[1]) == Integer.MIN_VALUE)
+      return false;
+    int len = -1;
+    for (int i = 2; i < 16 && len != 0; i++)
+      if ((len = lines[i].length()) != 69 && len != 45 && len != 0)
+        return false;
+    return true;
+  }
+
+  private static boolean checkMol(String[] lines) {
+    String line4trimmed = ("X" + lines[3]).trim().toUpperCase();
+    if (line4trimmed.length() < 7 || line4trimmed.indexOf(".") >= 0)
+      return false;
+    if (line4trimmed.endsWith("V2000") || line4trimmed.endsWith("V3000"))
+      return true;
+    try {
+      int n1 = Integer.parseInt(lines[3].substring(0, 3).trim());
+      int n2 = Integer.parseInt(lines[3].substring(3, 6).trim());
+      return (n1 > 0 && n2 >= 0 && lines[0].indexOf("@<TRIPOS>") != 0
+          && lines[1].indexOf("@<TRIPOS>") != 0 
+          && lines[2].indexOf("@<TRIPOS>") != 0);
+    } catch (NumberFormatException nfe) {
+    }
+    return false;
+  }
+
   /**
    * @param lines First lines of the files.
    * @return Indicates if the file is a Mopac GRAPHF output file.
@@ -857,26 +851,50 @@ public class Resolver {
     return (lines[0].indexOf("MOPAC-Graphical data") > 2); //nAtoms MOPAC-Graphical data
   }
 
-  private static boolean checkCube(String[] lines) {
+  private static boolean checkOdyssey(String[] lines) {
+    int i;
+    for (i = 0; i < lines.length; i++)
+      if (!lines[i].startsWith("C ") && lines[i].length() != 0)
+        break;
+    if (i >= lines.length 
+        || lines[i].charAt(0) != ' ' 
+        || (i = i + 2) + 1 >= lines.length)
+      return false;
     try {
-      StringTokenizer tokens2 = new StringTokenizer(lines[2]);
-      if (tokens2 == null || tokens2.countTokens() != 4)
+      // distinguishing between Spartan input and MOL file
+      // MOL files have aaabbb.... on the data line
+      // SPIN files have cc s on that line (c = charge; s = spin)
+      // so the typical MOL file, with more parameters, will fail getting the spin
+      int spin = Integer.parseInt(lines[i].substring(2).trim());
+      int charge = Integer.parseInt(lines[i].substring(0, 2).trim());
+      // and if it does not, then we get the next lines of info
+      int atom1 = Integer.parseInt(lines[++i].substring(0, 2).trim());
+      if (spin < 0 || spin > 5 || atom1 <= 0 || charge > 5)
         return false;
-      Integer.parseInt(tokens2.nextToken());
-      for (int i = 3; --i >= 0; )
-        new Float(tokens2.nextToken());
-      StringTokenizer tokens3 = new StringTokenizer(lines[3]);
-      if (tokens3 == null || tokens3.countTokens() != 4)
-        return false;
-      Integer.parseInt(tokens3.nextToken());
-      for (int i = 3; --i >= 0; )
-        if ((new Float(tokens3.nextToken())).floatValue() < 0)
-          return false;
+      // hard to believe we would get here for a MOL file
+      float[] atomline = AtomSetCollectionReader.getTokensFloat(lines[i], null, 5);
+      return !Float.isNaN(atomline[1]) && !Float.isNaN(atomline[2]) && !Float.isNaN(atomline[3]) && Float.isNaN(atomline[4]);
+    } catch (Exception e) {
+    }
+    return false;
+  }
+  
+ private static boolean checkWien2k(String[] lines) {
+   return (lines[2].startsWith("MODE OF CALC=") 
+       || lines[2].startsWith("             RELA")
+       || lines[2].startsWith("             NREL"));
+ }
+ 
+  private static boolean checkXyz(String[] lines) {
+    try {
+      Integer.parseInt(lines[0].trim());
       return true;
     } catch (NumberFormatException nfe) {
     }
     return false;
   }
+
+
 /*
   private void dumpLines(String[] lines) {
       for (int i = 0; i < lines.length; i++) {
@@ -893,7 +911,7 @@ public class Resolver {
   // these test files that startWith one of these strings
   ////////////////////////////////////////////////////////////////
 
-  private final static int LEADER_CHAR_MAX = 20;
+  private final static int LEADER_CHAR_MAX = 10;
   
   private final static String[] cubeFileStartRecords =
   {"Cube", "JVXL", "#JVXL"};
@@ -907,8 +925,11 @@ public class Resolver {
   private final static String[] moldenFileStartRecords =
   {"Molden", "[Molden"};
 
+  private final static String[] dcdFileStartRecords =
+  {"BinaryDcd", "T\0\0\0CORD", "\0\0\0TCORD"};
+
   private final static String[][] fileStartsWithRecords =
-  { cubeFileStartRecords, mol2Records, webmoFileStartRecords, moldenFileStartRecords };
+  { cubeFileStartRecords, mol2Records, webmoFileStartRecords, moldenFileStartRecords, dcdFileStartRecords };
 
   ////////////////////////////////////////////////////////////////
   // these test lines that startWith one of these strings
@@ -917,13 +938,16 @@ public class Resolver {
   private final static String[] pqrLineStartRecords = 
   { "Pqr", "REMARK   1 PQR" };
 
+  private final static String[] p2nLineStartRecords = 
+  { "P2n", "REMARK   1 P2N" };
+
   private final static String[] pdbLineStartRecords = {
     "Pdb", "HEADER", "OBSLTE", "TITLE ", "CAVEAT", "COMPND", "SOURCE", "KEYWDS",
     "EXPDTA", "AUTHOR", "REVDAT", "SPRSDE", "JRNL  ", "REMARK ",
     "DBREF ", "SEQADV", "SEQRES", "MODRES", 
     "HELIX ", "SHEET ", "TURN  ",
     "CRYST1", "ORIGX1", "ORIGX2", "ORIGX3", "SCALE1", "SCALE2", "SCALE3",
-    "ATOM  ", "HETATM", "MODEL ",
+    "ATOM  ", "HETATM", "MODEL ", "LINK  ",
   };
 
   private final static String[] shelxLineStartRecords =
@@ -952,17 +976,28 @@ public class Resolver {
   
   private final static String[] hyperChemLineStartRecords = 
   { "HyperChem", "mol 1" };
+
+  private final static String[] vaspOutcarLineStartRecords = 
+  { "VaspOutcar", " vasp.", " INCAR:" };
   
   private final static String[][] lineStartsWithRecords =
-  { cifLineStartRecords, pqrLineStartRecords, pdbLineStartRecords, shelxLineStartRecords, 
+  { cifLineStartRecords, pqrLineStartRecords, p2nLineStartRecords,
+    pdbLineStartRecords, shelxLineStartRecords, 
     ghemicalMMLineStartRecords, jaguarLineStartRecords, 
     mdlLineStartRecords, spartanSmolLineStartRecords, csfLineStartRecords, 
-    mol2Records, mdTopLineStartRecords, hyperChemLineStartRecords };
+    mol2Records, mdTopLineStartRecords, hyperChemLineStartRecords,
+    vaspOutcarLineStartRecords
+    };
+
+  
 
   ////////////////////////////////////////////////////////////////
   // contains formats
   ////////////////////////////////////////////////////////////////
 
+  private final static String[] sptContainsRecords = 
+  { "spt", "# Jmol state" };
+  
   private final static String[] xmlContainsRecords = 
   { "Xml", "<?xml", "<atom", "<molecule", "<reaction", "<cml", "<bond", ".dtd\"",
     "<list>", "<entry", "<identifier", "http://www.xml-cml.org/schema/cml2/core" };
@@ -980,7 +1015,7 @@ public class Resolver {
   
   private final static String[] mopacContainsRecords =
   { "Mopac", "MOPAC 93 (c) Fujitsu", "MOPAC2002 (c) Fujitsu",
-    "MOPAC FOR LINUX (PUBLIC DOMAIN VERSION), ",
+    "MOPAC FOR LINUX (PUBLIC DOMAIN VERSION)",
     "MOPAC:  VERSION  6", "MOPAC   7", "MOPAC2009" };
 
   private final static String[] qchemContainsRecords = 
@@ -1004,8 +1039,14 @@ public class Resolver {
   private final static String[] dgridContainsRecords =
   { "Dgrid", "BASISFILE   created by DGrid" };
   
+  private final static String[] dmolContainsRecords =
+  { "Dmol", "DMol^3" };
+
+  private final static String[] gulpContainsRecords =
+  { "Gulp", "GENERAL UTILITY LATTICE PROGRAM" };
+  
   private final static String[] psiContainsRecords =
-  { "Psi", "    PSI  3"};
+  { "Psi", "    PSI  3", "PSI3:"};
  
   private final static String[] nwchemContainsRecords =
   { "NWChem", " argument  1 = "};
@@ -1016,12 +1057,20 @@ public class Resolver {
   private final static String[] crystalContainsRecords =
   { "Crystal", "*                                CRYSTAL"};
 
-  private final static String[][] containsRecords =
-  { xmlContainsRecords, gaussianContainsRecords, 
+  private final static String[] espressoContainsRecords =
+  { "Espresso", "Program PWSCF", "Program PHONON" }; 
+
+  private final static String[] siestaContainsRecords =
+  { "Siesta", "MD.TypeOfRun", "SolutionMethod", "MeshCutoff", 
+    "WELCOME TO SIESTA" };
+
+  private final static String[][] headerContainsRecords =
+  { sptContainsRecords, xmlContainsRecords, gaussianContainsRecords, 
     ampacContainsRecords, mopacContainsRecords, qchemContainsRecords, 
     gamessUKContainsRecords, gamessUSContainsRecords,
     spartanBinaryContainsRecords, spartanContainsRecords, mol2Records, adfContainsRecords, psiContainsRecords,
-    nwchemContainsRecords, uicrcifContainsRecords, dgridContainsRecords, crystalContainsRecords
+    nwchemContainsRecords, uicrcifContainsRecords, dgridContainsRecords, crystalContainsRecords, 
+    dmolContainsRecords, gulpContainsRecords, espressoContainsRecords, siestaContainsRecords
   };
 }
 
@@ -1066,15 +1115,8 @@ class LimitedLineReader {
       if (ch == '\r' && ichCurrent < cchBuf && buf[ichCurrent] == '\n')
         ++ichCurrent;
       int cchLine = ichCurrent - ichBeginningOfLine;
-      if (buf[ichBeginningOfLine] == '#') {// flush comment lines;
-        if (buf.length < ichBeginningOfLine + 6 || 
-            buf[ichBeginningOfLine + 1] != ' '
-            || buf[ichBeginningOfLine + 2] != 'J'
-            || buf[ichBeginningOfLine + 3] != 'm'
-            || buf[ichBeginningOfLine + 4] != 'o'
-            || buf[ichBeginningOfLine + 5] != 'l')
-        continue;
-      }
+      if (buf[ichBeginningOfLine] == '#')
+        continue; // flush comment lines;
       StringBuffer sb = new StringBuffer(cchLine);
       sb.append(buf, ichBeginningOfLine, cchLine);
       return sb.toString();
