@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-02-19 20:07:38 +0100 (sam., 19 févr. 2011) $
- * $Revision: 15202 $
+ * $Date: 2011-09-14 13:32:25 +0200 (mer., 14 sept. 2011) $
+ * $Revision: 16068 $
  *
  * Copyright (C) 2003-2005  Miguel, Jmol Development, www.jmol.org
  *
@@ -25,7 +25,8 @@
 package org.jmol.adapter.smarter;
 
 import org.jmol.api.JmolAdapter;
-import org.jmol.api.JmolFileReaderInterface;
+import org.jmol.api.JmolFilesReaderInterface;
+import org.jmol.constant.EnumStructure;
 import org.jmol.util.CompoundDocument;
 import org.jmol.util.Logger;
 import org.jmol.util.TextFormat;
@@ -36,13 +37,17 @@ import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
 import java.io.InputStream;
 import java.io.StringReader;
+import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.BitSet;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 import java.util.zip.ZipEntry;
 import java.util.zip.ZipInputStream;
 
+import javax.vecmath.Point3f;
+import javax.vecmath.Vector3f;
+
 public class SmarterJmolAdapter extends JmolAdapter {
 
   public SmarterJmolAdapter() {
@@ -66,6 +71,7 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return a file type or null
    * 
    */
+  @Override
   public String getFileTypeName(Object atomSetCollectionOrReader) {
     if (atomSetCollectionOrReader instanceof AtomSetCollection)
       return ((AtomSetCollection)atomSetCollectionOrReader).getFileTypeName();
@@ -84,8 +90,9 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return        an AtomSetCollectionReader or an error string
    * 
    */
+  @Override
   public Object getAtomSetCollectionReader(String name, String type,
-                                   BufferedReader bufferedReader, Hashtable htParams) {
+                                   BufferedReader bufferedReader, Map<String, Object> htParams) {
     try {
       Object ret = Resolver.getAtomCollectionReader(name, type,
           bufferedReader, htParams, -1);
@@ -118,6 +125,7 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return an AtomSetCollection or an error string
    * 
    */
+  @Override
   public Object getAtomSetCollection(Object atomSetCollectionReader) {
     BufferedReader br = null;
     try {
@@ -156,33 +164,43 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return a set of AtomSetCollectionReaders, a single AtomSetCollection, or an error string
    * 
    */
-  public Object getAtomSetCollectionReaders(JmolFileReaderInterface fileReader, String[] names, String[] types,
-                                    Hashtable htParams, boolean getReadersOnly) {
+  @Override
+  public Object getAtomSetCollectionReaders(JmolFilesReaderInterface fileReader,
+                                            String[] names, String[] types,
+                                            Map<String, Object> htParams,
+                                            boolean getReadersOnly) {
     //FilesOpenThread
     int size = names.length;
-    AtomSetCollectionReader[] readers = (getReadersOnly ? new AtomSetCollectionReader[size] : null);
-    AtomSetCollection[] atomsets = (getReadersOnly ? null : new AtomSetCollection[size]);
+    AtomSetCollectionReader[] readers = (getReadersOnly ? new AtomSetCollectionReader[size]
+        : null);
+    AtomSetCollection[] atomsets = (getReadersOnly ? null
+        : new AtomSetCollection[size]);
     for (int i = 0; i < size; i++) {
       try {
-        Object reader = fileReader.getBufferedReader(i);
+        Object reader = fileReader.getBufferedReaderOrBinaryDocument(i, false);
         if (!(reader instanceof BufferedReader))
           return reader;
         Object ret = Resolver.getAtomCollectionReader(names[i],
-            (types == null ? null : types[i]), (BufferedReader) reader, htParams, i);
+            (types == null ? null : types[i]), (BufferedReader) reader,
+            htParams, i);
         if (!(ret instanceof AtomSetCollectionReader))
           return ret;
-          AtomSetCollectionReader r = (AtomSetCollectionReader) ret;
-          r.setup(names[i], htParams, (BufferedReader) reader);
-          if (getReadersOnly) {
-            readers[i] = r;
-          } else {
-            ret = r.readData();
-            if (!(ret instanceof AtomSetCollection))
-              return ret;
-            atomsets[i] = (AtomSetCollection) ret;
-            if (atomsets[i].errorMessage != null)
-              return atomsets[i].errorMessage;
-          }
+        AtomSetCollectionReader r = (AtomSetCollectionReader) ret;
+        if (r.isBinary) {
+          r.setup(names[i], htParams, fileReader.getBufferedReaderOrBinaryDocument(i, true));
+        } else { 
+          r.setup(names[i], htParams, reader);
+        }
+        if (getReadersOnly) {
+          readers[i] = r;
+        } else {
+          ret = r.readData();
+          if (!(ret instanceof AtomSetCollection))
+            return ret;
+          atomsets[i] = (AtomSetCollection) ret;
+          if (atomsets[i].errorMessage != null)
+            return atomsets[i].errorMessage;
+        }
       } catch (Throwable e) {
         Logger.error(null, e);
         return "" + e;
@@ -204,8 +222,10 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return a single AtomSetCollection or an error string
    * 
    */
+  @SuppressWarnings("unchecked")
+  @Override
   public Object getAtomSetCollectionFromSet(Object readerSet, Object atomsets,
-                                            Hashtable htParams) {
+                                            Map<String, Object> htParams) {
     AtomSetCollectionReader[] readers = (AtomSetCollectionReader[]) readerSet;
     AtomSetCollection[] asc = (atomsets == null ? new AtomSetCollection[readers.length]
         : (AtomSetCollection[]) atomsets);
@@ -228,7 +248,9 @@ public class SmarterJmolAdapter extends JmolAdapter {
       // this is one model with a set of coordinates from a 
       // molecular dynamics calculation
       // all the htParams[] entries point to the same Hashtable
-      asc[0].finalizeTrajectory((List) htParams.get("trajectorySteps"));
+      asc[0].finalizeTrajectory(
+          (List<Point3f[]>) htParams.get("trajectorySteps"),
+          (List<Vector3f[]>) htParams.get("vibrationSteps"));
       return asc[0];
     }
     AtomSetCollection result = new AtomSetCollection(asc);
@@ -248,18 +270,16 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return a single atomSetCollection
    * 
    */
+  @Override
   public Object getAtomSetCollectionOrBufferedReaderFromZip(InputStream is, String fileName, String[] zipDirectory,
-                             Hashtable htParams, boolean asBufferedReader) {
-    return staticGetAtomSetCollectionOrBufferedReaderFromZip(is, fileName, zipDirectory, htParams, 1, asBufferedReader);
+                             Map<String, Object> htParams, boolean asBufferedReader, boolean asBufferedInputStream) {
+    return staticGetAtomSetCollectionOrBufferedReaderFromZip(is, fileName, zipDirectory, htParams, 1, asBufferedReader, asBufferedInputStream);
   }
 
   private static Object staticGetAtomSetCollectionOrBufferedReaderFromZip(
-                                                                          InputStream is,
-                                                                          String fileName,
-                                                                          String[] zipDirectory,
-                                                                          Hashtable htParams,
-                                                                          int subFilePtr,
-                                                                          boolean asBufferedReader) {
+                                    InputStream is, String fileName,
+                                    String[] zipDirectory, Map<String, Object> htParams,
+                                    int subFilePtr, boolean asBufferedReader, boolean asBufferedInputStream) {
 
     // we're here because user is using | in a load file name
     // or we are opening a zip file.
@@ -299,8 +319,8 @@ public class SmarterJmolAdapter extends JmolAdapter {
     boolean exceptFiles = (manifest.indexOf("EXCEPT_FILES") >= 0);
     if (selectAll || subFileName != null)
       haveManifest = false;
-    List vCollections = new ArrayList();
-    Hashtable htCollections = (haveManifest ? new Hashtable() : null);
+    List<Object> vCollections = new ArrayList<Object>();
+    Map<String, Object> htCollections = (haveManifest ? new Hashtable<String, Object>() : null);
     int nFiles = 0;
     // 0 entry is manifest
 
@@ -310,13 +330,13 @@ public class SmarterJmolAdapter extends JmolAdapter {
     // running in FileManager now.
 
     Object ret = Resolver.checkSpecialData(is, zipDirectory);
-    if (ret instanceof String)
-      return (String) ret;
+    if (ret instanceof String) {
+      return ret;
+    }
     StringBuffer data = (StringBuffer) ret;
     try {
       if (data != null) {
-        BufferedReader reader = new BufferedReader(new StringReader(data
-            .toString()));
+        BufferedReader reader = new BufferedReader(new StringReader(data.toString()));
         if (asBufferedReader) {
           return reader;
         }
@@ -359,13 +379,13 @@ public class SmarterJmolAdapter extends JmolAdapter {
           bis = new BufferedInputStream(new ByteArrayInputStream(bytes));
           Object atomSetCollections = staticGetAtomSetCollectionOrBufferedReaderFromZip(
               bis, fileName + "|" + thisEntry, zipDir2, htParams, ++subFilePtr,
-              asBufferedReader);
+              asBufferedReader, asBufferedInputStream);
           if (atomSetCollections instanceof String) {
             if (ignoreErrors)
               continue;
             return atomSetCollections;
           } else if (atomSetCollections instanceof AtomSetCollection
-              || atomSetCollections instanceof List) {
+              || atomSetCollections instanceof List<?>) {
             if (haveManifest && !exceptFiles)
               htCollections.put(thisEntry, atomSetCollections);
             else
@@ -381,6 +401,13 @@ public class SmarterJmolAdapter extends JmolAdapter {
             zis.close();
             return "unknown zip reader error";
           }
+        } else if (asBufferedInputStream){ 
+          if (ZipUtil.isGzip(bytes))
+            return ZipUtil.getGzippedInputStream(bytes);
+          BufferedInputStream bis = new BufferedInputStream(new ByteArrayInputStream(bytes));
+            if (doCombine)
+              zis.close();
+            return bis;          
         } else {
           String sData = (CompoundDocument.isCompoundDocument(bytes) ? (new CompoundDocument(
               new BufferedInputStream(new ByteArrayInputStream(bytes))))
@@ -470,7 +497,8 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * @return a single AtomSetCollection or an error string
    * 
    */
-  public Object getAtomSetCollectionFromDOM(Object DOMNode, Hashtable htParams) {
+  @Override
+  public Object getAtomSetCollectionFromDOM(Object DOMNode, Map<String, Object> htParams) {
     try {
       Object ret = Resolver.DOMResolve(DOMNode, htParams);
       if (!(ret instanceof AtomSetCollectionReader))
@@ -490,37 +518,45 @@ public class SmarterJmolAdapter extends JmolAdapter {
     }
   }
 
+  @Override
   public String[] specialLoad(String name, String type) {
     return Resolver.specialLoad(name, type);  
   }
   
+  @Override
   public void finish(Object atomSetCollection) {
     ((AtomSetCollection)atomSetCollection).finish();
   }
 
   ////////////////////////// post processing ////////////////////////////
   
+  @Override
   public String getAtomSetCollectionName(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).getCollectionName();
   }
   
-  public Hashtable getAtomSetCollectionAuxiliaryInfo(Object atomSetCollection) {
+  @Override
+  public Map<String, Object> getAtomSetCollectionAuxiliaryInfo(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).getAtomSetCollectionAuxiliaryInfo();
   }
 
+  @Override
   public int getAtomSetCount(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).getAtomSetCount();
   }
 
+  @Override
   public int getAtomSetNumber(Object atomSetCollection, int atomSetIndex) {
     return ((AtomSetCollection)atomSetCollection).getAtomSetNumber(atomSetIndex);
   }
 
+  @Override
   public String getAtomSetName(Object atomSetCollection, int atomSetIndex) {
     return ((AtomSetCollection)atomSetCollection).getAtomSetName(atomSetIndex);
   }
   
-  public Hashtable getAtomSetAuxiliaryInfo(Object atomSetCollection, int atomSetIndex) {
+  @Override
+  public Map<String, Object> getAtomSetAuxiliaryInfo(Object atomSetCollection, int atomSetIndex) {
     return ((AtomSetCollection) atomSetCollection)
         .getAtomSetAuxiliaryInfo(atomSetIndex);
   }
@@ -529,18 +565,33 @@ public class SmarterJmolAdapter extends JmolAdapter {
    * The frame related methods
    * **************************************************************/
 
+  @Override
+  public int getHydrogenAtomCount(Object atomSetCollection) {
+    return ((AtomSetCollection)atomSetCollection).getHydrogenAtomCount();
+  }
+  
+  @Override
+  public String[][] getBondList(Object atomSetCollection) {
+    return ((AtomSetCollection)atomSetCollection).getBondList();
+  }
+
+
+  @Override
   public int getAtomCount(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).getAtomCount();
   }
 
+  @Override
   public boolean coordinatesAreFractional(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).coordinatesAreFractional;
   }
 
+  @Override
   public float[] getNotionalUnitcell(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).notionalUnitCell;
   }
 
+  @Override
   public float[] getPdbScaleMatrix(Object atomSetCollection) {
     float[] a = ((AtomSetCollection)atomSetCollection).notionalUnitCell;
     if (a.length < 22)
@@ -551,6 +602,7 @@ public class SmarterJmolAdapter extends JmolAdapter {
     return b;
   }
 
+  @Override
   public float[] getPdbScaleTranslate(Object atomSetCollection) {
     float[] a = ((AtomSetCollection)atomSetCollection).notionalUnitCell;
     if (a.length < 22)
@@ -564,16 +616,19 @@ public class SmarterJmolAdapter extends JmolAdapter {
   
   ////////////////////////////////////////////////////////////////
 
+  @Override
   public JmolAdapter.AtomIterator
     getAtomIterator(Object atomSetCollection) {
     return new AtomIterator((AtomSetCollection)atomSetCollection);
   }
 
+  @Override
   public JmolAdapter.BondIterator
     getBondIterator(Object atomSetCollection) {
     return new BondIterator((AtomSetCollection)atomSetCollection);
   }
 
+  @Override
   public JmolAdapter.StructureIterator
     getStructureIterator(Object atomSetCollection) {
     return ((AtomSetCollection)atomSetCollection).getStructureCount() == 0 ? 
@@ -588,76 +643,118 @@ public class SmarterJmolAdapter extends JmolAdapter {
     private Atom atom;
     private int atomCount;
     private Atom[] atoms;
+    private BitSet bsAtoms;
 
     AtomIterator(AtomSetCollection atomSetCollection) {
       atomCount = atomSetCollection.getAtomCount();
       atoms = atomSetCollection.getAtoms();
+      bsAtoms = atomSetCollection.bsAtoms;
       iatom = 0;
     }
+    @Override
     public boolean hasNext() {
       if (iatom == atomCount)
         return false;
-      atom = atoms[iatom];
-      atoms[iatom++] = null; // single pass
+      while ((atom = atoms[iatom++]) == null || (bsAtoms != null && !bsAtoms.get(atom.atomIndex)))
+        if (iatom == atomCount)
+          return false;
+      atoms[iatom - 1] = null; // single pass
       return true;
     }
+    @Override
     public int getAtomSetIndex() { return atom.atomSetIndex; }
+    @Override
     public BitSet getAtomSymmetry() { return atom.bsSymmetry; }
+    @Override
     public int getAtomSite() { return atom.atomSite + 1; }
-    public Object getUniqueID() { return new Integer(atom.atomIndex); }
-    public String getElementSymbol() { return atom.getElementSymbol(); }
+    @Override
+    public Object getUniqueID() { return Integer.valueOf(atom.atomIndex); }
+    @Override
     public short getElementNumber() { 
       return (atom.elementNumber > 0 ?
         atom.elementNumber : JmolAdapter.getElementNumber(atom.getElementSymbol())); }
+    @Override
     public String getAtomName() { return atom.atomName; }
+    @Override
     public int getFormalCharge() { return atom.formalCharge; }
+    @Override
     public float getPartialCharge() { return atom.partialCharge; }
+    @Override
     public Object[] getEllipsoid() { return atom.ellipsoid; }
+    @Override
     public float getRadius() { return atom.radius; }
+    @Override
     public float getX() { return atom.x; }
+    @Override
     public float getY() { return atom.y; }
+    @Override
     public float getZ() { return atom.z; }
+    @Override
     public float getVectorX() { return atom.vectorX; }
+    @Override
     public float getVectorY() { return atom.vectorY; }
+    @Override
     public float getVectorZ() { return atom.vectorZ; }
+    @Override
     public float getBfactor() { return Float.isNaN(atom.bfactor) && atom.anisoBorU != null ?
         atom.anisoBorU[7] * 100f : atom.bfactor; }
+    @Override
     public int getOccupancy() { return atom.occupancy; }
+    @Override
     public boolean getIsHetero() { return atom.isHetero; }
+    @Override
     public int getAtomSerial() { return atom.atomSerial; }
+    @Override
     public char getChainID() { return canonizeChainID(atom.chainID); }
+    @Override
     public char getAlternateLocationID()
     { return canonizeAlternateLocationID(atom.alternateLocationID); }
+    @Override
     public String getGroup3() { return atom.group3; }
+    @Override
     public int getSequenceNumber() { return atom.sequenceNumber; }
+    @Override
     public char getInsertionCode()
     { return canonizeInsertionCode(atom.insertionCode); }
+    @Override
+    public Point3f getXYZ() {
+      return atom;
+    }
     
   }
 
   class BondIterator extends JmolAdapter.BondIterator {
+    private BitSet bsAtoms;
     private Bond[] bonds;
     private int ibond;
     private Bond bond;
     private int bondCount;
     
     BondIterator(AtomSetCollection atomSetCollection) {
+      bsAtoms = atomSetCollection.bsAtoms;
       bonds = atomSetCollection.getBonds();
       bondCount = atomSetCollection.getBondCount();      
       ibond = 0;
     }
+    @Override
     public boolean hasNext() {
       if (ibond == bondCount)
         return false;
-      bond = bonds[ibond++];
+      while ((bond = bonds[ibond++]) == null 
+          || (bsAtoms != null && (!bsAtoms.get(bond.atomIndex1) || !bsAtoms.get(bond.atomIndex2))))
+        if (ibond == bondCount)
+          return false;
       return true;
     }
+    @Override
     public Object getAtomUniqueID1() {
-      return new Integer(bond.atomIndex1);
+      return Integer.valueOf(bond.atomIndex1);
     }
+    @Override
     public Object getAtomUniqueID2() {
-      return new Integer(bond.atomIndex2);
+      return Integer.valueOf(bond.atomIndex2);
     }
+    @Override
     public int getEncodedOrder() {
       return bond.order;
     }
@@ -675,6 +772,7 @@ public class SmarterJmolAdapter extends JmolAdapter {
       istructure = 0;
     }
 
+    @Override
     public boolean hasNext() {
       if (istructure == structureCount)
         return false;
@@ -682,52 +780,65 @@ public class SmarterJmolAdapter extends JmolAdapter {
       return true;
     }
 
+    @Override
     public int getModelIndex() {
       return structure.modelIndex;
     }
 
-    public int getStructureType() {
+    @Override
+    public EnumStructure getStructureType() {
       return structure.structureType;
     }
 
-    public int getSubstructureType() {
+    @Override
+    public EnumStructure getSubstructureType() {
       return structure.substructureType;
     }
 
+    @Override
     public String getStructureID() {
       return structure.structureID;
     }
 
+    @Override
     public int getSerialID() {
       return structure.serialID;
     }
 
+    @Override
     public char getStartChainID() {
       return canonizeChainID(structure.startChainID);
     }
     
+    @Override
     public int getStartSequenceNumber() {
       return structure.startSequenceNumber;
     }
     
+    @Override
     public char getStartInsertionCode() {
       return canonizeInsertionCode(structure.startInsertionCode);
     }
     
+    @Override
     public char getEndChainID() {
       return canonizeChainID(structure.endChainID);
     }
     
+    @Override
     public int getEndSequenceNumber() {
       return structure.endSequenceNumber;
     }
       
+    @Override
     public char getEndInsertionCode() {
       return structure.endInsertionCode;
     }
 
+    @Override
     public int getStrandCount() {
       return structure.strandCount;
     }
   }
+
 }
