@@ -143,11 +143,18 @@ public final class EnvelopeCalculation {
   private int atomCount;
   private FastBitSet emptySet;
   
-  public EnvelopeCalculation(AtomDataServer viewer, int atomCount, short[] mads, boolean asJavaBitSet) {
+  /**
+   * 
+   * @param viewer
+   * @param atomCount
+   * @param mads
+   */
+  public EnvelopeCalculation(AtomDataServer viewer, int atomCount, short[] mads/*, boolean asJavaBitSet*/) {
     this.viewer = viewer;
     this.atomCount = atomCount; //preliminary, for setFromBits()
     this.mads = mads;
-    geodesicCount = Geodesic.getVertexVectorsCount();
+//    geodesicCount = Geodesic.getVertexVectorsCount();
+    geodesicCount = Geodesic.getVertexCount(MAX_LEVEL);
     
     //if (asJavaBitSet) {
     //  geodesicMap = SlowBitSet.allocateBitmap(geodesicCount);
@@ -163,7 +170,7 @@ public final class EnvelopeCalculation {
    
   public final static float SURFACE_DISTANCE_FOR_CALCULATION = 3f;
 
-  public final static int MAX_LEVEL = Geodesic.standardLevel;
+  public final static int MAX_LEVEL = 3;//Geodesic.standardLevel;
   
   private float maxRadius = 0;
   private boolean modelZeroBased;
@@ -267,9 +274,8 @@ public final class EnvelopeCalculation {
     atomData.modelIndex = (multiModel ? -1 : 0);
     modelZeroBased = !multiModel;
 
-    viewer.fillAtomData(atomData,
-        mads == null ? AtomData.MODE_FILL_COORDS_AND_RADII
-            : AtomData.MODE_FILL_COORDS);
+    viewer.fillAtomData(atomData, AtomData.MODE_FILL_COORDS
+        | (mads == null ? AtomData.MODE_FILL_RADII : 0));
     atomCount = atomData.atomCount;
     if (mads != null)
       for (int i = 0; i < atomCount; i++)
@@ -284,7 +290,8 @@ public final class EnvelopeCalculation {
     bsSurface = new BitSet();
     // now, calculate surface for selected atoms
     boolean isAll = (bsSelected == null);
-    AtomIndexIterator iter = viewer.getSelectedAtomIterator(bsMySelected, false, modelZeroBased);
+    AtomIndexIterator iter = viewer.getSelectedAtomIterator(bsMySelected,
+        false, modelZeroBased, false);
     //true ==> only atom index > this atom accepted
     int i0 = (isAll ? atomCount - 1 : bsSelected.nextSetBit(0));
     for (int i = i0; i >= 0; i = (isAll ? i - 1 : bsSelected.nextSetBit(i + 1)))
@@ -306,7 +313,7 @@ public final class EnvelopeCalculation {
   
   public Point3f[] getPoints() {
     if (dotsConvexMaps == null) {
-      calculate(new RadiusData(SURFACE_DISTANCE_FOR_CALCULATION, RadiusData.TYPE_ABSOLUTE, 0),
+      calculate(new RadiusData(SURFACE_DISTANCE_FOR_CALCULATION, RadiusData.EnumType.ABSOLUTE, null),
           Float.MAX_VALUE, bsMySelected, null, false, false, false, false);
     }
     if (currentPoints != null)
@@ -540,7 +547,12 @@ public final class EnvelopeCalculation {
     return iter;
   }
   
-  public void deleteAtoms(int firstAtomDeleted, int nAtomsDeleted, BitSet bsAtoms) {
+  /**
+   * 
+   * @param firstAtomDeleted
+   * @param nAtomsDeleted
+   */
+  public void deleteAtoms(int firstAtomDeleted, int nAtomsDeleted) {
     dotsConvexMaps = (FastBitSet[]) ArrayUtil.deleteElements(dotsConvexMaps, firstAtomDeleted, nAtomsDeleted);
     dotsConvexMax = dotsConvexMaps.length;
     if (mads != null)
