@@ -23,10 +23,12 @@
  */
 package org.jmol.jvxl.readers;
 
+
 import java.io.BufferedReader;
-import java.util.BitSet;
-import java.util.List;
 import java.util.ArrayList;
+import java.util.BitSet;
+import java.util.Hashtable;
+import java.util.List;
 
 import javax.vecmath.Point3f;
 import javax.vecmath.Point4f;
@@ -37,6 +39,7 @@ import org.jmol.jvxl.data.JvxlCoder;
 import org.jmol.jvxl.data.JvxlData;
 import org.jmol.jvxl.data.MeshData;
 import org.jmol.shapesurface.IsosurfaceMesh;
+import org.jmol.util.ArrayUtil;
 import org.jmol.util.ColorEncoder;
 import org.jmol.util.Escape;
 import org.jmol.util.Logger;
@@ -45,14 +48,16 @@ import org.jmol.util.XmlReader;
 
 public class JvxlXmlReader extends VolumeFileReader {
 
-  protected String JVXL_VERSION = "2.1";
+  protected String JVXL_VERSION = "2.2";
+  // 2.2 adds full support for return of rendering information, retrieval of triangle edge data when
+  //     edges have been modified by slabbing.
   
   protected int surfaceDataCount;
   protected int edgeDataCount;
   protected int colorDataCount;
   private int excludedTriangleCount;
   private int excludedVertexCount;
-  private int invalidatedVertexCount;  
+  private int invalidatedVertexCount;
   protected boolean haveContourData;
 
   private XmlReader xr;
@@ -66,9 +71,10 @@ public class JvxlXmlReader extends VolumeFileReader {
   }
 
   protected boolean thisInside;
-  
+
   /////////////reading the format///////////
 
+  @Override
   protected boolean readVolumeData(boolean isMapData) {
     if (!super.readVolumeData(isMapData))
       return false;
@@ -76,39 +82,52 @@ public class JvxlXmlReader extends VolumeFileReader {
     fractionPtr = 0;
     return true;
   }
+
+  @Override
   protected boolean gotoAndReadVoxelData(boolean isMapData) {
     initializeVolumetricData();
-    if (nPointsX < 0 || nPointsY < 0 || nPointsZ < 0) 
+    if (nPointsX < 0 || nPointsY < 0 || nPointsZ < 0)
       return true;
     try {
       gotoData(params.fileIndex - 1, nPointsX * nPointsY * nPointsZ);
       if (vertexDataOnly)
         return true;
+      volumeData.setMappingPlane(params.thePlane);
       readSurfaceData(isMapData);
+      volumeData.setMappingPlane(null);
+
       if (edgeDataCount > 0)
         jvxlEdgeDataRead = jvxlReadData("edge", edgeDataCount);
       params.bsExcluded = jvxlData.jvxlExcluded = new BitSet[4];
       if (colorDataCount > 0)
         jvxlColorDataRead = jvxlReadData("color", colorDataCount);
       if (excludedVertexCount > 0) {
-        jvxlData.jvxlExcluded[0]  
-            = JvxlCoder.jvxlDecodeBitSet(
-                xr.getXmlData("jvxlExcludedVertexData", null, false, false));
+        jvxlData.jvxlExcluded[0] = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData(
+            "jvxlExcludedVertexData", null, false, false));
         if (xr.isNext("jvxlExcludedPlaneData"))
-          jvxlData.jvxlExcluded[2]  
-                                = JvxlCoder.jvxlDecodeBitSet(
-                                    xr.getXmlData("jvxlExcludedPlaneData", null, false, false));          
+          jvxlData.jvxlExcluded[2] = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData(
+              "jvxlExcludedPlaneData", null, false, false));
       }
       if (excludedTriangleCount > 0)
-        jvxlData.jvxlExcluded[3]  
-            = JvxlCoder.jvxlDecodeBitSet(
-                xr.getXmlData("jvxlExcludedTriangleData", null, false, false));
+        jvxlData.jvxlExcluded[3] = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData(
+            "jvxlExcludedTriangleData", null, false, false));
       if (invalidatedVertexCount > 0)
-        jvxlData.jvxlExcluded[1]  
-                            = JvxlCoder.jvxlDecodeBitSet(
-                                xr.getXmlData("jvxlInvalidatedVertexData", null, false, false));
+        jvxlData.jvxlExcluded[1] = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData(
+            "jvxlInvalidatedVertexData", null, false, false));
       if (haveContourData)
-        jvxlDecodeContourData(jvxlData, xr.getXmlData("jvxlContourData", null, false, false));
+        jvxlDecodeContourData(jvxlData, xr.getXmlData("jvxlContourData", null,
+            false, false));
+      if (jvxlData.nVertexColors > 0) {
+        jvxlData.vertexColorMap = new Hashtable<String, BitSet>();
+        for (int i = 0; i < jvxlData.nVertexColors; i++) {
+          String s = xr.getXmlData("jvxlColorMap", null, true, false);
+          String color = XmlReader.getXmlAttrib(s, "color");
+          BitSet bs = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData("jvxlColorMap",
+              s, false, false));
+          jvxlData.vertexColorMap.put(color, bs);
+        }
+      }
+
     } catch (Exception e) {
       Logger.error(e.toString());
       return false;
@@ -118,6 +137,7 @@ public class JvxlXmlReader extends VolumeFileReader {
   
   String tempDataXml; 
   
+  @Override
   protected void readParameters() throws Exception {
     String s = xr.getXmlData("jvxlFileTitle", null, false, false);
     jvxlFileHeaderBuffer = new StringBuffer(s);
@@ -148,6 +168,7 @@ public class JvxlXmlReader extends VolumeFileReader {
       setVectorAnisotropy(volumetricVectors[voxelVectorIndex]);
   }
 
+  @Override
   protected void gotoData(int n, int nPoints) throws Exception {
     if (n > 0)
       Logger.info("skipping " + n + " data sets, " + nPoints + " points each");
@@ -159,7 +180,8 @@ public class JvxlXmlReader extends VolumeFileReader {
     jvxlReadSurfaceInfo();
   }
 
-  protected void jvxlSkipData(int nPoints, boolean doSkipColorData)
+  protected void jvxlSkipData(@SuppressWarnings("unused") int nPoints,
+                              @SuppressWarnings("unused") boolean doSkipColorData)
       throws Exception {
     readLine();
     xr.skipTag("jvxlSurface");
@@ -196,13 +218,29 @@ public class JvxlXmlReader extends VolumeFileReader {
       params.nContours = (haveContourData ? nContourData : nContoursRead);
       //TODO ? params.contourFromZero = false; // MEP data to complete the plane
     }
+    jvxlData.nVertexColors = parseInt(XmlReader.getXmlAttrib(data, "nVertexColors"));
     params.isBicolorMap = XmlReader.getXmlAttrib(data, "bicolorMap").equals("true");
+    if (params.isBicolorMap) {
+      // TODO -- not quite right, because
+      s = XmlReader.getXmlAttrib(data, "colorPositive");
+      if (s.length() > 0 && params.colorRgb == Integer.MIN_VALUE 
+          && params.colorPos == Parameters.defaultColorPositive)
+        params.colorPos = Graphics3D.getArgbFromString(s);
+      s = XmlReader.getXmlAttrib(data, "colorNegative");
+      if (s.length() > 0 && params.colorRgb == Integer.MIN_VALUE
+          && params.colorNeg == Parameters.defaultColorNegative)
+        params.colorNeg = Graphics3D.getArgbFromString(s);
+    }
     if (params.isBicolorMap || params.colorBySign)
       jvxlCutoff = 0;
-    jvxlDataIsColorMapped = params.isBicolorMap || XmlReader.getXmlAttrib(data, "colorMapped").equals("true");
+    jvxlDataIsColorMapped = 
+      ((params.colorRgb == Integer.MIN_VALUE || params.colorRgb == Integer.MAX_VALUE)
+    && (params.isBicolorMap || XmlReader.getXmlAttrib(data, "colorMapped").equals("true")));
     //next is for information only -- will be superceded by "encoding" attribute of jvxlColorData
     jvxlData.isJvxlPrecisionColor = XmlReader.getXmlAttrib(data, "precisionColor").equals("true");
-    jvxlData.colorDensity = params.colorDensity = XmlReader.getXmlAttrib(data, "colorDensity").equals("true");
+    jvxlData.jvxlDataIsColorDensity = params.colorDensity = (params.colorRgb == Integer.MIN_VALUE && XmlReader.getXmlAttrib(data, "colorDensity").equals("true"));
+    s = XmlReader.getXmlAttrib(data, "allowVolumeRender");
+      jvxlData.allowVolumeRender = params.allowVolumeRender = (s.length() == 0 || s.equalsIgnoreCase("true"));
     s = XmlReader.getXmlAttrib(data, "plane");
     if (s.indexOf("{") >= 0) {
       try {
@@ -227,8 +265,34 @@ public class JvxlXmlReader extends VolumeFileReader {
     excludedVertexCount = parseInt(XmlReader.getXmlAttrib(data, "nExcludedVertexes"));
     excludedTriangleCount = parseInt(XmlReader.getXmlAttrib(data, "nExcludedTriangles"));
     invalidatedVertexCount = parseInt(XmlReader.getXmlAttrib(data, "nInvalidatedVertexes"));
+    s = XmlReader.getXmlAttrib(data, "slabInfo");
+    if (s.length() > 0)
+      jvxlData.slabInfo = s;
     colorDataCount = Math.max(0, parseInt(XmlReader.getXmlAttrib(data, "nBytesUncompressedColorData")));
     jvxlDataIs2dContour = (params.thePlane != null && jvxlDataIsColorMapped);
+
+    // new Jmol 12.1.50
+    jvxlData.color = XmlReader.getXmlAttrib(data, "color");
+    if (jvxlData.color.length() == 0 || jvxlData.color.indexOf("null") >= 0)
+      jvxlData.color = "orange";
+    jvxlData.translucency = parseFloat(XmlReader.getXmlAttrib(data, "translucency"));
+    if (Float.isNaN(jvxlData.translucency))
+      jvxlData.translucency = 0;
+    s = XmlReader.getXmlAttrib(data, "meshColor");
+    if (s.length() > 0)
+      jvxlData.meshColor = s;
+    s = XmlReader.getXmlAttrib(data, "rendering");
+    if (s.length() > 0)
+      jvxlData.rendering = s;
+    jvxlData.colorScheme = XmlReader.getXmlAttrib(data, "colorScheme");
+    if (jvxlData.colorScheme.length() == 0)
+      jvxlData.colorScheme = null;
+    jvxlData.slabValue = parseInt(XmlReader.getXmlAttrib(data, "slabValue"));    
+    jvxlData.isSlabbable = (XmlReader.getXmlAttrib(data, "slabbable").equalsIgnoreCase("true"));    
+    jvxlData.diameter = parseInt(XmlReader.getXmlAttrib(data, "diameter"));
+    if (jvxlData.diameter == Integer.MIN_VALUE)
+      jvxlData.diameter = 0;
+    
     if (jvxlDataIs2dContour)
       params.isContoured = true;
     
@@ -296,6 +360,7 @@ public class JvxlXmlReader extends VolumeFileReader {
     params.insideOut = jvxlData.insideOut;
   }
 
+  @Override
   protected void readSurfaceData(boolean isMapDataIgnored) throws Exception {
     thisInside = !params.isContoured;
     if (readSurfaceData())
@@ -327,9 +392,11 @@ public class JvxlXmlReader extends VolumeFileReader {
   
   protected void readVolumeFileSurfaceData() throws Exception {
     super.readSurfaceData(false);
+    volumeData.setMappingPlane(null);
   }
 
-  protected String jvxlReadData(String type, int nPoints) {
+  protected String jvxlReadData(String type,
+                                @SuppressWarnings("unused") int nPoints) {
     String str;
     try {
       if (type.equals("edge")) {
@@ -348,6 +415,7 @@ public class JvxlXmlReader extends VolumeFileReader {
   
   protected BitSet bsVoxelBitSet;
 
+  @Override
   protected BitSet getVoxelBitSet(int nPoints) throws Exception {
     if (bsVoxelBitSet != null)
       return bsVoxelBitSet;
@@ -378,28 +446,27 @@ public class JvxlXmlReader extends VolumeFileReader {
     }
     return bs;
   }
-  
+
+  @Override
   protected float getSurfacePointAndFraction(float cutoff,
                                              boolean isCutoffAbsolute,
                                              float valueA, float valueB,
                                              Point3f pointA,
                                              Vector3f edgeVector,
-                                             float[] fReturn, Point3f ptReturn) {
+                                             int x, int y, int z, int vA, int vB, float[] fReturn, Point3f ptReturn) {
     if (edgeDataCount <= 0)
       return super.getSurfacePointAndFraction(cutoff, isCutoffAbsolute, valueA,
-          valueB, pointA, edgeVector, fReturn, ptReturn);
+          valueB, pointA, edgeVector, x, y, z, vA, vB, fReturn, ptReturn);
     ptReturn.scaleAdd(fReturn[0] = jvxlGetNextFraction(edgeFractionBase,
         edgeFractionRange, 0.5f), edgeVector, pointA);
     if (Float.isNaN(valueMin))
-      setValueMinMax();
+      setValueMinMax();      
     return (valueCount == 0 || includeValueNaN && Float.isNaN(fReturn[0]) 
         ? fReturn[0] : getNextValue());
   }
 
   private float getNextValue() {
     float fraction = Float.NaN;
-    if (colorPtr >= valueCount)
-      System.out.println("JvxlXmlREader test 1243 " + colorPtr + " " + valueCount);
     while (colorPtr < valueCount && Float.isNaN(fraction)) {
       if (jvxlData.isJvxlPrecisionColor) {
         // this COULD be an option for mapped surfaces; 
@@ -459,6 +526,7 @@ public class JvxlXmlReader extends VolumeFileReader {
 
   boolean haveReadColorData;
   
+  @Override
   protected String readColorData() {
     // overloads SurfaceReader
     // standard jvxl file read for color 
@@ -511,7 +579,7 @@ public class JvxlXmlReader extends VolumeFileReader {
     boolean needContourMinMax = (params.mappedDataMin == Float.MAX_VALUE);
     for (int i = 0; i < vertexCount; i += vertexIncrement) {
       float value;
-      if(getValues)
+      if (getValues)
         value = vertexValues[i] = getNextValue();
       else
         value = vertexValues[i];
@@ -526,25 +594,26 @@ public class JvxlXmlReader extends VolumeFileReader {
       params.mappedDataMin = contourPlaneMinimumValue;
       params.mappedDataMax = contourPlaneMaximumValue;
     }
-
-    for (int i = 0; i < vertexCount; i += vertexIncrement) {
-      float value = vertexValues[i];
-      //note: these are just default colorings
-      //orbital color had a bug through 11.2.6/11.3.6
-      if (marchingSquares != null && params.isContoured) {
-        marchingSquares.setContourData(i, value);
-      } else if (params.colorBySign) {
-        colixes[i] = ((params.isColorReversed ? value > 0 : value <= 0) ? colixNeg
-            : colixPos);
-      } else {
-        colixes[i] = params.colorEncoder.getColorIndex(value);
+    if (jvxlData.colorScheme != null)
+      for (int i = 0; i < vertexCount; i += vertexIncrement) {
+        float value = vertexValues[i];
+        //note: these are just default colorings
+        //orbital color had a bug through 11.2.6/11.3.6
+        if (marchingSquares != null && params.isContoured) {
+          marchingSquares.setContourData(i, value);
+          continue;
+        }
+        short colix = (!params.colorBySign ? params.colorEncoder
+            .getColorIndex(value) : (params.isColorReversed ? value > 0
+            : value <= 0) ? colixNeg : colixPos);
+        colixes[i] = Graphics3D.getColixTranslucent(colix, true,
+            jvxlData.translucency);
       }
-    }
     return jvxlColorDataRead + "\n";
   }
 
   /**
-   * retrieve Jvxl 2.0 format vertex/triangle/color data found
+   * retrieve Jvxl 2.0 format vertex/triangle/edge/color data found
    * within <jvxlSurfaceData> element 
    * 
    * @throws Exception
@@ -552,16 +621,17 @@ public class JvxlXmlReader extends VolumeFileReader {
   protected void getEncodedVertexData() throws Exception {
     String data = xr.getXmlData("jvxlSurfaceData", null, true, false);
     String tData = xr.getXmlData("jvxlTriangleData", data, true, false);
+    String edgeData = xr.getXmlData("jvxlTriangleEdgeData", data, true, false);
     jvxlDecodeVertexData(xr.getXmlData("jvxlVertexData", data, true, false), false);
     String polygonColorData = xr.getXmlData("jvxlPolygonColorData", data, false, false);
-    jvxlDecodeTriangleData(tData, polygonColorData);
+    jvxlDecodeTriangleData(tData, edgeData, polygonColorData);
     Logger.info("Checking for vertex values");
     data = xr.getXmlData("jvxlColorData", data, true, false);
     jvxlData.isJvxlPrecisionColor = XmlReader.getXmlAttrib(data, "encoding").endsWith("2");
     jvxlColorDataRead = JvxlCoder.jvxlUncompressString(XmlReader.getXmlAttrib(data, "data"));
     if (jvxlColorDataRead.length() == 0)
       jvxlColorDataRead = xr.getXmlData("jvxlColorData", data, false, false);
-    jvxlDataIsColorMapped = (jvxlColorDataRead.length() > 0);
+    jvxlDataIsColorMapped = ((params.colorRgb == Integer.MIN_VALUE || params.colorRgb == Integer.MAX_VALUE) && jvxlColorDataRead.length() > 0);
     if (haveContourData)
       jvxlDecodeContourData(jvxlData, xr.getXmlData("jvxlContourData", null, false, false));
   }
@@ -616,26 +686,32 @@ public class JvxlXmlReader extends VolumeFileReader {
    * 
    * @param data
    *          tag and contents
+   * @param edgeData 
    * @param colorData
    * @return int[][] if desired
    * @throws Exception
    */
-  int[][] jvxlDecodeTriangleData(String data, String colorData)
+  int[][] jvxlDecodeTriangleData(String data, String edgeData, String colorData)
       throws Exception {
     int nColors = (colorData == null ? -1 : 0);
     int color = 0;
     int nData = parseInt(XmlReader.getXmlAttrib(data, "count"));
+    if (nData < 0)
+      return null;
     Logger.info("Reading " + nData + " triangles");
     int[][] triangles = null;
     int[] triangle = new int[3];
     String s = JvxlCoder.jvxlUncompressString(XmlReader.getXmlAttrib(data, "data"));
     if (s.length() == 0)
       s = xr.getXmlData("jvxlTriangleData", data, false, false);
+    String sEdge = JvxlCoder.jvxlUncompressString(XmlReader.getXmlAttrib(edgeData, "data")).trim();
     int[] nextp = new int[1];
     int[] nextc = new int[1];
+    int edgeMask = 7;
+    boolean haveEdgeInfo = (sEdge.length() == nData);
     int ilast = 0;
     int p = 0;
-    int b0 = (int) '\\';
+    int b0 = '\\';
     for (int i = 0, pt = -1; i < nData;) {
       char ch = s.charAt(++pt);
       int idiff;
@@ -667,13 +743,16 @@ public class JvxlXmlReader extends VolumeFileReader {
         pt = nextp[0] - 1;
         break;
       default:
-        idiff = (int) ch - b0;
+        idiff = ch - b0;
       }
       ilast += idiff;
       triangle[p] = ilast;
       if (++p % 3 == 0) {
-        i++;
-        p = 0;
+        if(haveEdgeInfo) {
+          edgeMask = sEdge.charAt(i) - '0';
+          if (edgeMask < 0 || edgeMask > 7)
+            edgeMask = 7;
+        }
         if (nColors >= 0) {
           if (nColors == 0) {
             nColors = Parser.parseInt(colorData, nextc);
@@ -683,16 +762,17 @@ public class JvxlXmlReader extends VolumeFileReader {
           }
           nColors--;
         }
-        addTriangleCheck(triangle[0], triangle[1], triangle[2], 7, 0, false,
+        addTriangleCheck(triangle[0], triangle[1], triangle[2], edgeMask, 0, false,
             color);
+        i++;
+        p = 0;
       }
     }
     return triangles;
   }
 
-  protected void jvxlDecodeContourData(JvxlData jvxlData, String data)
-      throws Exception {
-    List vs = new ArrayList();
+  protected void jvxlDecodeContourData(JvxlData jvxlData, String data) throws Exception {
+    List<List<Object>> vs = new ArrayList<List<Object>>();
     StringBuffer values = new StringBuffer();
     StringBuffer colors = new StringBuffer();
     int pt = -1;
@@ -700,31 +780,29 @@ public class JvxlXmlReader extends VolumeFileReader {
     if (data == null)
       return;
     while ((pt = data.indexOf("<jvxlContour", pt + 1)) >= 0) {
-      List v = new ArrayList();
+      List<Object> v = new ArrayList<Object>();
       String s = xr.getXmlData("jvxlContour", data.substring(pt), true, false);
       float value = parseFloat(XmlReader.getXmlAttrib(s, "value"));
       values.append(" ").append(value);
-      short colix = Graphics3D.getColix(Graphics3D.getArgbFromString(XmlReader
-          .getXmlAttrib(s, "color")));
+      short colix = Graphics3D.getColix(Graphics3D.getArgbFromString(XmlReader.getXmlAttrib(s,
+          "color")));
       int color = Graphics3D.getArgb(colix);
       colors.append(" ").append(Escape.escapeColor(color));
-      String fData = JvxlCoder.jvxlUncompressString(XmlReader.getXmlAttrib(s,
-          "data"));
-      BitSet bs = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData("jvxlContour", s,
-          false, false));
+      String fData = JvxlCoder.jvxlUncompressString(XmlReader.getXmlAttrib(s, "data"));
+      BitSet bs = JvxlCoder.jvxlDecodeBitSet(xr.getXmlData("jvxlContour", s, false, false));
       int n = bs.length();
-      IsosurfaceMesh.setContourVector(v, n, bs, value, colix, color,
-          new StringBuffer(fData));
+      IsosurfaceMesh.setContourVector(v, n, bs, value, colix, color, new StringBuffer(
+          fData));
       vs.add(v);
     }
     int n = vs.size();
     if (n > 0) {
-      jvxlData.vContours = new ArrayList[n];
+      jvxlData.vContours = ArrayUtil.createArrayOfArrayList(n);
       // 3D contour values and colors
       jvxlData.contourColixes = params.contourColixes = new short[n];
       jvxlData.contourValues = params.contoursDiscrete = new float[n];
       for (int i = 0; i < n; i++) {
-        jvxlData.vContours[i] = (List) vs.get(i);
+        jvxlData.vContours[i] = vs.get(i);
         jvxlData.contourValues[i] = ((Float) jvxlData.vContours[i].get(2))
             .floatValue();
         jvxlData.contourColixes[i] = ((short[]) jvxlData.vContours[i].get(3))[0];
@@ -735,7 +813,8 @@ public class JvxlXmlReader extends VolumeFileReader {
       Logger.info("JVXL read: contour colors: " + colors);
     }
   }
-
+  
+  @Override
   protected void postProcessVertices() {
     BitSet bsInvalid = params.bsExcluded[1]; 
     if (bsInvalid != null) {
@@ -749,4 +828,5 @@ public class JvxlXmlReader extends VolumeFileReader {
       updateTriangles();
     }
   }
+
 }
