@@ -24,8 +24,8 @@
 
 package org.jmol.minimize.forcefield;
 
-import java.util.List;
 import java.util.ArrayList;
+import java.util.List;
 
 import org.jmol.minimize.MinAtom;
 import org.jmol.minimize.MinBond;
@@ -70,7 +70,7 @@ class CalculationsUFF extends Calculations {
     
   CalculationsUFF(ForceField ff, MinAtom[] minAtoms, MinBond[] minBonds, 
       int[][] angles, int[][] torsions, double[] partialCharges,
-      List constraints) {
+      List<Object[]> constraints) {
     super(ff, minAtoms, minBonds, angles, torsions, partialCharges, constraints);    
     bondCalc = new DistanceCalc();
     angleCalc = new AngleCalc();
@@ -80,16 +80,18 @@ class CalculationsUFF extends Calculations {
     esCalc = new ESCalc();
   }
   
+  @Override
   String getUnit() {
     return "kJ/mol"; // Note that we convert from kcal/mol internally
   }
 
+  @Override
   boolean setupCalculations() {
 
-    List calc;
+    List<Object[]> calc;
 
     DistanceCalc distanceCalc = new DistanceCalc();
-    calc = calculations[CALC_DISTANCE] = new ArrayList();
+    calc = calculations[CALC_DISTANCE] = new ArrayList<Object[]>();
     for (int i = 0; i < bondCount; i++) {
       MinBond bond = bonds[i];
       double bondOrder = bond.atomIndexes[2];
@@ -100,17 +102,17 @@ class CalculationsUFF extends Calculations {
       distanceCalc.setData(calc, bond.atomIndexes[0], bond.atomIndexes[1], bondOrder);
     }
 
-    calc = calculations[CALC_ANGLE] = new ArrayList();
+    calc = calculations[CALC_ANGLE] = new ArrayList<Object[]>();
     AngleCalc angleCalc = new AngleCalc();
     for (int i = angles.length; --i >= 0;)
       angleCalc.setData(calc, i);
 
-    calc = calculations[CALC_TORSION] = new ArrayList();
+    calc = calculations[CALC_TORSION] = new ArrayList<Object[]>();
     TorsionCalc torsionCalc = new TorsionCalc();
     for (int i = torsions.length; --i >= 0;)
       torsionCalc.setData(calc, i);
 
-    calc = calculations[CALC_OOP] = new ArrayList();
+    calc = calculations[CALC_OOP] = new ArrayList<Object[]>();
     // set up the special atom arrays
     OOPCalc oopCalc = new OOPCalc();
     int elemNo;
@@ -120,7 +122,7 @@ class CalculationsUFF extends Calculations {
         oopCalc.setData(calc, i, elemNo);
     }
 
-    pairSearch(calculations[CALC_VDW] = new ArrayList(), new VDWCalc());
+    pairSearch(calculations[CALC_VDW] = new ArrayList<Object[]>(), new VDWCalc());
 
     return true;
   }
@@ -140,7 +142,7 @@ class CalculationsUFF extends Calculations {
     }
   }
 
-  private void pairSearch(List calc, PairCalc type) {
+  private void pairSearch(List<Object[]> calc, PairCalc type) {
     /*A:*/ for (int i = 0; i < atomCount - 1; i++) { // one atom...
       MinAtom atomA = atoms[i];
       int[] atomList1 = atomA.getBondedAtomIndexes();
@@ -177,6 +179,7 @@ class CalculationsUFF extends Calculations {
     }
   }
 
+  @Override
   boolean setupElectrostatics() {
 
     // Note that while the UFF paper mentions an electrostatic term,
@@ -186,7 +189,7 @@ class CalculationsUFF extends Calculations {
     if (partialCharges == null)
       return true;
 
-    pairSearch(calculations[CALC_ES] = new ArrayList(), new ESCalc());
+    pairSearch(calculations[CALC_ES] = new ArrayList<Object[]>(), new ESCalc());
     return true;
   }
 
@@ -205,6 +208,7 @@ class CalculationsUFF extends Calculations {
     return (ri + rj + rbo - ren);
   }
 
+  @Override
   double compute(int iType, Object[] dataIn) {
 
     switch (iType) {
@@ -228,7 +232,7 @@ class CalculationsUFF extends Calculations {
 
     double r0, kb;
 
-    void setData(List calc, int ia, int ib, double bondOrder) {
+    void setData(List<Object[]> calc, int ia, int ib, double bondOrder) {
       parA = getParameter(atoms[ia].type, ffParams);
       parB = getParameter(atoms[ib].type, ffParams);
       r0 = calculateR0(parA.dVal[PAR_R], parB.dVal[PAR_R], parA.dVal[PAR_XI],
@@ -242,6 +246,7 @@ class CalculationsUFF extends Calculations {
           new double[] { r0, kb, bondOrder } });
     }
 
+    @Override
     double compute(Object[] dataIn) {
       getPointers(dataIn);
       r0 = dData[0];
@@ -280,8 +285,8 @@ class CalculationsUFF extends Calculations {
   
   class AngleCalc extends Calculation {
   
-    void setData(List calc, int i) {
-      int[] angle = (int[]) angles[i];
+    void setData(List<Object[]> calc, int i) {
+      int[] angle = angles[i];
       a = atoms[ia = angle[0]];
       b = atoms[ib = angle[1]];
       c = atoms[ic = angle[2]];
@@ -339,6 +344,7 @@ class CalculationsUFF extends Calculations {
           new double[] { ka, c0 - c2, c1, 2 * c2, theta0 * RAD_TO_DEG, preliminaryMagnification * ka } });
     }
 
+    @Override
     double compute(Object[] dataIn) {
       
       getPointers(dataIn);
@@ -418,7 +424,7 @@ class CalculationsUFF extends Calculations {
 
   class TorsionCalc extends Calculation {
 
-   void setData(List calc, int i) {
+   void setData(List<Object[]> calc, int i) {
       int[] t = torsions[i];
       double cosNPhi0 = -1; // n * phi0 = 180; max at 0 
       int n = 0;
@@ -520,6 +526,7 @@ class CalculationsUFF extends Calculations {
     }
 
     
+    @Override
     double compute(Object[] dataIn) {
        
       getPointers(dataIn);
@@ -568,7 +575,7 @@ class CalculationsUFF extends Calculations {
   
   class OOPCalc extends Calculation {
 
-    void setData(List calc, int ib, int elemNo) {
+    void setData(List<Object[]> calc, int ib, int elemNo) {
 
       // The original Rappe paper in JACS isn't very clear about the parameters
       // The following was adapted from Towhee
@@ -752,6 +759,7 @@ class CalculationsUFF extends Calculations {
           new double[] { koop, a0, a1, a2, koop * 10 } });
     }
 
+    @Override
     double compute(Object[] dataIn) {
 
       getPointers(dataIn);
@@ -812,13 +820,14 @@ class CalculationsUFF extends Calculations {
 
   abstract class PairCalc extends Calculation {
    
-    abstract void setData(List calc, int ia, int ib);
+    abstract void setData(List<Object[]> calc, int ia, int ib);
 
   }
   
   class VDWCalc extends PairCalc {
     
-    void setData(List calc, int ia, int ib) {
+    @Override
+    void setData(List<Object[]> calc, int ia, int ib) {
       a = atoms[ia];
       b = atoms[ib];
       
@@ -846,6 +855,7 @@ class CalculationsUFF extends Calculations {
           new double[] { Xab, Dab } });
     }
 
+    @Override
     double compute(Object[] dataIn) {
 
       getPointers(dataIn);
@@ -890,7 +900,8 @@ class CalculationsUFF extends Calculations {
   
   class ESCalc extends PairCalc {
 
-    void setData(List calc, int ia, int ib) {
+    @Override
+    void setData(List<Object[]> calc, int ia, int ib) {
       a = atoms[ia];
       b = atoms[ib];
       double qq = KCAL332 * partialCharges[ia]
@@ -901,6 +912,7 @@ class CalculationsUFF extends Calculations {
             new double[] { qq } });
     }
 
+    @Override
     double compute(Object[] dataIn) {
       
       getPointers(dataIn);
@@ -936,6 +948,7 @@ class CalculationsUFF extends Calculations {
   
   ///////// REPORTING /////////////
   
+  @Override
   String getAtomList(String title) {
     String trailer =
           "----------------------------------------"
@@ -965,6 +978,7 @@ class CalculationsUFF extends Calculations {
     return sb.toString();
   }
 
+  @Override
   String getDebugHeader(int iType) {
     switch (iType){
     case -1:
@@ -985,9 +999,9 @@ class CalculationsUFF extends Calculations {
     case CALC_TORSION:
       return 
            "\nT O R S I O N A L (" + torsions.length + " torsions)\n\n"
-          +"      ATOMS           ATOM TYPES             FORCE      TORSION\n"
-          +"  I   J   K   L   I     J     K     L       CONSTANT     ANGLE        ENERGY\n"
-          +"----------------------------------------------------------------------------";
+          +"      ATOMS           ATOM TYPES            n    COS          FORCE      TORSION\n"
+          +"  I   J   K   L   I     J     K     L          (n phi0)      CONSTANT     ANGLE        ENERGY\n"
+          +"---------------------------------------------------------------------------------------------";
     case CALC_OOP:
       return 
            "\nO U T - O F - P L A N E   B E N D I N G\n\n"
@@ -1031,13 +1045,13 @@ class CalculationsUFF extends Calculations {
               atoms[c.ic].atom.getAtomNumber()} });
       case CALC_TORSION:
       return TextFormat.sprintf(
-          "%3d %3d %3d %3d  %-5s %-5s %-5s %-5s  %8.3f     %8.3f     %8.3f", 
+          "%3d %3d %3d %3d  %-5s %-5s %-5s %-5s  %3d %8.3f     %8.3f     %8.3f     %8.3f", 
           new Object[] { atoms[c.ia].type, atoms[c.ib].type, 
-              atoms[c.ic].type, atoms[c.id].type,
-          new float[] { (float) c.dData[0]/*V*/, 
+              atoms[c.ic].type, atoms[c.id].type, 
+          new float[] { (float) c.dData[1]/*cosNphi0*/, (float) c.dData[0]/*V*/, 
               (float) (c.theta * RAD_TO_DEG), (float) c.energy },
           new int[] { atoms[c.ia].atom.getAtomNumber(), atoms[c.ib].atom.getAtomNumber(),
-              atoms[c.ic].atom.getAtomNumber(), atoms[c.id].atom.getAtomNumber() } });
+              atoms[c.ic].atom.getAtomNumber(), atoms[c.id].atom.getAtomNumber(), c.iData[4] } });
     case CALC_OOP:
       return TextFormat.sprintf("" +
           "%3d %3d %3d %3d  %-5s %-5s %-5s %-5s  %8.3f   %8.3f     %8.3f",
@@ -1061,6 +1075,7 @@ class CalculationsUFF extends Calculations {
     return "";
   }
 
+  @Override
   String getDebugFooter(int iType, double energy) {
     String s = "";
     switch (iType){
@@ -1084,7 +1099,7 @@ class CalculationsUFF extends Calculations {
       break;
     }
     return TextFormat.sprintf("\n     TOTAL %s ENERGY = %8.3f %s\n", 
-        new Object[] { s, getUnit(), new Float(energy) });
+        new Object[] { s, getUnit(), Float.valueOf((float) energy) });
   }
 
 }
