@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-03-25 05:19:52 +0100 (ven., 25 mars 2011) $
- * $Revision: 15342 $
+ * $Date: 2011-11-05 17:22:54 +0100 (sam., 05 nov. 2011) $
+ * $Revision: 16457 $
 
  *
  * Copyright (C) 2003-2005  The Jmol Development Team
@@ -25,15 +25,19 @@
 
 package org.jmol.modelset;
 
+
 import org.jmol.viewer.JmolConstants;
 import org.jmol.script.Token;
 import org.jmol.viewer.Viewer;
-import org.jmol.api.JmolEdge;
-import org.jmol.api.JmolNode;
 import org.jmol.api.SymmetryInterface;
 import org.jmol.atomdata.RadiusData;
+import org.jmol.constant.EnumPalette;
+import org.jmol.constant.EnumStructure;
+import org.jmol.constant.EnumVdw;
 import org.jmol.g3d.Graphics3D;
 import org.jmol.util.Elements;
+import org.jmol.util.JmolEdge;
+import org.jmol.util.JmolNode;
 import org.jmol.util.Point3fi;
 
 import java.util.BitSet;
@@ -53,12 +57,16 @@ final public class Atom extends Point3fi implements JmolNode {
 
   Group group;
   private BitSet atomSymmetry;
+  public void setAtomSymmetry(BitSet bsSymmetry) {
+    atomSymmetry = bsSymmetry;
+  }
+
   int atomSite;
   private float userDefinedVanDerWaalRadius;
   
   private short atomicAndIsotopeNumber;
   private byte formalChargeAndFlags;
-  private byte valence;
+  byte valence;
   char alternateLocationID;
   public byte atomID;
   public byte getAtomID() {
@@ -68,10 +76,15 @@ final public class Atom extends Point3fi implements JmolNode {
   public short madAtom;
 
   short colixAtom;
-  byte paletteID = JmolConstants.PALETTE_CPK;
+  byte paletteID = EnumPalette.CPK.id;
 
   Bond[] bonds;
   
+  /**
+   * 
+   * @return  bonds -- WHICH MAY BE NULL
+   * 
+   */
   public Bond[] getBonds() {
     return bonds;
   }
@@ -89,6 +102,7 @@ final public class Atom extends Point3fi implements JmolNode {
   
   int clickabilityFlags;
   int shapeVisibilityFlags;
+  public static final int BACKBONE_VISIBILITY_FLAG = JmolConstants.getShapeVisibilityFlag(JmolConstants.SHAPE_BACKBONE);
 
   public Atom(int modelIndex, int atomIndex,
         float x, float y, float z, float radius,
@@ -112,14 +126,23 @@ final public class Atom extends Point3fi implements JmolNode {
     shapeVisibilityFlags = flag;
   }
 
-  public final void setShapeVisibility(int shapeVisibilityFlag, boolean isVisible) {
+  public final void setShapeVisibility(int flag, boolean isVisible) {
     if(isVisible) {
-      shapeVisibilityFlags |= shapeVisibilityFlag;        
+      shapeVisibilityFlags |= flag;        
     } else {
-      shapeVisibilityFlags &=~shapeVisibilityFlag;
+      shapeVisibilityFlags &=~flag;
     }
   }
   
+  public boolean isCovalentlyBonded(Atom atomOther) {
+    if (bonds != null)
+      for (int i = bonds.length; --i >= 0;)
+        if (bonds[i].isCovalent() 
+            && bonds[i].getOtherAtom(this) == atomOther)
+          return true;
+    return false;
+  }
+
   public boolean isBonded(Atom atomOther) {
     if (bonds != null)
       for (int i = bonds.length; --i >= 0;)
@@ -136,13 +159,13 @@ final public class Atom extends Point3fi implements JmolNode {
     return null;
   }
 
-  void addDisplayedBond(int stickVisibilityFlag, boolean isVisible){
-    nBondsDisplayed+=(isVisible ? 1 : -1);
-    setShapeVisibility(stickVisibilityFlag, isVisible);
+  void addDisplayedBond(int stickVisibilityFlag, boolean isVisible) {
+    nBondsDisplayed += (isVisible ? 1 : -1);
+    setShapeVisibility(stickVisibilityFlag, (nBondsDisplayed > 0));
   } 
   
-  public void addDisplayedBackbone(int backboneVisibilityFlag, boolean isVisible){
-    nBackbonesDisplayed+=(isVisible ? 1 : -1);
+  public void addDisplayedBackbone(int backboneVisibilityFlag, boolean isVisible) {
+    nBackbonesDisplayed += (isVisible ? 1 : -1);
     setShapeVisibility(backboneVisibilityFlag, isVisible);
   }
   
@@ -201,11 +224,6 @@ final public class Atom extends Point3fi implements JmolNode {
   public void setMadAtom(Viewer viewer, RadiusData rd) {
     madAtom = calculateMad(viewer, rd);
   }
-  /* what about these?    
-  if (Float.isNaN(fsize)) {
-    case -1000: // temperature
-      break;
-*/
   
   public short calculateMad(Viewer viewer, RadiusData rd) {
     if (rd == null)
@@ -213,34 +231,36 @@ final public class Atom extends Point3fi implements JmolNode {
     float f = rd.value;
     if (f == 0)
       return 0;
-    switch (rd.type) {
-    case RadiusData.TYPE_SCREEN:
+    switch (rd.factorType) {
+    case SCREEN:
        return (short) f;
-    case RadiusData.TYPE_FACTOR:
-    case RadiusData.TYPE_OFFSET:
+    case FACTOR:
+    case OFFSET:
       float r = 0;
       switch (rd.vdwType) {
-      case Token.temperature:
+      case TEMP:
         float tmax = viewer.getBfactor100Hi();
         r = (tmax > 0 ? getBfactor100() / tmax : 0);
         break;
-      case Token.ionic:
+      case HYDRO:
+        r = Math.abs(getHydrophobicity());
+        break;
+      case IONIC:
         r = getBondingRadiusFloat();
         break;
-      case Token.adpmin:
-      case Token.adpmax:
-        r = getADPMinMax(rd.vdwType == Token.adpmax);
+      case ADPMIN:
+      case ADPMAX:
+        r = getADPMinMax(rd.vdwType == EnumVdw.ADPMAX);
         break;
       default:
         r = getVanderwaalsRadiusFloat(viewer, rd.vdwType);
       }
-      if (rd.type == RadiusData.TYPE_FACTOR)
+      if (rd.factorType == RadiusData.EnumType.FACTOR)
         f *= r;
       else
         f += r;
       break;
-    default:
-    case RadiusData.TYPE_ABSOLUTE:
+    case ABSOLUTE:
       break;
     }
     short mad = (short) (f < 0 ? f: f * 2000);
@@ -307,11 +327,11 @@ final public class Atom extends Point3fi implements JmolNode {
   }
 
   public short getElementNumber() {
-    return (short) (atomicAndIsotopeNumber % 128);
+    return Elements.getElementNumber(atomicAndIsotopeNumber);
   }
   
   public short getIsotopeNumber() {
-    return (short) (atomicAndIsotopeNumber >> 7);
+    return Elements.getIsotopeNumber(atomicAndIsotopeNumber);
   }
   
   public short getAtomicAndIsotopeNumber() {
@@ -355,7 +375,7 @@ final public class Atom extends Point3fi implements JmolNode {
     return (formalChargeAndFlags & VIBRATION_VECTOR_FLAG) != 0;
   }
 
-  void setFormalCharge(int charge) {
+  public void setFormalCharge(int charge) {
     formalChargeAndFlags = (byte)((formalChargeAndFlags & FLAG_MASK) 
         | ((charge == Integer.MIN_VALUE ? 0 : charge > 7 ? 7 : charge < -3 ? -3 : charge) << 2));
   }
@@ -383,6 +403,13 @@ final public class Atom extends Point3fi implements JmolNode {
     return bfactor100s[index];
   }
 
+  private float getHydrophobicity() {
+    float[] values = group.chain.modelSet.hydrophobicities;
+    if (values == null)
+      return Elements.getHydrophobicity(group.getGroupID());
+    return values[index];
+  }
+
   public boolean setRadius(float radius) {
     return !Float.isNaN(userDefinedVanDerWaalRadius = (radius > 0 ? radius : Float.NaN));  
   }
@@ -419,12 +446,7 @@ final public class Atom extends Point3fi implements JmolNode {
   }
 
   public int getImplicitHydrogenCount() {
-    int targetValence = getTargetValence();
-    int charge = getFormalCharge();
-    if (charge != 0)
-      targetValence += (targetValence == 4 ? -Math.abs(charge) : charge);
-    int n = targetValence - getValence();
-    return (n < 0 ? 0 : n);
+    return group.chain.modelSet.getImplicitHydrogenCount(this);
   }
 
   int getTargetValence() {
@@ -453,62 +475,63 @@ final public class Atom extends Point3fi implements JmolNode {
     return (dimension == 0 ? x : (dimension == 1 ? y : z));
   }
 
-  public float getVanderwaalsRadiusFloat(Viewer viewer, int iType) {
+  public float getVanderwaalsRadiusFloat(Viewer viewer, EnumVdw type) {
     // called by atomPropertyFloat as VDW_AUTO,
-    // AtomCollection.filAtomData with VDW_AUTO or VDW_NOJMOL
+    // AtomCollection.fillAtomData with VDW_AUTO or VDW_NOJMOL
     // AtomCollection.findMaxRadii with VDW_AUTO
     // AtomCollection.getAtomPropertyState with VDW_AUTO
     // AtomCollection.getVdwRadius with passed on type
     return (Float.isNaN(userDefinedVanDerWaalRadius) 
-        ? viewer.getVanderwaalsMar(atomicAndIsotopeNumber % 128, getVdwType(iType)) / 1000f
+        ? viewer.getVanderwaalsMar(atomicAndIsotopeNumber % 128, getVdwType(type)) / 1000f
         : userDefinedVanDerWaalRadius);
   }
 
   /**
    * 
-   * @param iType
+   * @param type 
    * @return if VDW_AUTO, will return VDW_AUTO_JMOL, VDW_AUTO_RASMOL, or VDW_AUTO_BABEL
    *         based on the model type
    */
-  private int getVdwType(int iType) {
-    switch (iType) {
-    case JmolConstants.VDW_AUTO:
-      iType = group.chain.modelSet.getDefaultVdwType(modelIndex);
+  @SuppressWarnings("incomplete-switch")
+  private EnumVdw getVdwType(EnumVdw type) {
+    switch (type) {
+    case AUTO:
+      type = group.chain.modelSet.getDefaultVdwType(modelIndex);
       break;
-    case JmolConstants.VDW_NOJMOL:
-      iType = group.chain.modelSet.getDefaultVdwType(modelIndex);
-      if (iType == JmolConstants.VDW_AUTO_JMOL)
-        iType = JmolConstants.VDW_AUTO_BABEL;
+    case NOJMOL:
+      type = group.chain.modelSet.getDefaultVdwType(modelIndex);
+      if (type == EnumVdw.AUTO_JMOL)
+        type = EnumVdw.AUTO_BABEL;
       break;
     }
-    return iType;
+    return type;
   }
 
   public float getCovalentRadiusFloat() {
-    return JmolConstants.getBondingRadiusFloat(atomicAndIsotopeNumber % 128, 0);
+    return Elements.getBondingRadiusFloat(atomicAndIsotopeNumber, 0);
   }
 
   public float getBondingRadiusFloat() {
     float[] ionicRadii = group.chain.modelSet.ionicRadii;
     float r = (ionicRadii == null ? 0 : ionicRadii[index]);
-    return (r == 0 ? JmolConstants.getBondingRadiusFloat(atomicAndIsotopeNumber % 128,
+    return (r == 0 ? Elements.getBondingRadiusFloat(atomicAndIsotopeNumber,
         getFormalCharge()) : r);
   }
 
-  float getVolume(Viewer viewer, int iType) {
-    float r1 = (iType < 0 ? userDefinedVanDerWaalRadius : Float.NaN);
+  float getVolume(Viewer viewer, EnumVdw vType) {
+    float r1 = (vType == null ? userDefinedVanDerWaalRadius : Float.NaN);
     if (Float.isNaN(r1))
-      r1 = viewer.getVanderwaalsMar(getElementNumber(), getVdwType(iType)) / 1000f;
+      r1 = viewer.getVanderwaalsMar(getElementNumber(), getVdwType(vType)) / 1000f;
     double volume = 0;
     if (bonds != null)
       for (int j = 0; j < bonds.length; j++) {
         if (!bonds[j].isCovalent())
           continue;
         Atom atom2 = bonds[j].getOtherAtom(this);
-        float r2 = (iType < 0 ? atom2.userDefinedVanDerWaalRadius : Float.NaN);
+        float r2 = (vType == null ? atom2.userDefinedVanDerWaalRadius : Float.NaN);
         if (Float.isNaN(r2))
           r2 = viewer.getVanderwaalsMar(atom2.getElementNumber(), atom2
-              .getVdwType(iType)) / 1000f;
+              .getVdwType(vType)) / 1000f;
         float d = distance(atom2);
         if (d > r1 + r2)
           continue;
@@ -560,7 +583,7 @@ final public class Atom extends Point3fi implements JmolNode {
      return group;
    }
    
-   public void setGroupBits(BitSet bs) {
+   public void getGroupBits(BitSet bs) {
      group.selectAtoms(bs);
    }
    
@@ -684,8 +707,8 @@ final public class Atom extends Point3fi implements JmolNode {
     return modelIndex;
   }
    
-  public int getMoleculeNumber() {
-    return (group.chain.modelSet.getMoleculeIndex(index) + 1);
+  public int getMoleculeNumber(boolean inModel) {
+    return (group.chain.modelSet.getMoleculeIndex(index, inModel) + 1);
   }
    
   private float getFractionalCoord(char ch, boolean asAbsolute) {
@@ -700,50 +723,51 @@ final public class Atom extends Point3fi implements JmolNode {
 
   private Point3f getFractionalCoord(boolean asAbsolute) {
     // asAbsolute TRUE uses the original unshifted matrix
-    SymmetryInterface[] c = group.chain.modelSet.unitCells;
+    SymmetryInterface c = group.chain.modelSet.getUnitCell(modelIndex);
+    if (c == null) 
+      return this;
     Point3f pt = new Point3f(this);
-    if (c != null && c[modelIndex].haveUnitCell())
-      c[modelIndex].toFractional(pt, asAbsolute);
+    c.toFractional(pt, asAbsolute);
     return pt;
   }
   
   public Point3f getFractionalUnitCoord(boolean asCartesian) {
-    SymmetryInterface[] c = group.chain.modelSet.unitCells;
+    SymmetryInterface c = group.chain.modelSet.getUnitCell(modelIndex);
+    if (c == null)
+      return this;
     Point3f pt = new Point3f(this);
-    if (c != null && c[modelIndex].haveUnitCell()) {
-      if (group.chain.model.isJmolDataFrame) {
-        c[modelIndex].toFractional(pt, false);
-        if (asCartesian)
-          c[modelIndex].toCartesian(pt, false);
-      } else {
-        c[modelIndex].toUnitCell(pt, null);
-        if (!asCartesian)
-          c[modelIndex].toFractional(pt, false);
-      }
+    if (group.chain.model.isJmolDataFrame) {
+      c.toFractional(pt, false);
+      if (asCartesian)
+        c.toCartesian(pt, false);
+    } else {
+      c.toUnitCell(pt, null);
+      if (!asCartesian)
+        c.toFractional(pt, false);
     }
     return pt;
   }
   
   public float getFractionalUnitDistance(Point3f pt, Point3f ptTemp1, Point3f ptTemp2) {
-    SymmetryInterface[] c = group.chain.modelSet.unitCells;
+    SymmetryInterface c = group.chain.modelSet.getUnitCell(modelIndex);
     if (c == null) 
       return distance(pt);
     ptTemp1.set(this);
     ptTemp2.set(pt);
     if (group.chain.model.isJmolDataFrame) {
-      c[modelIndex].toFractional(ptTemp1, true);
-      c[modelIndex].toFractional(ptTemp2, true);
+      c.toFractional(ptTemp1, true);
+      c.toFractional(ptTemp2, true);
     } else {
-      c[modelIndex].toUnitCell(ptTemp1, null);
-      c[modelIndex].toUnitCell(ptTemp2, null);
+      c.toUnitCell(ptTemp1, null);
+      c.toUnitCell(ptTemp2, null);
     }
     return ptTemp1.distance(ptTemp2);
   }
   
   void setFractionalCoord(int tok, float fValue, boolean asAbsolute) {
-    SymmetryInterface[] c = group.chain.modelSet.unitCells;
-    if (c != null && c[modelIndex].haveUnitCell())
-      c[modelIndex].toFractional(this, asAbsolute);
+    SymmetryInterface c = group.chain.modelSet.getUnitCell(modelIndex);
+    if (c != null)
+      c.toFractional(this, asAbsolute);
     switch (tok) {
     case Token.fux:
     case Token.fracx:
@@ -758,8 +782,8 @@ final public class Atom extends Point3fi implements JmolNode {
       z = fValue;
       break;
     }
-    if (c != null && c[modelIndex].haveUnitCell())
-      c[modelIndex].toCartesian(this, asAbsolute);
+    if (c != null)
+      c.toCartesian(this, asAbsolute);
   }
   
   void setFractionalCoord(Point3f ptNew, boolean asAbsolute) {
@@ -768,9 +792,9 @@ final public class Atom extends Point3fi implements JmolNode {
   
   void setFractionalCoord(Point3f pt, Point3f ptNew, boolean asAbsolute) {
     pt.set(ptNew);
-    SymmetryInterface[] c = group.chain.modelSet.unitCells;
-    if (c != null && c[modelIndex].haveUnitCell())
-      c[modelIndex].toCartesian(pt, asAbsolute && !group.chain.model.isJmolDataFrame);
+    SymmetryInterface c = group.chain.modelSet.getUnitCell(modelIndex);
+    if (c != null)
+      c.toCartesian(pt, asAbsolute && !group.chain.model.isJmolDataFrame);
   }
 
   boolean isCursorOnTopOf(int xCursor, int yCursor,
@@ -821,7 +845,7 @@ final public class Atom extends Point3fi implements JmolNode {
     if (useChimeFormat) {
       String group3 = getGroup3(true);
       char chainID = getChainID();
-      Point3f pt = (group.chain.modelSet.unitCells == null ? null : getFractionalCoord(true));
+      Point3f pt = getFractionalCoord(true);
       return "Atom: " + (group3 == null ? getElementSymbol() : getAtomName()) + " " + getAtomNumber() 
           + (group3 != null && group3.length() > 0 ? 
               (isHetero() ? " Hetero: " : " Group: ") + group3 + " " + getResno() 
@@ -885,7 +909,9 @@ final public class Atom extends Point3fi implements JmolNode {
   
   public String getGroup3(boolean allowNull) {
     String group3 = group.getGroup3();
-    return (allowNull || group3 != null && group3.length() > 0 ? group3 : "UNK");
+    return (allowNull 
+        || group3 != null && group3.length() > 0 
+        ? group3 : "UNK");
   }
 
   public String getGroup1(char c0) {
@@ -961,7 +987,7 @@ final public class Atom extends Point3fi implements JmolNode {
     // Is the atom's model visible? Is the atom NOT hidden?
     if (!isInFrame() || group.chain.modelSet.isAtomHidden(index))
       return false;
-    // Is any shape associated with this atom visible? 
+    // Is any shape associated with this atom visible?
     if (flags != 0)
       return (isShapeVisible(flags));  
     flags = shapeVisibilityFlags;
@@ -970,7 +996,7 @@ final public class Atom extends Point3fi implements JmolNode {
     //  if it does not show up itself as part of the structure
     //  (this will be a difference in terms of *clickability*).
     // except BACKBONE -- in which case we only see the lead atoms
-    if (group.shapeVisibilityFlags != JmolConstants.BACKBONE_VISIBILITY_FLAG
+    if (group.shapeVisibilityFlags != Atom.BACKBONE_VISIBILITY_FLAG
         || isLeadAtom())
       flags |= group.shapeVisibilityFlags;
 
@@ -1053,15 +1079,15 @@ final public class Atom extends Point3fi implements JmolNode {
     return group.chain.modelSet.getModelFileNumber(modelIndex);
   }
   
-  public String getGroupType() {
-    return JmolConstants.getProteinStructureName(getProteinStructureType(), true);
+  public String getBioStructureTypeName() {
+    return getProteinStructureType().getBioStructureTypeName(true);
   }
   
-  public byte getProteinStructureType() {
+  public EnumStructure getProteinStructureType() {
     return group.getProteinStructureType();
   }
   
-  public byte getProteinStructureSubType() {
+  public EnumStructure getProteinStructureSubType() {
     return group.getProteinStructureSubType();
   }
   
@@ -1093,10 +1119,12 @@ final public class Atom extends Point3fi implements JmolNode {
     return group.getInsertionCode();
   }
   
+  @Override
   public boolean equals(Object obj) {
     return (this == obj);
   }
 
+  @Override
   public int hashCode() {
     //this overrides the Point3fi hashcode, which would
     //give a different hashcode for an atom depending upon
@@ -1139,6 +1167,8 @@ final public class Atom extends Point3fi implements JmolNode {
     case Token.element:
     case Token.elemno:
       return atom.getElementNumber();
+    case Token.elemisono:
+      return atom.atomicAndIsotopeNumber;
     case Token.file:
       return atom.getModelFileIndex() + 1;
     case Token.formalcharge:
@@ -1157,7 +1187,7 @@ final public class Atom extends Point3fi implements JmolNode {
     case Token.modelindex:
       return atom.modelIndex;
     case Token.molecule:
-      return atom.getMoleculeNumber();
+      return atom.getMoleculeNumber(true);
     case Token.occupancy:
       return atom.getOccupancy100();
     case Token.polymer:
@@ -1172,9 +1202,9 @@ final public class Atom extends Point3fi implements JmolNode {
     case Token.site:
       return atom.getAtomSite();
     case Token.structure:
-      return atom.getProteinStructureType();
+      return atom.getProteinStructureType().getId();
     case Token.substructure:
-      return atom.getProteinStructureSubType();
+      return atom.getProteinStructureSubType().getId();
     case Token.strucno:
       return atom.getStrucNo();
     case Token.valence:
@@ -1194,7 +1224,6 @@ final public class Atom extends Point3fi implements JmolNode {
    * 
    */  
   public static float atomPropertyFloat(Viewer viewer, Atom atom, int tokWhat) {
-
     switch (tokWhat) {
     case Token.radius:
       return atom.getRadius();
@@ -1205,8 +1234,10 @@ final public class Atom extends Point3fi implements JmolNode {
       return atom.getSurfaceDistance100() / 100f;
     case Token.temperature: // 0 - 9999
       return atom.getBfactor100() / 100f;
+    case Token.hydrophobic:
+      return atom.getHydrophobicity();
     case Token.volume:
-      return atom.getVolume(viewer, JmolConstants.VDW_AUTO);
+      return atom.getVolume(viewer, EnumVdw.AUTO);
 
     // these next have to be multiplied by 100 if being compared
     // note that spacefill here is slightly different than radius -- no integer option
@@ -1240,6 +1271,8 @@ final public class Atom extends Point3fi implements JmolNode {
       return atom.getFractionalCoord('Z', false);
     case Token.ionic:
       return atom.getBondingRadiusFloat();
+    case Token.mass:
+      return atom.getMass();
     case Token.occupancy:
       return atom.getOccupancy100() / 100f;
     case Token.partialcharge:
@@ -1273,18 +1306,27 @@ final public class Atom extends Point3fi implements JmolNode {
     case Token.unitz:
       return atom.getFractionalUnitCoord('Z');
     case Token.vanderwaals:
-      return atom.getVanderwaalsRadiusFloat(viewer, JmolConstants.VDW_AUTO);
+      return atom.getVanderwaalsRadiusFloat(viewer, EnumVdw.AUTO);
     case Token.vibx:
       return atom.getVibrationCoord('X');
     case Token.viby:
       return atom.getVibrationCoord('Y');
     case Token.vibz:
       return atom.getVibrationCoord('Z');
+    case Token.vectorscale:
+      Vector3f v = atom.getVibrationVector();
+      return (v == null ? 0 : v.length() * viewer.getVectorScale());
+      
     }
     return atomPropertyInt(atom, tokWhat);
   }
 
-  public static String atomPropertyString(Atom atom, int tokWhat) {
+  private float getMass() {
+    float mass = getIsotopeNumber();
+    return (mass > 0 ? mass : Elements.getAtomicMass(getElementNumber()));
+  }
+
+  public static String atomPropertyString(Viewer viewer, Atom atom, int tokWhat) {
     char ch;
     switch (tokWhat) {
     case Token.altloc:
@@ -1317,11 +1359,13 @@ final public class Atom extends Point3fi implements JmolNode {
         s = "";
       return s;
     case Token.structure:
-      return JmolConstants.getProteinStructureName(atom.getProteinStructureType(), false);
+      return atom.getProteinStructureType().getBioStructureTypeName(false);
     case Token.substructure:
-      return JmolConstants.getProteinStructureName(atom.getProteinStructureSubType(), false);
+      return atom.getProteinStructureSubType().getBioStructureTypeName(false);
     case Token.strucid:
       return atom.getStructureId();
+    case Token.shape:
+      return viewer.getHybridizationAndAxes(atom.index, null, null, "d");
     case Token.symbol:
       return atom.getElementSymbol(false);
     case Token.symmetry:
@@ -1354,7 +1398,7 @@ final public class Atom extends Point3fi implements JmolNode {
     return null;
   }
 
-  boolean isWithinStructure(byte type) {
+  boolean isWithinStructure(EnumStructure type) {
     return group.isWithinStructure(type);
   }
   
@@ -1366,12 +1410,30 @@ final public class Atom extends Point3fi implements JmolNode {
     return group.isCrossLinked(((Atom) node).getGroup());
   }
 
-  public boolean getCrossLinkLeadAtomIndexes(List vReturn) {
+  public boolean getCrossLinkLeadAtomIndexes(List<Integer> vReturn) {
     return group.getCrossLinkLeadAtomIndexes(vReturn);
   }
   
+  @Override
   public String toString() {
     return getInfo();
   }
 
+  public boolean isWithinFourBonds(Atom atomOther) {
+    if (modelIndex != atomOther.modelIndex)
+      return  false;
+    if (isCovalentlyBonded(atomOther))
+      return true; 
+    Bond[] bondsOther = atomOther.bonds;
+    for (int i = 0; i < bondsOther.length; i++) {
+      Atom atom2 = bondsOther[i].getOtherAtom(atomOther);
+      if (isCovalentlyBonded(atom2))
+        return true;
+      for (int j = 0; j < bonds.length; j++)
+        if (bonds[j].getOtherAtom(this).isCovalentlyBonded(atom2))
+          return true;
+    }
+    return false;
+  }
+
 }
