@@ -24,7 +24,9 @@
 package org.jmol.modelsetbio;
 
 
+import org.jmol.constant.EnumStructure;
 import org.jmol.modelset.Atom;
+import org.jmol.modelset.Bond;
 import org.jmol.modelset.Group;
 import org.jmol.modelset.LabelToken;
 import org.jmol.modelset.Polymer;
@@ -50,7 +52,7 @@ public abstract class BioPolymer extends Polymer {
 
   Monomer[] monomers;
 
-  
+  @Override
   public Group[] getGroups() {
     return monomers;
   }
@@ -63,8 +65,9 @@ public abstract class BioPolymer extends Polymer {
     model = monomers[0].getModel();
   }
 
-  
+  @Override
   public void getRange(BitSet bs) {
+    // this is OK -- doesn't relate to added hydrogens
     if (monomerCount == 0)
       return;
     bs.set(monomers[0].firstAtomIndex,
@@ -107,7 +110,7 @@ public abstract class BioPolymer extends Polymer {
     throw new NullPointerException();
   }
 
-  
+  @Override
   public void clearStructures() {
     for (int i = 0; i < monomerCount; i++)
       monomers[i].setStructure(null);
@@ -115,11 +118,11 @@ public abstract class BioPolymer extends Polymer {
 
   void removeProteinStructure(int monomerIndex, int count) {
     Monomer m = monomers[monomerIndex];
-    byte iType = m.getProteinStructureType();
+    EnumStructure type = m.getProteinStructureType();
     int mLast = -1;
     for (int i = 0, pt = monomerIndex; i < count && pt < monomerCount; i++, pt++) {
       monomers[pt].setStructure(null);
-      mLast = monomers[pt].setProteinStructureType(iType, mLast);
+      mLast = monomers[pt].setProteinStructureType(type, mLast);
     }
   }
 
@@ -185,7 +188,7 @@ public abstract class BioPolymer extends Polymer {
     return monomers[polymerIndex].getWingAtom();
   }
 
-  
+  @Override
   public void getConformation(BitSet bsConformation, int conformationIndex) {
     Atom[] atoms = model.getModelSet().atoms;
     for (int i = monomerCount; --i >= 0;)
@@ -193,7 +196,7 @@ public abstract class BioPolymer extends Polymer {
     recalculateLeadMidpointsAndWingVectors();
   }
 
-  
+  @Override
   public void setConformation(BitSet bsSelected) {
     Atom[] atoms = model.getModelSet().atoms;
     for (int i = monomerCount; --i >= 0;)
@@ -204,7 +207,7 @@ public abstract class BioPolymer extends Polymer {
   private boolean invalidLead;
   protected boolean invalidControl = false;
   
-  
+  @Override
   public void recalculateLeadMidpointsAndWingVectors() {
     invalidLead = invalidControl = true;
     getLeadAtomIndices();
@@ -216,7 +219,7 @@ public abstract class BioPolymer extends Polymer {
     // amino polymer only
   }
   
-  
+  @Override
   public Point3f[] getLeadMidpoints() {
     if (leadMidpoints == null)
       calcLeadMidpointsAndWingVectors();
@@ -250,7 +253,8 @@ public abstract class BioPolymer extends Polymer {
       this.sheetSmoothing = sheetSmoothing;
     for (int i = 0; i < monomerCount; i++) 
       controlPoints[i] = getControlPoint(i, v);
-    controlPoints[monomerCount] = controlPoints[monomerCount - 1];
+    controlPoints[monomerCount] = getTerminatorPoint();
+    //controlPoints[monomerCount] = controlPoints[monomerCount - 1];
     invalidControl = false;
     return controlPoints;
   }
@@ -367,7 +371,7 @@ public abstract class BioPolymer extends Polymer {
 
   BitSet bsSelectedMonomers;
 
-  
+  @Override
   public void calcSelectedMonomersCount(BitSet bsSelected) {
     selectedMonomerCount = 0;
     if (bsSelectedMonomers == null)
@@ -385,8 +389,8 @@ public abstract class BioPolymer extends Polymer {
     return (i >= 0 && bsSelectedMonomers.get(i));
   }
 
-  
-  public int getPolymerPointsAndVectors(int last, BitSet bs, List vList,
+  @Override
+  public int getPolymerPointsAndVectors(int last, BitSet bs, List<Point3f[]> vList,
                                         boolean isTraceAlpha,
                                         float sheetSmoothing) {
     Point3f[] points = getControlPoints(isTraceAlpha, sheetSmoothing, false);
@@ -406,7 +410,7 @@ public abstract class BioPolymer extends Polymer {
     return last;
   }
 
-  
+  @Override
   public String getSequence() {
     char[] buf = new char[monomerCount];
     for (int i = 0; i < monomerCount; i++)
@@ -414,24 +418,24 @@ public abstract class BioPolymer extends Polymer {
     return String.valueOf(buf);
   }
 
-  
-  public Map getPolymerInfo(BitSet bs) {
-    Map returnInfo = new Hashtable();
-    List info = new ArrayList();
-    List structureInfo = null;
+  @Override
+  public Map<String, Object> getPolymerInfo(BitSet bs) {
+    Map<String, Object> returnInfo = new Hashtable<String, Object>();
+    List<Map<String, Object>> info = new ArrayList<Map<String,Object>>();
+    List<Map<String, Object>> structureInfo = null;
     ProteinStructure ps;
     ProteinStructure psLast = null;
     int n = 0;
     for (int i = 0; i < monomerCount; i++) {
       if (bs.get(monomers[i].leadAtomIndex)) {
-        Map monomerInfo = monomers[i].getMyInfo();
+        Map<String, Object> monomerInfo = monomers[i].getMyInfo();
         monomerInfo.put("monomerIndex", Integer.valueOf(i));
         info.add(monomerInfo);
         if ((ps = getProteinStructure(i)) != null && ps != psLast) {
-          Map psInfo = new Hashtable();
+          Map<String, Object> psInfo = new Hashtable<String, Object>();
           (psLast = ps).getInfo(psInfo);
           if (structureInfo == null) {
-            structureInfo = new ArrayList();
+            structureInfo = new ArrayList<Map<String,Object>>();
           }
           psInfo.put("index", Integer.valueOf(n++));
           structureInfo.add(psInfo);
@@ -447,11 +451,10 @@ public abstract class BioPolymer extends Polymer {
     return returnInfo;
   }
 
-  
+  @Override
   public void getPolymerSequenceAtoms(int group1, int nGroups, BitSet bsInclude,
                                       BitSet bsResult) {
-    int max = group1 + nGroups;
-    for (int i = group1; i < monomerCount && i < max; i++)
+    for (int i = Math.min(monomerCount, group1 + nGroups); --i >= group1;)
       monomers[i].getMonomerSequenceAtoms(bsInclude, bsResult);
   }
 
@@ -476,11 +479,11 @@ public abstract class BioPolymer extends Polymer {
   final private static String[] qColor = { "yellow", "orange", "purple" };
 
   final public static void getPdbData(Viewer viewer, BioPolymer p, char ctype, char qtype,
-                                      int mStep, int derivType, boolean isDraw,
-                                      BitSet bsAtoms, OutputStringBuffer pdbATOM,
-                                      StringBuffer pdbCONECT,
-                                      BitSet bsSelected, boolean addHeader, boolean bothEnds,
-                                      BitSet bsWritten) {
+                                      int mStep, int derivType, BitSet bsAtoms,
+                                      BitSet bsSelected, boolean bothEnds,
+                                      boolean isDraw,
+                                      boolean addHeader, LabelToken[] tokens, OutputStringBuffer pdbATOM,
+                                      StringBuffer pdbCONECT, BitSet bsWritten) {
     boolean calcRamachandranStraightness = (qtype == 'C' || qtype == 'P');
     boolean isRamachandran = (ctype == 'R' || ctype == 'S' && 
         calcRamachandranStraightness);
@@ -564,14 +567,14 @@ public abstract class BioPolymer extends Polymer {
     for (int j = 0; j < (bothEnds? 2 : 1); j++, factor *= -1)
     for (int i = 0; i < (mStep < 1 ? 1 : mStep); i++)
       getData(viewer, i, mStep, p, ctype, qtype, derivType, 
-          bsAtoms, bsSelected, bsWritten, 
-          isDraw, isRamachandran,
-          calcRamachandranStraightness,
+          bsAtoms, bsSelected, isDraw, 
+          isRamachandran, calcRamachandranStraightness,
           useQuaternionStraightness,
           writeRamachandranStraightness,
-          quaternionStraightness, factor,
-          isAmino, isRelativeAlias,
-          pdbATOM, pdbCONECT);
+          quaternionStraightness,
+          factor, isAmino,
+          isRelativeAlias, tokens,
+          pdbATOM, pdbCONECT, bsWritten);
   }
   
   /**
@@ -585,7 +588,6 @@ public abstract class BioPolymer extends Polymer {
    * @param derivType
    * @param bsAtoms
    * @param bsSelected
-   * @param bsWritten
    * @param isDraw
    * @param isRamachandran
    * @param calcRamachandranStraightness
@@ -595,21 +597,24 @@ public abstract class BioPolymer extends Polymer {
    * @param factor
    * @param isAmino
    * @param isRelativeAlias
+   * @param tokens
    * @param pdbATOM
    * @param pdbCONECT
+   * @param bsWritten
    */
   private static void getData(Viewer viewer, int m0, int mStep, BioPolymer p,
                               char ctype, char qtype, int derivType,
                               BitSet bsAtoms, BitSet bsSelected,
-                              BitSet bsWritten, boolean isDraw,
-                              boolean isRamachandran,
+                              boolean isDraw, boolean isRamachandran,
                               boolean calcRamachandranStraightness,
                               boolean useQuaternionStraightness,
                               boolean writeRamachandranStraightness,
-                              boolean quaternionStraightness, float factor, 
-                              boolean isAmino,
+                              boolean quaternionStraightness,
+                              float factor, boolean isAmino, 
                               boolean isRelativeAlias,
-                              OutputStringBuffer pdbATOM, StringBuffer pdbCONECT) {
+                              LabelToken[] tokens,
+                              OutputStringBuffer pdbATOM, StringBuffer pdbCONECT,
+                              BitSet bsWritten) {
     String prefix = (derivType > 0 ? "dq" + (derivType == 2 ? "2" : "") : "q");
     Quaternion q;
     Atom aprev = null;
@@ -622,6 +627,8 @@ public abstract class BioPolymer extends Polymer {
     String strExtra = "";
     float val1 = Float.NaN;
     float val2 = Float.NaN;
+    Point3f pt = (isDraw ? new Point3f() : null);
+    
     int dm = (mStep <= 1 ? 1 : mStep);
     for (int m = m0; m < p.monomerCount; m += dm) {
       Monomer monomer = p.monomers[m];
@@ -659,32 +666,37 @@ public abstract class BioPolymer extends Polymer {
             // standard dihedral/Jmol definitions for anticlockwise positive
             // angles
             AminoMonomer aa = (AminoMonomer) monomer;
-            pdbATOM.append("draw phi" + id + " ARROW ARC ").append(
-                Escape.escape(aa.getNitrogenAtom())).append(
-                Escape.escape(a)).append(
-                Escape.escape(aa.getCarbonylCarbonAtom())).append(
-                "{" + (-x) + " " + x + " 0.5} \"phi = " + (int) x + "\"")
+            pt.set(-x, x, 0.5f);
+            pdbATOM.append("draw ID \"phi").append(id).append("\" ARROW ARC ")
+                .append(Escape.escape(aa.getNitrogenAtom()))
+                .append(Escape.escape(a))
+                .append(Escape.escape(aa.getCarbonylCarbonAtom()))
+                .append(Escape.escape(pt))
+                .append(" \"phi = ").append(String.valueOf((int) x)).append('\"')
                 .append(" color ").append(qColor[2]).append('\n');
-            pdbATOM.append("draw psi" + id + " ARROW ARC ").append(
-                Escape.escape(a)).append(
-                Escape.escape(aa.getCarbonylCarbonAtom())).append(
-                Escape.escape(aa.getNitrogenAtom())).append(
-                "{0 " + y + " 0.5} \"psi = " + (int) y + "\"")
+            pt.set(0, y, 0.5f);
+            pdbATOM.append("draw ID \"psi").append(id).append("\" ARROW ARC ")
+                .append(Escape.escape(a))
+                .append(Escape.escape(aa.getCarbonylCarbonAtom()))
+                .append(Escape.escape(aa.getNitrogenAtom()))
+                .append(Escape.escape(pt))
+                .append(" \"psi = ").append(String.valueOf((int) y)).append('\"')
                 .append(" color ").append(qColor[1]).append('\n');
-            pdbATOM.append("draw planeNCC" + id + " ").append(
-                Escape.escape(aa.getNitrogenAtom())).append(Escape.escape(a))
-                .append(Escape.escape(aa.getCarbonylCarbonAtom())).append(
-                    " color ").append(qColor[0]).append('\n');
-            pdbATOM.append("draw planeCNC" + id + " ").append(
-                Escape.escape(((AminoMonomer) p.monomers[m - 1])
-                    .getCarbonylCarbonAtom())).append(
-                Escape.escape(aa.getNitrogenAtom())).append(Escape.escape(a))
+            pdbATOM.append("draw ID \"planeNCC").append(id).append("\" ")
+                .append(Escape.escape(aa.getNitrogenAtom()))
+                .append(Escape.escape(a))
+                .append(Escape.escape(aa.getCarbonylCarbonAtom()))
+                .append(" color ").append(qColor[0]).append('\n');
+            pdbATOM.append("draw ID \"planeCNC").append(id).append("\" ")
+                .append(Escape.escape(((AminoMonomer) p.monomers[m - 1]).getCarbonylCarbonAtom()))
+                .append(Escape.escape(aa.getNitrogenAtom()))
+                .append(Escape.escape(a))
                 .append(" color ").append(qColor[1]).append('\n');
-            pdbATOM.append("draw planeCCN" + id + " ").append(Escape.escape(a))
-                .append(Escape.escape(aa.getCarbonylCarbonAtom())).append(
-                    Escape.escape(((AminoMonomer) p.monomers[m + 1])
-                        .getNitrogenAtom())).append(" color ")
-                .append(qColor[2]).append('\n');
+            pdbATOM.append("draw ID \"planeCCN").append(id).append("\" ")
+                .append(Escape.escape(a))
+                .append(Escape.escape(aa.getCarbonylCarbonAtom()))
+                .append(Escape.escape(((AminoMonomer) p.monomers[m + 1]).getNitrogenAtom()))
+                .append(" color ").append(qColor[2]).append('\n');
             continue;
           }
           if (Float.isNaN(angledeg)) {
@@ -856,8 +868,8 @@ public abstract class BioPolymer extends Polymer {
                 Point3f ptH = ((AminoMonomer) monomer)
                     .getNitrogenHydrogenPoint();
                 if (ptH != null)
-                  pdbATOM.append("draw " + prefix + "nh" + id + " width 0.1 "
-                      + Escape.escape(ptH) + "\n");
+                  pdbATOM.append("draw ID \"").append(prefix).append("nh").append(id).append('\"')
+                      .append(" width 0.1 ").append(Escape.escape(ptH)).append('\n');
               }
             }
             if (derivType == 1) {
@@ -866,11 +878,13 @@ public abstract class BioPolymer extends Polymer {
                   .append('\n');
               continue;
             }
-            pdbATOM.append(
-                "draw " + prefix + "a" + id + " VECTOR "
-                    + Escape.escape(ptCenter) + " {" + (x * 2) + "," + (y * 2)
-                    + "," + (z * 2) + "}" + " \">" + deg + "\"").append(
-                " color ").append(qColor[derivType]).append('\n');
+            pt.set(x * 2, y * 2, z * 2);
+            pdbATOM.append("draw ID \"").append(prefix).append("a").append(id).append('\"')
+                .append(" VECTOR ")
+                .append(Escape.escape(ptCenter))
+                .append(Escape.escape(pt))
+                .append(" \">").append(String.valueOf(deg)).append('\"')
+                .append(" color ").append(qColor[derivType]).append('\n');
             continue;
           }
           strExtra = q.getInfo()
@@ -888,8 +902,7 @@ public abstract class BioPolymer extends Polymer {
         if (pdbATOM == null)// || bsSelected != null && !bsSelected.get(a.getIndex()))
           continue;
         bsWritten.set(((Monomer) a.getGroup()).leadAtomIndex);
-        pdbATOM.append(LabelToken.formatLabel(viewer, a,
-            "ATOM  %5i %4a%1A%3n %1c%4R%1E   "));
+        pdbATOM.append(LabelToken.formatLabel(viewer, a, tokens, '\0', null));
         pdbATOM.append(TextFormat
             .sprintf("%8.2f%8.2f%8.2f      %6.3f          %2s    %s\n",
                 new Object[] {
@@ -958,27 +971,26 @@ public abstract class BioPolymer extends Polymer {
     return (float) (1 - 2 * Math.acos(Math.abs(cosHalfTheta))/Math.PI);   
   }
 
-  
+  @Override
   public boolean isDna() { return (monomerCount > 0 && monomers[0].isDna()); }
-  
+  @Override
   public boolean isRna() { return (monomerCount > 0 && monomers[0].isRna()); }
 
-  
+  @Override
   public void getRangeGroups(int nResidues, BitSet bsAtoms, BitSet bsResult) {
     BitSet bsTemp = new BitSet();
     for (int i = 0; i < monomerCount; i++) {
-      int n = bsAtoms.nextSetBit(monomers[i].firstAtomIndex);
-      if (n < 0 || n > monomers[i].lastAtomIndex)
+      if (!monomers[i].isSelected(bsAtoms))
         continue;
       bsTemp.set(Math.max(0, i - nResidues), i + nResidues + 1);
       i += nResidues - 1;
     }
     for (int i = bsTemp.nextSetBit(0); i >= 0 && i < monomerCount; i = bsTemp.nextSetBit(i + 1)) 
-      bsResult.set(monomers[i].firstAtomIndex, monomers[i].lastAtomIndex + 1);
+      monomers[i].selectAtoms(bsResult);
   }
   
-  
-  public String calculateStructures(Polymer[] bioPolymers, int bioPolymerCount, List vHBonds, boolean doReport, boolean dsspIgnoreHydrogens, boolean setStructure) {
+  @Override
+  public String calculateStructures(Polymer[] bioPolymers, int bioPolymerCount, List<Bond> vHBonds, boolean doReport, boolean dsspIgnoreHydrogens, boolean setStructure) {
     // Here because we are calling a static method in AminoPolymer for the 
     // entire SET of polymers, just using the first one, which may or may not
     // be an AminoPolymer.
