@@ -23,17 +23,17 @@
 
 package org.jmol.script;
 
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
+
+import org.jmol.util.JmolEdge;
 import org.jmol.util.Logger;
 import org.jmol.util.TextFormat;
 import org.jmol.viewer.JmolConstants;
 import org.jmol.viewer.Viewer;
-import org.jmol.api.JmolEdge;
 import org.jmol.i18n.GT;
 
-import java.util.ArrayList;
-import java.util.List;
-import java.util.Map;
-
 import javax.vecmath.Point3f;
 
 abstract class ScriptCompilationTokenParser {
@@ -52,7 +52,7 @@ abstract class ScriptCompilationTokenParser {
   protected Viewer viewer;
 
   protected String script;
-  boolean isStateScript;
+  protected boolean isStateScript;
 
   protected short lineCurrent;
   protected int iCommand;
@@ -89,7 +89,7 @@ abstract class ScriptCompilationTokenParser {
   protected boolean isMathExpressionCommand;
   protected boolean isSetOrDefine;
 
-  private List ltokenPostfix;
+  private List<Token> ltokenPostfix;
 
   protected boolean isEmbeddedExpression;
   protected boolean isCommaAsOrAllowed;
@@ -105,6 +105,7 @@ abstract class ScriptCompilationTokenParser {
         && tokCommand != Token.end && !Token.tokAttrOr(tokCommand, Token.atomExpressionCommand,
             Token.implicitStringCommand));
     isMathExpressionCommand = (tokCommand == Token.identifier || Token.tokAttr(tokCommand, Token.mathExpressionCommand));
+
     boolean checkExpression = isEmbeddedExpression
         || (Token.tokAttr(tokCommand, Token.atomExpressionCommand));
 
@@ -143,15 +144,45 @@ abstract class ScriptCompilationTokenParser {
 
   protected boolean compileExpression() {
     int firstToken = (isSetOrDefine && !isSetBrace ? 2 : 1);
-    ltokenPostfix = new ArrayList();
+    ltokenPostfix = new ArrayList<Token>();
     itokenInfix = 0;
     Token tokenBegin = null;
-    if (tokCommand == Token.restrict && tokAt(1) == Token.bonds) {
-      addNextToken();
-      addNextToken();
-    } else {
-      for (int i = 0; i < firstToken && addNextToken(); i++) {
+    int tok = tokAt(1);
+    switch (tokCommand) {
+    case Token.define:
+      if (tokAt(1) == Token.integer && tokAt(2) == Token.per && tokAt(4) == Token.opEQ) {
+        // @2.xxx = 
+        tokCommand = Token.set;
+        isSetBrace = true;
+        ptNewSetModifier = 4;
+        isMathExpressionCommand = true;
+        isEmbeddedExpression = true;
+        addTokenToPostfix(Token.tokenSetProperty);
+        addTokenToPostfix(Token.tokenExpressionBegin);
+        addNextToken();
+        addNextToken();
+        addTokenToPostfix(Token.tokenExpressionEnd);
+        firstToken = 0;
       }
+      break;
+    case Token.restrict:
+      if (tok == Token.bonds) 
+        firstToken = 2;
+      break;
+    case Token.hide:
+    case Token.display:
+    case Token.select:
+      switch(tok) {
+      case Token.add:
+      case Token.remove:
+        firstToken = 2;
+        tok = tokAt(2);
+        break;
+      }
+      if (tok == Token.group)
+        firstToken++;
+    }
+    for (int i = 0; i < firstToken && addNextToken(); i++) {
     }
     while (moreTokens()) {
       if (isEmbeddedExpression) {
@@ -170,7 +201,7 @@ abstract class ScriptCompilationTokenParser {
                 //             and it isn't the name of a user function
 
                 atokenInfix[itokenInfix] = t;
-            }
+              }
           }
           if (!addNextToken())
             break;
@@ -208,11 +239,11 @@ abstract class ScriptCompilationTokenParser {
     }
     atokenInfix = new Token[ltokenPostfix.size()];
     for (int i = ltokenPostfix.size(); --i >= 0;)
-      atokenInfix[i] = (Token) ltokenPostfix.get(i);
+      atokenInfix[i] = ltokenPostfix.get(i);
     return true;
   }
 
-  protected Map htUserFunctions;
+  protected Map<String, Boolean> htUserFunctions;
   protected boolean isUserFunction(String name) {
     return (!isStateScript && (viewer.isFunction(name) || htUserFunctions.containsKey(name)));
   }
@@ -313,7 +344,7 @@ abstract class ScriptCompilationTokenParser {
     if (token == null)
       return false;
     if (logMessages)
-        Logger.debug("addTokenToPostfix" + token);
+        Logger.info("addTokenToPostfix" + token);
     ltokenPostfix.add(token);
     lastToken = token;
     return true;
@@ -416,7 +447,8 @@ abstract class ScriptCompilationTokenParser {
     case Token.smiles:
       return clauseSubstructure();
     case Token.within:
-      return clauseWithin();
+    case Token.contact:
+      return clauseWithin(tok == Token.within);
     case Token.define:
       return clauseDefine(false, false);
     case Token.bonds:
@@ -479,8 +511,8 @@ abstract class ScriptCompilationTokenParser {
      * We assume an expression initially and then adjust accordingly
      * if it turns out this is a coordinate. 
      * 
-     * Note that due to tha nuances of how expressions such as (1-4) are
-     * reported as special codes, Eval must still intepret these
+     * Note that due to the nuances of how expressions such as (1-4) are
+     * reported as special codes, Eval must still interpret these
      * carefully. This could be corrected for here, I think.
      * 
      */
@@ -566,11 +598,16 @@ abstract class ScriptCompilationTokenParser {
     return true;
   }
   
-  private boolean clauseWithin() {
+  private boolean clauseWithin(boolean isWithin) {
 
+    //    // contact(distance, {}, {}) // default 100 for distance
+    // within ( plane, planeExpression)
+    // within ( hkl, hklExpression)
     // within ( distance, plane, planeExpression)
     // within ( distance, hkl, hklExpression)
     // within ( distance, coord, point)
+    // within ( distance, point)
+    // within ( distance, $surfaceId)
     // within ( distance, orClause)
     // within ( group|branch|etc, ....)
     // within ( distance, group, ....)
@@ -582,9 +619,26 @@ abstract class ScriptCompilationTokenParser {
       return false;
     float distance = Float.MAX_VALUE;
     String key = null;
-    boolean allowComma = true;
-    int tok0 = theToken.tok;
+    boolean allowComma = isWithin;
     int tok;
+    int tok0 = theToken.tok;
+    if (!isWithin) {
+      tok = -1;
+      for (int i = itokenInfix; tok != Token.nada; i++) {
+        switch (tok = tokAt(i)) {
+        case Token.comma:
+          tok = Token.nada;
+          break;
+        case Token.leftbrace:
+        case Token.leftparen:
+        case Token.rightparen:
+          distance = 100;
+          returnToken();
+          tok0 = tok = Token.nada;
+          break;
+        }
+      }
+    }
     switch (tok0) {
     case Token.minus:
       if (getToken() == null)
@@ -597,100 +651,131 @@ abstract class ScriptCompilationTokenParser {
     case Token.decimal:
       distance = floatValue();
       break;
-    case Token.search: 
-    case Token.smiles:  
-      addTokenToPostfix(Token.string, theValue);
-      if (!addNextTokenIf(Token.comma))
+    case Token.define:
+      addTokenToPostfix(theToken);
+      if (!clauseDefine(true, false))
         return false;
+      key = "";
       allowComma = false;
-      tok = tokPeek();
-      switch (tok) {
-      case Token.nada:
-        return false;
-      case Token.string:
-        addNextToken();
-        key = "";
-        break;
+      break;
+    }
+    if (isWithin && distance == Float.MAX_VALUE)
+      switch (tok0) {
       case Token.define:
-        if (!clauseDefine(false, true))
+        break;
+      case Token.search:
+      case Token.smiles:
+      case Token.substructure:
+        addTokenToPostfix(Token.string, theValue);
+        if (!addNextTokenIf(Token.comma))
           return false;
-        key = "";
+        allowComma = false;
+        tok = tokPeek();
+        switch (tok) {
+        case Token.nada:
+          return false;
+        case Token.string:
+          addNextToken();
+          key = "";
+          break;
+        case Token.define:
+          if (!clauseDefine(false, true))
+            return false;
+          key = "";
+          break;
+        default:
+          return false;
+        }
+        break;
+      case Token.branch:
+        allowComma = false;
+        // fall through
+      case Token.atomtype:
+      case Token.atomname:
+      case Token.basepair:
+      case Token.boundbox:
+      case Token.chain:
+      case Token.coord:
+      case Token.element:
+      case Token.group:
+      case Token.helix:
+      case Token.model:
+      case Token.molecule:
+      case Token.plane:
+      case Token.hkl:
+      case Token.polymer:
+      case Token.sequence:
+      case Token.sheet:
+      case Token.site:
+      case Token.structure:
+      case Token.string:
+      case Token.vanderwaals:
+        key = (String) theValue;
+        break;
+      case Token.identifier:
+        key = ((String) theValue).toLowerCase();
         break;
       default:
-        return false;
+        return error(ERROR_unrecognizedParameter, "WITHIN", ": "
+            + theToken.value);
       }
-      break;
-    case Token.branch:
-      allowComma = false;
-      // fall through
-    case Token.atomtype:
-    case Token.atomname:
-    case Token.basepair:
-    case Token.boundbox:
-    case Token.chain:
-    case Token.coord:
-    case Token.element:
-    case Token.group:
-    case Token.helix:
-    case Token.hkl:
-    case Token.model:
-    case Token.molecule:
-    case Token.plane:
-    case Token.polymer:
-    case Token.sequence:
-    case Token.sheet:
-    case Token.site:
-    case Token.structure:
-    case Token.string:
-      key = (String) theValue;
-      break;
-    case Token.identifier:
-      key = ((String) theValue).toLowerCase();
-      break;
-    default:
-      return error(ERROR_unrecognizedParameter, "WITHIN", ": " + theToken.value);
-    }
     if (key == null)
       addTokenToPostfix(Token.decimal, new Float(distance));
     else if (key.length() > 0)
       addTokenToPostfix(Token.string, key);
-    while (true) {
-      if (!addNextTokenIf(Token.comma))
+    boolean done = false;
+    while (!done) {
+      if (tok0 != Token.nada && !addNextTokenIf(Token.comma))
         break;
-      tok = tokPeek();
-      switch (tok0) {
-      case Token.integer:
-      case Token.decimal:
-        if (tok == Token.on || tok == Token.off) {
-          addTokenToPostfix(getToken());
-          if (!addNextTokenIf(Token.comma))
-            break;
-          tok = tokPeek();
-        }
-        break;
-      }
+      if (tok0 == Token.nada)
+        tok0 = Token.contact;
       boolean isCoordOrPlane = false;
-      if (key == null) {
-        switch (tok) {
-        case Token.hkl:
-        case Token.coord:
-        case Token.plane:
-          isCoordOrPlane = true;
-          addNextToken();
+      tok = tokPeek();
+      if (isWithin) {
+        switch (tok0) {
+        case Token.integer:
+        case Token.decimal:
+          if (tok == Token.on || tok == Token.off) {
+            addTokenToPostfix(getToken());
+            if (!addNextTokenIf(Token.comma))
+              break;
+            tok = tokPeek();
+          }
           break;
-        case Token.group:
-          getToken();
-          addTokenToPostfix(Token.string, "group");
-          break;
-        case Token.leftbrace:
-          returnToken();
-          isCoordOrPlane = true;
-          addTokenToPostfix(Token
-              .getTokenFromName(distance == Float.MAX_VALUE ? "plane" : "coord"));
         }
-        addNextTokenIf(Token.comma);
+        if (key == null) {
+          switch (tok) {
+          case Token.hkl:
+          case Token.coord:
+          case Token.plane:
+            isCoordOrPlane = true;
+            addNextToken();
+            break;
+          case Token.dollarsign:
+            getToken();
+            getToken();
+            addTokenToPostfix(Token.string, "$" + theValue);
+            done = true;
+            break;
+          case Token.group:
+          case Token.vanderwaals:
+            getToken();
+            addTokenToPostfix(Token.string, Token.nameOf(tok));
+            break;
+          case Token.leftbrace:
+            returnToken();
+            isCoordOrPlane = true;
+            addTokenToPostfix(Token
+                .getTokenFromName(distance == Float.MAX_VALUE ? "plane"
+                    : "coord"));
+          }
+        if (!done)
+          addNextTokenIf(Token.comma);
+        }
       }
       tok = tokPeek();
+      if (done)
+        break;
       if (isCoordOrPlane) {
         while (!tokPeek(Token.rightparen)) {
           switch (tokPeek()) {
@@ -743,7 +828,7 @@ abstract class ScriptCompilationTokenParser {
         if (!addNextTokenIf(Token.comma))
           break;
         String strOrder = (String) getToken().value;
-        int intType = JmolConstants.getBondOrderFromString(strOrder);        if (intType == JmolEdge.BOND_ORDER_NULL) {
+        int intType = ScriptEvaluator.getBondOrderFromString(strOrder);        if (intType == JmolEdge.BOND_ORDER_NULL) {
           returnToken();
         } else {
           addTokenToPostfix(Token.string, strOrder);
@@ -811,7 +896,8 @@ abstract class ScriptCompilationTokenParser {
       return false;
     }
     if (tokenAttr(tokenAtomProperty, Token.strproperty) 
-        && tokenComparator.tok != Token.opEQ && tokenComparator.tok != Token.opNE)
+        && tokenComparator.tok != Token.opEQ 
+        && tokenComparator.tok != Token.opNE)
       return error(ERROR_tokenExpected, "== !=");
     if (getToken() == null)
       return error(ERROR_unrecognizedExpressionToken, "" + valuePeek());
@@ -1067,7 +1153,7 @@ abstract class ScriptCompilationTokenParser {
       // can have open-ended range  
       // select 3-
     }
-    return new Token(Token.spec_seqcode, seqvalue, new Integer(seqcode));
+    return new Token(Token.spec_seqcode, seqvalue, Integer.valueOf(seqcode));
   }
 
   private boolean clauseChainSpec(int tok) {
@@ -1145,7 +1231,7 @@ abstract class ScriptCompilationTokenParser {
     }
     switch (tokPeek()) {
     case Token.integer:
-      return generateResidueSpecCode(new Token(Token.spec_model, new Integer(
+      return generateResidueSpecCode(new Token(Token.spec_model, Integer.valueOf(
           getToken().intValue)));
     case Token.decimal:
             return generateResidueSpecCode(new Token(Token.spec_model,
@@ -1153,7 +1239,7 @@ abstract class ScriptCompilationTokenParser {
     case Token.comma:
     case Token.rightbrace:
     case Token.nada:
-      return generateResidueSpecCode(new Token(Token.spec_model, new Integer(1)));
+      return generateResidueSpecCode(new Token(Token.spec_model, Integer.valueOf(1)));
     }
     return error(ERROR_invalidModelSpecification);
   }
