@@ -1,6 +1,6 @@
 /* $Author: hansonr $
- * $Date: 2011-02-20 05:51:45 +0100 (dim., 20 févr. 2011) $
- * $Revision: 15213 $
+ * $Date: 2012-03-14 12:05:59 +0100 (mer., 14 mars 2012) $
+ * $Revision: 16883 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -36,15 +36,16 @@ import org.jmol.modelset.Bond.BondSet;
 
 import java.util.ArrayList;
 import java.util.Hashtable;
-import java.util.List;
 import java.util.BitSet;
+import java.util.List;
+import java.util.Map;
 
 import javax.vecmath.Matrix3f;
 import javax.vecmath.Matrix4f;
 
 public class ScriptCompiler extends ScriptCompilationTokenParser {
 
-  /**
+  /*
    * The Compiler class is really two parts -- 
    * 
    * Compiler.class          going from characters to tokens
@@ -52,14 +53,13 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
    * 
    * The data structures follow the following sequences:
    * 
-   * String script ==> List lltoken[][] --> Token[][] aatokenCompiled[][]
+   * String script ==> Vector lltoken[][] --> Token[][] aatokenCompiled[][]
    * 
    * A given command goes through the sequence:
    * 
-   * String characters --> Token token --> List ltoken[] --> Token[][] aatokenCompiled[][]
+   * String characters --> Token token --> Vector ltoken[] --> Token[][] aatokenCompiled[][]
    * 
    */
-  private static final String LOAD_TYPES = "auto;append;data;files;inline;menu;smiles;trajectory;models;" + JmolConstants.LOAD_ATOM_DATA_TYPES;
   
   public ScriptCompiler(Viewer viewer) {
     this.viewer = viewer;
@@ -74,7 +74,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
 
   // returns:
   
-  private Hashtable contextVariables;
+  private Map<String, ScriptVariable> contextVariables;
   private Token[][] aatokenCompiled;
   private short[] lineNumbers;
   private int[][] lineIndices;
@@ -150,14 +150,14 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     }
     if (thisFunction == null) {
       if (contextVariables == null)
-        contextVariables = new Hashtable();
+        contextVariables = new Hashtable<String, ScriptVariable>();
       addContextVariable(contextVariables, ident);
     } else {
       thisFunction.addVariable(ident, false);
     }
   }
   
-  static void addContextVariable(Hashtable contextVariables, String ident) {
+  static void addContextVariable(Map<String, ScriptVariable> contextVariables, String ident) {
     contextVariables.put(ident, (new ScriptVariable(Token.string, "")).setName(ident));
   }
 
@@ -168,7 +168,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         return true;
     }
     return (thisFunction != null ? thisFunction.isVariable(ident)
-        : contextVariables != null && contextVariables.containsKey(ident));
+      : contextVariables != null && contextVariables.containsKey(ident));
   }
   
   /**
@@ -193,6 +193,12 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
    * @return cleaned script
    */
   private String cleanScriptComments(String script) {
+    if (script.indexOf('\u201C') >= 0)
+      script = script.replace('\u201C', '"');
+    if (script.indexOf('\u201D') >= 0)
+      script = script.replace('\u201D', '"');
+    if (script.indexOf('\uFEFF') >= 0)
+      script = script.replace('\uFEFF', ' ');
     int pt = (script.indexOf("\1##"));
     if (pt >= 0) {
       // these are for jmolConsole and scriptEditor
@@ -225,9 +231,9 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
   }
 
   private ScriptFlowContext flowContext;
-  private List ltoken;
-  private List lltoken;
-  private List vBraces;
+  private List<Token> ltoken;
+  private List<Token[]> lltoken;
+  private List<Token> vBraces;
 
 
   private int ichBrace;
@@ -265,13 +271,13 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
   private final static int ERROR = 4;
 
   private int tokLastMath;
+  private boolean checkImpliedScriptCmd;
   
-  private List vFunctionStack;
+  private List<ScriptFunction> vFunctionStack;
   
   private boolean compile0(boolean isFull) {
-    vFunctionStack = new ArrayList();
-    htUserFunctions = new Hashtable();
-    script = script.replace('\u201C', '"').replace('\u201D', '"');
+    vFunctionStack = new ArrayList<ScriptFunction>();
+    htUserFunctions = new Hashtable<String, Boolean>();
     script = cleanScriptComments(script);
     ichToken = script.indexOf(Viewer.STATE_VERSION_STAMP);
     isStateScript = (ichToken >= 0);
@@ -304,8 +310,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     iCommand = 0;
     tokLastMath = 0;
     lastToken = Token.tokenOff;
-    vBraces = new ArrayList();
-    vPush = new ArrayList();
+    vBraces = new ArrayList<Token>();
+    vPush = new ArrayList<Token>();
     pushCount = 0;
     iBrace = 0;
     braceCount = 0;
@@ -317,13 +323,15 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     lineIndices = new int[lnLength][2];
     isNewSet = isSetBrace = false;
     ptNewSetModifier = 1;
-    isShowScriptOutput = false;
+    isShowScriptOutput = false;    
     iHaveQuotedString = false;
-    lltoken = new ArrayList();
-    ltoken = new ArrayList();
+    checkImpliedScriptCmd = false;
+    lltoken = new ArrayList<Token[]>();
+    ltoken = new ArrayList<Token>();
     tokCommand = Token.nada;
     lastFlowCommand = null;
     tokenAndEquals = null;
+    tokInitialPlusPlus = Token.nada;
     setBraceCount = 0;
     bracketCount = 0;
     forPoint3 = -1;
@@ -334,9 +342,10 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     needRightParen = false;
     theTok = Token.nada;
     short iLine = 1;
+    
 
     for (; true; ichToken += cchToken) {
-      if ((nTokens = ltoken.size()) == 0) {
+      if ((nTokens = ltoken.size()) == 0) { 
         if (thisFunction != null && thisFunction.chpt0 == 0)
           thisFunction.chpt0 = ichToken;
         ichCurrentCommand = ichToken;
@@ -359,10 +368,9 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
           comment = script.substring(ichToken, ichToken + cchToken).trim();
           break;
         }
-        isEndOfCommand = isEndOfCommand || endOfLine
-            || lookingAtEndOfStatement();
+        isEndOfCommand = isEndOfCommand || endOfLine || lookingAtEndOfStatement();
       }
-
+      
       if (isEndOfCommand) {
         isEndOfCommand = false;
         switch (processTokenList(iLine, isFull)) {
@@ -371,13 +379,14 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         case ERROR:
           return false;
         }
+        checkImpliedScriptCmd = false;
         if (ichToken < cchScript)
           continue;
         setAaTokenCompiled();
-        return (flowContext == null || error(ERROR_missingEnd, Token
-            .nameOf(flowContext.token.tok)));
+        return (flowContext == null 
+            || error(ERROR_missingEnd, Token.nameOf(flowContext.token.tok)));
       }
-
+      
       if (nTokens > 0) {
         switch (checkSpecialParameterSyntax()) {
         case CONTINUE:
@@ -386,9 +395,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
           return false;
         }
       }
-      if (lookingAtLookupToken(ichToken)) {
+      if (lookingAtLookupToken(ichToken)) {        
         String ident = getPrefixToken();
-        //System.out.println(ident);
         switch (parseKnownToken(ident)) {
         case CONTINUE:
           continue;
@@ -412,7 +420,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
             cchToken = 0;
             continue;
           }
-          if (lookingAtImpliedString(true))
+          if (lookingAtImpliedString(true, true))
             ichEnd = ichToken + cchToken;
         }
         return commandExpected();
@@ -425,7 +433,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
   private void setAaTokenCompiled() {
     aatokenCompiled = new Token[lltoken.size()][];
     for (int i = lltoken.size(); --i >= 0;)
-      aatokenCompiled[i] = (Token[]) lltoken.get(i);
+      aatokenCompiled[i] = lltoken.get(i);
   }
 
   private boolean lookingAtLeadingWhitespace() {
@@ -556,7 +564,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     for (; ichT < cchScript; ichT++) {
       if (eol(ch = script.charAt(ichT))) {
         ichEnd = ichT;
-        if (isLineContinuation(ichT - 1, false)) {
+        if (ichT > 0 && isLineContinuation(ichT - 1, false)) {
           ichT += nCharNewLine(ichT);
           continue;
         }
@@ -620,7 +628,30 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       } else if (setBraceCount > 0 && endOfLine && ichToken < cchScript) {
         return CONTINUE;
       }
-      // end of command or comment
+      if (tokCommand == Token.script && checkImpliedScriptCmd) {
+        String s = (nTokens == 2 ? lastToken.value.toString().toUpperCase() : null);
+        if (nTokens > 2 || s != null && (s.endsWith(".SORT") || s.endsWith(".REVERSE"))) {
+          // check for improperly parsed implied script command 
+          ichToken = ichCurrentCommand;
+          nTokens = 0;
+          ltoken.clear();
+          cchToken = 0;
+          tokCommand = Token.nada;
+          return CONTINUE;
+        }
+      }
+      if (isNewSet && nTokens > 2 && tokAt(2) == Token.per
+          && (tokAt(3) == Token.sort || tokAt(3) == Token.reverse)) {
+        // check for x.sort or x.reverse
+        // x.sort / x.reverse ==> x = x.sort / x = x.reverse
+        ltoken.set(0, Token.tokenSet);
+        ltoken.add(1, ltoken.get(1));
+      } else if (tokInitialPlusPlus != Token.nada) {
+        // check for ++x or --x
+        if (!isNewSet)
+          checkNewSetCommand();
+        tokenizePlusPlus(tokInitialPlusPlus, true);
+      }
       iCommand = lltoken.size();
       if (thisFunction != null && thisFunction.cmdpt0 < 0) {
         thisFunction.cmdpt0 = iCommand;
@@ -639,7 +670,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         } else {
           parenCount = setBraceCount = 0;
           setCommand(lastFlowCommand);
-          if (lastFlowCommand.tok != Token.process 
+          if (lastFlowCommand.tok != Token.process
               && (tokAt(0) == Token.leftbrace))
             ltoken.remove(0);
           lastFlowCommand = null;
@@ -665,9 +696,9 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         boolean doEval = true;
         switch (tokCommand) {
         case Token.trycmd:
-        case Token.parallel: 
+        case Token.parallel:
         case Token.function: // formerly "noeval"
-        case Token.end:          
+        case Token.end:
           // end switch may have - or + intValue, depending upon default or not
           // end function and the function call itself has intValue 0,
           // but the FUNCTION declaration itself will have MAX_VALUE intValue
@@ -700,6 +731,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       ptNewSetModifier = 1;
       ltoken.clear();
       nTokens = nSemiSkip = 0;
+      tokInitialPlusPlus = Token.nada;
       tokenAndEquals = null;
       ptSemi = -10;
       forPoint3 = -1;
@@ -707,7 +739,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
 
     }
     if (endOfLine) {
-      if (flowContext != null && flowContext.checkForceEndIf(1)) {
+      if (flowContext != null && flowContext.checkForceEndIf()) {
         forceFlowEnd(flowContext.token);
         isEndOfCommand = true;
         cchToken = 0;
@@ -769,8 +801,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
 
     atokenInfix = new Token[size = ltoken.size()];
     for (int i = ltoken.size(); --i >= 0;)
-      atokenInfix[i] = (Token) ltoken.get(i);
-    
+      atokenInfix[i] = ltoken.get(i);
+
     if (logMessages) {
       Logger.debug("token list:");
       for (int i = 0; i < atokenInfix.length; i++)
@@ -780,16 +812,17 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         Logger.debug(i + ": " + vBraces.get(i));
       Logger.debug("-------------------------------------");
     }
-    
+
     // compile expressions  (ScriptCompilerTokenParser.java)
     return compileExpressions();
-    
+
   }
 
   private Token tokenAt(int i) {
-    return (Token) ltoken.get(i);
+    return ltoken.get(i);
   }
 
+  @Override
   protected int tokAt(int i) {
     return (i < ltoken.size() ? tokenAt(i).tok : Token.nada);
   }
@@ -809,13 +842,13 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return token;
   }
 
-
   private void replaceCommand(Token token) {
     ltoken.remove(0);
     ltoken.add(0, setCommand(token));
   }
 
   private boolean isUserToken;
+  private int tokInitialPlusPlus;
   
   private String getPrefixToken() {
     String ident = script.substring(ichToken, ichToken + cchToken);
@@ -867,15 +900,17 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       boolean isAndEquals = ("+-\\*/&|=".indexOf(ch) >= 0);
       boolean isOperation = (isAndEquals || ch == '.' || ch == '[');
       char ch2 = (ichToken + 1 >= cchScript ? 0 : script.charAt(ichToken + 1));
-      if (!isNewSet && isUserToken && isOperation && (ch == '=' || ch2 == ch || ch2 == '=')) {
+      if (!isNewSet && isUserToken && isOperation
+          && (ch == '=' || ch2 == ch || ch2 == '=')) {
         isNewSet = true;
         // Var data = ""
         // data = 5
         // data++
         // data += what
-        
+
       }
-      if (isNewSet || tokCommand == Token.set || Token.tokAttr(tokCommand, Token.setparam)) {
+      if (isNewSet || tokCommand == Token.set
+          || Token.tokAttr(tokCommand, Token.setparam)) {
         if (ch == '=')
           setEqualPt = ichToken;
 
@@ -920,51 +955,79 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         }
       }
     }
-    
+
     // cd, echo, gotocmd, help, hover, javascript, label, message, and pause
     // all are implicitly strings. You CAN use "..." but you don't have to,
     // and you cannot use '...'. This way the introduction of single quotes 
     // as an equivalent of double quotes cannot break existing scripts. -- BH 06/2009
-    
+
     if (lookingAtString(!Token.tokAttr(tokCommand, Token.implicitStringCommand))) {
       if (cchToken < 0)
         return ERROR(ERROR_endOfCommandUnexpected);
       String str;
-      if ((tokCommand == Token.load || tokCommand == Token.background || tokCommand == Token.script)
+      if ((tokCommand == Token.set && nTokens == 2 && lastToken.tok == Token.defaultdirectory
+          || tokCommand == Token.load || tokCommand == Token.background || tokCommand == Token.script)
           && !iHaveQuotedString) {
-        str = script.substring(ichToken + 1, ichToken + cchToken - 1);
-        if (str.indexOf("\\u") >= 0)
-          str = Escape.unescapeUnicode(str);
+        if (lastToken.tok == Token.inline) {
+          str = getUnescapedStringLiteral();
+        } else {
+          str = script.substring(ichToken + 1, ichToken + cchToken - 1);
+          if (str.indexOf("\\u") >= 0)
+            str = Escape.unescapeUnicode(str);
+        }
       } else {
         str = getUnescapedStringLiteral();
       }
       iHaveQuotedString = true;
-      if (tokCommand == Token.load && lastToken.tok == Token.data 
+      if (tokCommand == Token.load && lastToken.tok == Token.data
           || tokCommand == Token.data && str.indexOf("@") < 0) {
         if (!getData(str))
           return ERROR(ERROR_missingEnd, "data");
       } else {
         addTokenToPrefix(new Token(Token.string, str));
+        if (Token.tokAttr(tokCommand, Token.implicitStringCommand))
+          isEndOfCommand = true;
       }
       return CONTINUE;
     }
     if (tokCommand == Token.sync && nTokens == 1 && charToken()) {
       String ident = script.substring(ichToken, ichToken + cchToken);
-      addTokenToPrefix(new Token(Token.identifier, ident));
+      int iident = Parser.parseInt(ident);
+      if (iident == Integer.MIN_VALUE || Math.abs(iident) < 1000)
+        addTokenToPrefix(new Token(Token.identifier, ident));
+      else
+        addTokenToPrefix(new Token(Token.integer, iident));
       return CONTINUE;
-    } else if (tokCommand == Token.load) {
+    }
+    switch (tokCommand) {
+    case Token.load:
       if (script.charAt(ichToken) == '@') {
         iHaveQuotedString = true;
         return OK;
       }
       if (nTokens == 1 && lookingAtLoadFormat()) {
         String strFormat = script.substring(ichToken, ichToken + cchToken);
-        int tok = (strFormat.indexOf("=") == 0 || strFormat.indexOf("$") == 0 ? Token.string
-            : (strFormat = strFormat.toLowerCase()).equals("data") ? Token.data 
-            : Parser.isOneOf(strFormat, LOAD_TYPES) ? Token.identifier : 0);
-        if (tok != 0) {
-          addTokenToPrefix(new Token(tok, strFormat));
-          iHaveQuotedString = (tok == Token.string);
+        Token token = Token.getTokenFromName(strFormat.toLowerCase());
+        switch (token == null ? Token.nada : token.tok) {
+        case Token.append:
+        case Token.data:
+        case Token.file:
+        case Token.inline:
+        case Token.menu:
+        case Token.model:
+        case Token.smiles:
+        case Token.trajectory:
+          addTokenToPrefix(token);
+          break;
+        default:
+          // skip entirely if not recognized
+          int tok = (strFormat.indexOf("=") == 0 || strFormat.indexOf("$") == 0 ? Token.string
+              : Parser.isOneOf(strFormat = strFormat.toLowerCase(),
+                  JmolConstants.LOAD_ATOM_DATA_TYPES) ? Token.identifier : 0);
+          if (tok != 0) {
+            addTokenToPrefix(new Token(tok, strFormat));
+            iHaveQuotedString = (tok == Token.string);
+          }
         }
         return CONTINUE;
       }
@@ -973,21 +1036,31 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       } else if ((bs = lookingAtBitset()) != null) {
         addTokenToPrefix(new Token(Token.bitset, bs));
         return CONTINUE;
-      } else if (!iHaveQuotedString && lookingAtImpliedString(false)) {
+      } else if (!iHaveQuotedString && lookingAtImpliedString(false, true)) {
         String str = script.substring(ichToken, ichToken + cchToken);
         addTokenToPrefix(new Token(Token.string, str));
         iHaveQuotedString = true;
         return CONTINUE;
       }
-    } else if (tokCommand == Token.script || tokCommand == Token.getproperty) {
-      if (!iHaveQuotedString && lookingAtImpliedString(false)) {
+      break;
+    case Token.script:
+    case Token.getproperty:
+      if (script.charAt(ichToken) == '@') {
+        iHaveQuotedString = true;
+        return OK;
+      }
+      if (!iHaveQuotedString && lookingAtImpliedString(false, false)) {
         String str = script.substring(ichToken, ichToken + cchToken);
+        if (tokCommand == Token.script && str.startsWith("javascript:")) {
+          lookingAtImpliedString(true, true);
+          str = script.substring(ichToken, ichToken + cchToken);
+        }        
         addTokenToPrefix(new Token(Token.string, str));
         iHaveQuotedString = true;
         return CONTINUE;
       }
-    } else if (tokCommand == Token.write) {
-      int pt = cchToken;
+      break;
+    case Token.write:
       // write image 300 300 filename
       // write script filename
       // write spt filename
@@ -995,23 +1068,29 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       // write filename
       if (nTokens == 2 && lastToken.tok == Token.frame)
         iHaveQuotedString = true;
-      if (!iHaveQuotedString && lookingAtImpliedString(true)) {
-        String str = script.substring(ichToken, ichToken + cchToken);
-        if (str.startsWith("@{")) {
+      if (!iHaveQuotedString) {
+        if (script.charAt(ichToken) == '@') {
           iHaveQuotedString = true;
-        } else if (str.indexOf(" ") < 0) {
-          addTokenToPrefix(new Token(Token.string, str));
-          iHaveQuotedString = true;
-          return CONTINUE;
+          return OK;
+        }
+        if (lookingAtImpliedString(true, true)) {
+          int pt = cchToken;
+          String str = script.substring(ichToken, ichToken + cchToken);
+          if (str.indexOf(" ") < 0) {
+            addTokenToPrefix(new Token(Token.string, str));
+            iHaveQuotedString = true;
+            return CONTINUE;
+          }
+          cchToken = pt;
         }
-        cchToken = pt;
       }
+      break;
     }
     if (Token.tokAttr(tokCommand, Token.implicitStringCommand)
         && !(tokCommand == Token.script && iHaveQuotedString)
-        && lookingAtImpliedString(true)) {
+        && lookingAtImpliedString(true, true)) {
       String str = script.substring(ichToken, ichToken + cchToken);
-      if (tokCommand == Token.label 
+      if (tokCommand == Token.label
           && Parser.isOneOf(str.toLowerCase(), "on;off;hide;display"))
         addTokenToPrefix(Token.getTokenFromName(str.toLowerCase()));
       else
@@ -1030,13 +1109,10 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       return CONTINUE;
     }
     if (lookingAtDecimal()) {
-      value = 
-      // can't use parseFloat with jvm 1.1
-      // Float.parseFloat(script.substring(ichToken, ichToken +
-      // cchToken));
-      Float.valueOf(script.substring(ichToken, ichToken + cchToken))
+      value =  Float.valueOf(script.substring(ichToken, ichToken + cchToken))
           .floatValue();
-      int intValue = (JmolConstants.modelValue(script.substring(ichToken, ichToken + cchToken)));
+      int intValue = (ScriptEvaluator.getFloatEncodedInt(script.substring(ichToken,
+          ichToken + cchToken)));
       addTokenToPrefix(new Token(Token.decimal, intValue, new Float(value)));
       return CONTINUE;
     }
@@ -1077,7 +1153,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       return CONTINUE;
     }
     if (!isMathExpressionCommand && parenCount == 0
-        || lastToken.tok != Token.identifier && !tokenAttr(lastToken, Token.mathfunc)) {
+        || lastToken.tok != Token.identifier
+        && !tokenAttr(lastToken, Token.mathfunc)) {
       // here if:
       //   structure helix ({...})
       //   frame align ({...})
@@ -1095,16 +1172,14 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         if (isBondOrMatrix) {
           Object m = lookingAtMatrix();
           if (m instanceof Matrix3f || m instanceof Matrix4f) {
-            addTokenToPrefix(new Token((m instanceof Matrix3f ? Token.matrix3f : Token.matrix4f), m));            
+            addTokenToPrefix(new Token((m instanceof Matrix3f ? Token.matrix3f
+                : Token.matrix4f), m));
             return CONTINUE;
           }
         }
       } else {
         if (isBondOrMatrix)
           addTokenToPrefix(new Token(Token.bitset, new BondSet(bs)));
-        // occasionally BondSet appears unknown in Eclipse even though it
-        // is defined
-        // in Eval.java -- doesn't seem to matter.
         else
           addTokenToPrefix(new Token(Token.bitset, bs));
         return CONTINUE;
@@ -1128,19 +1203,18 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
   private int parseKnownToken(String ident) {
 
     // specific token-based issues depend upon where we are in the command
-    
+
     Token token;
 
     if (tokLastMath != 0)
       tokLastMath = theTok;
-    if (flowContext != null && flowContext.token.tok == Token.switchcmd 
-        && flowContext.var != null 
-        && theTok != Token.casecmd && theTok != Token.defaultcmd
-        && lastToken.tok != Token.switchcmd)
+    if (flowContext != null && flowContext.token.tok == Token.switchcmd
+        && flowContext.var != null && theTok != Token.casecmd
+        && theTok != Token.defaultcmd && lastToken.tok != Token.switchcmd)
       return ERROR(ERROR_badContext, ident);
     switch (theTok) {
     case Token.identifier:
-      if (nTokens == 0) {
+      if (nTokens == 0 && !checkImpliedScriptCmd) {
         if (ident.charAt(0) == '\'') {
           addTokenToPrefix(setCommand(Token.tokenScript));
           cchToken = 0;
@@ -1150,43 +1224,46 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
           addTokenToPrefix(setCommand(Token.tokenScript));
           nTokens = 1;
           cchToken = 0;
+          checkImpliedScriptCmd = true;
           return CONTINUE;
         }
       }
       break;
     case Token.andequals:
-      if (theTok == Token.andequals) {
-        if (nSemiSkip == forPoint3 && nTokens == ptSemi + 2) {
-          token = lastToken;
-          addTokenToPrefix(Token.tokenEquals);
-          addTokenToPrefix(token);
-          token = Token.getTokenFromName(ident.substring(0, 1));
-          addTokenToPrefix(token);
-          addTokenToPrefix(Token.tokenLeftParen);
-          needRightParen = true;
-          return CONTINUE;
-        }
-        if (tokCommand == Token.set) {
-          tokenAndEquals = Token.getTokenFromName(ident.substring(0, 1));
-          setEqualPt = ichToken;
-          return OK;
-        }
-        if (tokCommand == Token.slab || tokCommand == Token.depth) {
-          addTokenToPrefix(tokenCommand);
-          replaceCommand(Token.tokenSet);
-          tokenAndEquals = Token.getTokenFromName(ident.substring(0, 1));
-          setEqualPt = ichToken;
-          return OK;
-        }
-        // otherwise ignore
+      if (nSemiSkip == forPoint3 && nTokens == ptSemi + 2) {
+        token = lastToken;
+        addTokenToPrefix(Token.tokenEquals);
+        addTokenToPrefix(token);
+        token = Token.getTokenFromName(ident.substring(0, 1));
+        addTokenToPrefix(token);
+        addTokenToPrefix(Token.tokenLeftParen);
+        needRightParen = true;
         return CONTINUE;
       }
-      break;
+      // check to see if we have a command name that
+      // was not registered yet as a local variable:
+      // var color = 3
+      // color += 3
+      checkNewSetCommand();
+      if (tokCommand == Token.set) {
+        tokenAndEquals = Token.getTokenFromName(ident.substring(0, 1));
+        setEqualPt = ichToken;
+        return OK;
+      }
+      if (tokCommand == Token.slab || tokCommand == Token.depth) {
+        addTokenToPrefix(tokenCommand);
+        replaceCommand(Token.tokenSet);
+        tokenAndEquals = Token.getTokenFromName(ident.substring(0, 1));
+        setEqualPt = ichToken;
+        return OK;
+      }
+      // otherwise ignore
+      return CONTINUE;
     case Token.end:
     case Token.endifcmd:
       if (flowContext != null)
-       flowContext.forceEndIf = false;
-       // fall through
+        flowContext.forceEndIf = false;
+      // fall through
     case Token.elsecmd:
       if (nTokens > 0) {
         isEndOfCommand = true;
@@ -1195,7 +1272,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       }
       break;
     case Token.forcmd:
-      if (bracketCount > 0)  // ignore [FOR], as in 1C4D 
+      if (bracketCount > 0) // ignore [FOR], as in 1C4D 
         break;
       // fall through
     case Token.casecmd:
@@ -1210,27 +1287,23 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         if (flowContext != null)
           flowContext.forceEndIf = true;
         cchToken = 0;
-        return CONTINUE;            
+        return CONTINUE;
       }
       break;
     case Token.minusMinus:
     case Token.plusPlus:
       if (!isNewSet && nTokens == 1)
         checkNewSetCommand();
-      if (isNewSet && parenCount == 0 && bracketCount == 0 && ichToken <= setEqualPt) {
-        nTokens = ltoken.size();
-        addTokenToPrefix(Token.tokenEquals);
-        setEqualPt = 0;
-        for (int i = 1; i < nTokens; i++)
-          addTokenToPrefix((Token)ltoken.get(i));
-        addTokenToPrefix(theTok == Token.minusMinus ? Token.tokenMinus : Token.tokenPlus);
-        addTokenToPrefix(Token.intToken(1));
-        return CONTINUE;  
-      } else if (nSemiSkip == forPoint3 && nTokens == ptSemi + 2) {            
+      if (isNewSet && parenCount == 0 && bracketCount == 0
+          && ichToken <= setEqualPt) {
+        tokenizePlusPlus(theTok, false);
+        return CONTINUE;
+      } else if (nSemiSkip == forPoint3 && nTokens == ptSemi + 2) {
         token = lastToken;
         addTokenToPrefix(Token.tokenEquals);
         addTokenToPrefix(token);
-        addTokenToPrefix(theTok == Token.minusMinus ? Token.tokenMinus : Token.tokenPlus);
+        addTokenToPrefix(theTok == Token.minusMinus ? Token.tokenMinus
+            : Token.tokenPlus);
         addTokenToPrefix(Token.intToken(1));
         return CONTINUE;
       }
@@ -1240,12 +1313,13 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         setEqualPt = ichToken;
       break;
     case Token.per:
-      if (tokCommand == Token.set && parenCount == 0 && bracketCount == 0 && ichToken < setEqualPt) {
+      if (tokCommand == Token.set && parenCount == 0 && bracketCount == 0
+          && ichToken < setEqualPt) {
         ltoken.add(1, Token.tokenExpressionBegin);
         addTokenToPrefix(Token.tokenExpressionEnd);
         ltoken.set(0, Token.tokenSetProperty);
         setEqualPt = 0;
-      }            
+      }
       break;
     case Token.leftbrace:
       braceCount++;
@@ -1262,8 +1336,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       // but we must differentiate from isosurface select(...) and set
       // picking select
       if (nTokens > 1
-          && (lastToken.tok == Token.select
-              || lastToken.tok == Token.forcmd || lastToken.tok == Token.ifcmd))
+          && (lastToken.tok == Token.select || lastToken.tok == Token.forcmd || lastToken.tok == Token.ifcmd))
         nSemiSkip += 2;
       break;
     case Token.rightbrace:
@@ -1309,8 +1382,24 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return OK;
   }
 
+  private void tokenizePlusPlus(int tok, boolean isPlusPlusX) {
+    //   ++ipt;   or ipt++
+    if (isPlusPlusX) {
+      setCommand(Token.tokenSet);
+      ltoken.add(0, tokenCommand);
+    }
+    nTokens = ltoken.size();
+    addTokenToPrefix(Token.tokenEquals);
+    setEqualPt = 0;
+    for (int i = 1; i < nTokens; i++)
+      addTokenToPrefix(ltoken.get(i));
+    addTokenToPrefix(tok == Token.minusMinus ? Token.tokenMinus
+        : Token.tokenPlus);
+    addTokenToPrefix(Token.intToken(1));
+  }
+
   private boolean checkNewSetCommand() {
-    String name = ((Token) ltoken.get(0)).value.toString();
+    String name = ltoken.get(0).value.toString();
     if (!isContextVariable(name.toLowerCase()))
       return false;
     Token t = setNewSetCommand(false, name);
@@ -1326,7 +1415,6 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     // checking tokens based on the current command
     // all command starts are handled by case Token.nada
 
-    
     nTokens = ltoken.size();
     switch (tokCommand) {
     case Token.nada:
@@ -1395,9 +1483,13 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         switch (theTok) {
         case Token.plusPlus:
         case Token.minusMinus:
+          tokInitialPlusPlus = theTok;
+          tokCommand = Token.nada;
+          return CONTINUE;
         case Token.identifier:
         case Token.var:
         case Token.define:
+        case Token.leftparen:
           break;
         default:
           if (!Token.tokAttr(theTok, Token.misc)
@@ -1417,7 +1509,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
           return ERROR(ERROR_tokenExpected, "(");
         break; 
       case 2:
-        ((ContextToken)tokenCommand).name0 = ident;
+        if (theTok != Token.rightparen)
+          ((ContextToken)tokenCommand).name0 = ident;
         addContextVariable(ident);
         break;
       case 3:
@@ -1538,10 +1631,12 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
       } else if (nTokens == 3 && tokAt(2) == Token.var) {
         addContextVariable(ident);
       } else if ((nTokens == 3 || nTokens == 4) && theTok == Token.in) {
-        nSemiSkip -= 2;
-        forPoint3 = 2;
         // for ( var x IN
         // for ( x IN
+        nSemiSkip -= 2;
+        forPoint3 = 2;
+        addTokenToPrefix(theToken);
+        theToken = Token.tokenLeftParen;
       } else if (braceCount == 0 && parenCount == 0) {
         isEndOfCommand = true;
         ichEnd = ichToken + 1;
@@ -1565,8 +1660,7 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
           setBraceCount = 0;
           break;
         }
-        if (theTok != Token.identifier && theTok != Token.andequals 
-            && theTok != Token.define
+        if (theTok != Token.identifier && theTok != Token.andequals && theTok != Token.define
             && (!Token.tokAttr(theTok, Token.setparam))) {
           if (isNewSet)
             commandExpected();
@@ -1651,8 +1745,9 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     case Token.pmesh:
     case Token.isosurface:
       // isosurface ... name.xxx
+      char ch = nextChar();
       if (parenCount == 0 && bracketCount == 0
-          && ".:/\\+-!?".indexOf(nextChar()) >= 0)
+          && ".:/\\+-!?".indexOf(ch) >= 0 && !(ch == '-' && ident.equals("=")))
         checkUnquotedFileName();
     }
     return OK;
@@ -1664,8 +1759,8 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     setBraceCount = (isSetBrace ? 1 : 0);
     bracketCount = 0;
     setEqualPt = Integer.MAX_VALUE;
-    ptNewSetModifier = (isNewSet ? 1 : Integer.MAX_VALUE);
-    return (isSetBrace ? theToken : new Token(Token.identifier, ident));
+    ptNewSetModifier = (isNewSet ? (ident.equals("(") ? 2 : 1) : Integer.MAX_VALUE);
+    return ((isSetBrace || theToken.tok == Token.plusPlus || theToken.tok == Token.minusMinus)? theToken : new Token(Token.identifier, ident));
   }
 
   private char nextChar() {
@@ -1700,17 +1795,17 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return true;
   }
 
-  List vPush = new ArrayList();
+  List<Token> vPush = new ArrayList<Token>();
   int pushCount;
   
   private int checkFlowEndBrace() {
     
     if (iBrace <= 0
-        || ((Token) vBraces.get(iBrace - 1)).tok != Token.rightbrace)
+        || vBraces.get(iBrace - 1).tok != Token.rightbrace)
       return OK;
     // time to execute end
     vBraces.remove(--iBrace);
-    Token token = (Token) vBraces.remove(--iBrace);
+    Token token = vBraces.remove(--iBrace);
     if (theTok == Token.leftbrace) {
       braceCount--;
       parenCount--;
@@ -1821,6 +1916,9 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     case Token.continuecmd:
       isNew = false;
       ScriptFlowContext f = (flowContext == null ? null : flowContext.getBreakableContext(0));
+      if (tokCommand == Token.continuecmd)
+        while (f != null  && f.token.tok != Token.forcmd && f.token.tok != Token.whilecmd)
+          f = f.getParent();
       if (f == null)
         return error(ERROR_badContext, ident);
       setCommand( new Token(tokCommand, f.pt0, ident)); //copy
@@ -2040,10 +2138,12 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
         break;
       previousCharBackslash = (ch == '\\' ? !previousCharBackslash : false);
     }
-    if (ichT == cchScript)
+    if (ichT == cchScript) {
       cchToken = -1;
-    else
+      ichEnd = cchScript;
+    } else {
       cchToken = ++ichT - ichToken;
+    }
     return true;
   }
 
@@ -2097,19 +2197,16 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     return sb.toString();
   }
 
-  // static String[] loadFormats = { "append", "files", "trajectory", "menu",
-  // "models",
-  // /*ancient:*/ "alchemy", "mol2", "mopac", "nmrpdb", "charmm", "xyz", "mdl",
-  // "pdb" };
-
   private boolean lookingAtLoadFormat() {
     // just allow a simple word or =xxxx or $CCCC
+    // old load formats are simple unneeded words like PDB or XYZ -- no numbers
     int ichT = ichToken;
     char ch = '\0';
     boolean allchar = (ichT < cchScript && ((ch = script.charAt(ichT)) == '$' || ch == '='));
     while (ichT < cchScript
-        && (Character.isLetterOrDigit(ch = script.charAt(ichT)) || allchar
-            && (!eol(ch) && !Character.isWhitespace(ch))))
+        && (Character.isLetterOrDigit(ch = script.charAt(ichT))
+            && (allchar || Character.isLetter(ch))
+            || allchar && (!eol(ch) && !Character.isWhitespace(ch))))
       ++ichT;
     if (!allchar && ichT == ichToken || !isSpaceOrTab(ch))
       return false;
@@ -2124,10 +2221,11 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
    * characters not involving white space.
    * echo, hover, label, message, pause are odd-valued; no initial parsing of variables for them. 
    * @param allowSpace 
+   * @param allowEquals TODO
    * 
    * @return true or false
    */
-  private boolean lookingAtImpliedString(boolean allowSpace) {
+  private boolean lookingAtImpliedString(boolean allowSpace, boolean allowEquals) {
     int ichT = ichToken;
     char ch = script.charAt(ichT);
     boolean parseVariables = !(Token.tokAttr(tokCommand, Token.implicitStringCommand) 
@@ -2145,13 +2243,19 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     int parenpt = 0;
     while (isOK && ichT < cchScript && !eol(ch = script.charAt(ichT))) {
       switch (ch) {
+      case '=':
+        if (!allowEquals) {
+          isOK = false;
+          continue;
+        }
+          break;
       case '{':
         parenpt++;
         break;
       case '}':
         // only consider this if it is extra
         parenpt--;
-        if (parenpt < 0 && braceCount > 0) {
+        if (parenpt < 0 && (braceCount > 0 || iBrace > 0)) {
           isOK = false;
           continue;
         }
@@ -2528,6 +2632,10 @@ public class ScriptCompiler extends ScriptCompilationTokenParser {
     errorMessage = GT._("script compiler ERROR: ") + errorMessage
          + ScriptEvaluator.setErrorLineMessage(null, filename, lineCurrent, iCommand, lineInfo);
     if (!isSilent) {
+      ichToken = Math.max(ichEnd, ichToken);
+      while (!lookingAtEndOfLine() && !lookingAtEndOfStatement())
+        ichToken++;
+      errorLine = script.substring(ichCurrentCommand, ichToken);      
       viewer.addCommand(errorLine + CommandHistory.ERROR_FLAG);
       Logger.error(errorMessage);
     }
