@@ -61,7 +61,7 @@ public class ScriptVariable extends Token {
   private final static int FLAG_LOCALVAR = 2;
 
   private int flags = ~FLAG_CANINCREMENT & FLAG_LOCALVAR;
-  String name;
+  private String myName;
 
   public ScriptVariable() {
     tok = string;
@@ -118,6 +118,7 @@ public class ScriptVariable extends Token {
     return "?";
   }
 
+  @SuppressWarnings("unchecked")
   public static int sizeOf(Token x) {
     switch (x == null ? nada : x.tok) {
     case bitset:
@@ -143,7 +144,7 @@ public class ScriptVariable extends Token {
       return x.intValue == Integer.MAX_VALUE ? ((ScriptVariable)x).getList().size()
           : sizeOf(selectItem(x));
     case hash:
-      return ((Map) x.value).size();
+      return ((Map<String, ScriptVariable>) x.value).size();
     default:
       return 0;
     }
@@ -164,11 +165,12 @@ public class ScriptVariable extends Token {
         || x instanceof Point4f    // stored as point4f
         || x instanceof Quaternion // stored as point4f
         || x instanceof String
-        || x instanceof Map  // stored as Map<String, ScriptVariable>
-        || x instanceof List    // stored as list
+        || x instanceof Map<?, ?>  // stored as Map<String, ScriptVariable>
+        || x instanceof List<?>    // stored as list
         || x instanceof ScriptVariable[] // stored as list
         || x instanceof double[]   // stored as list
         || x instanceof float[]    // stored as list
+        || x instanceof float[][]  // stored as list
         || x instanceof Float[]    // stored as list
         || x instanceof int[]      // stored as list
         || x instanceof int[][]    // stored as list
@@ -181,6 +183,7 @@ public class ScriptVariable extends Token {
    * @return  a ScriptVariable of the input type, or if x is null, then a new ScriptVariable,
    *     or, if the type is not found, a string version
    */
+  @SuppressWarnings("unchecked")
   public static ScriptVariable getVariable(Object x) {
     if (x == null)
       return new ScriptVariable();
@@ -228,15 +231,16 @@ public class ScriptVariable extends Token {
     if (x instanceof Float[])
       return new ScriptVariable(listf, x);
     if (x instanceof Map) {
-      Map ht = (Map) x;
-      Iterator e = ht.keySet().iterator();
+      Map<String, Object> ht = (Map<String, Object>) x;
+      Iterator<String> e = ht.keySet().iterator();
       while (e.hasNext()) {
         if (!(ht.get(e.next()) instanceof ScriptVariable)) {
-          Map x2 = new Hashtable();
-          e = ht.keySet().iterator();
-          while (e.hasNext()) {
-            String key = (String) e.next();
-            x2.put(key, getVariable(ht.get(key)));
+          Map<String, ScriptVariable> x2 = new Hashtable<String, ScriptVariable>();
+          for (Map.Entry<String, Object> entry : ht.entrySet()) {
+            String key = entry.getKey();
+            Object o = entry.getValue();
+            x2.put(key, getVariable(isVariableType(o) ? o : Escape.toReadable(
+                null, o)));
           }
           x = x2;
         }
@@ -246,64 +250,69 @@ public class ScriptVariable extends Token {
     }
 
     // all the rest are stored as list
-    // at least for now, still saving as a string, 
-    // but ultimately, that should not be necessary
 
-    List objects = null;
+    List<ScriptVariable> objects = null;
     if (x instanceof List) {
       // will be turned into list
-      List v = (List) x;
+      List<Object> v = (List<Object>) x;
       int len = v.size();
       if (len > 0 && v.get(0) instanceof ScriptVariable)
         return new ScriptVariable(varray, v);
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < len; i++)
         objects.add(getVariable(v.get(i)));
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof ScriptVariable[]) {
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       ScriptVariable[] v = (ScriptVariable[]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < v.length; i++)
         objects.add(v[i]);
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof String[]) {
       String[] s = (String[]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < s.length; i++)
         objects.add(getVariable(s[i]));
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof int[]) {
       int[] ix = (int[]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < ix.length; i++)
         objects.add(getVariable(Integer.valueOf(ix[i])));
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof float[]) {
       float[] f = (float[]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < f.length; i++)
         objects.add(getVariable(Float.valueOf(f[i])));
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof double[]) {
       double[] f = (double[]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < f.length; i++)
         objects.add(getVariable(Float.valueOf((float) f[i])));
       return new ScriptVariable(varray, objects);
     }
     if (x instanceof int[][]) {
       int[][] ix = (int[][]) x;
-      objects = new ArrayList();
+      objects = new ArrayList<ScriptVariable>();
       for (int i = 0; i < ix.length; i++)
         objects.add(getVariable(ix[i]));
       return new ScriptVariable(varray, objects);
     }
+    if (x instanceof float[][]) {
+      float[][] fx = (float[][]) x;
+      objects = new ArrayList<ScriptVariable>();
+      for (int i = 0; i < fx.length; i++)
+        objects.add(getVariable(fx[i]));
+      return new ScriptVariable(varray, objects);
+    }
     return new ScriptVariable(string, Escape.toReadable(x));
   }
 
@@ -316,6 +325,7 @@ public class ScriptVariable extends Token {
    *                for an array; copies an associative array 
    * @return  new ScriptVariable
    */
+  @SuppressWarnings("unchecked")
   public ScriptVariable set(ScriptVariable v, boolean asCopy) {
     // note: bitset, point3f ,point4f will not be copied
     //       because they are essentially immutable here
@@ -326,11 +336,12 @@ public class ScriptVariable extends Token {
     if (asCopy) {
       switch (tok) {
       case hash:
-        value = new Hashtable((Map) v.value);
+        value = new Hashtable<String, ScriptVariable>(
+            (Map<String, ScriptVariable>) v.value);
         break;
       case varray:
-        List o2 = new ArrayList();
-        List o1 = v.getList();
+        List<ScriptVariable> o2 = new ArrayList<ScriptVariable>();
+        List<ScriptVariable> o1 = v.getList();
         for (int i = 0; i < o1.size(); i++)
           o2.add(o1.get(i));
         value = o2;
@@ -341,7 +352,7 @@ public class ScriptVariable extends Token {
   }
 
   public ScriptVariable setName(String name) {
-    this.name = name;
+    this.myName = name;
     flags |= FLAG_CANINCREMENT;
     //System.out.println("Variable: " + name + " " + intValue + " " + value);
     return this;
@@ -417,6 +428,10 @@ public class ScriptVariable extends Token {
       return Boolean.FALSE;
     case integer:
       return Integer.valueOf(x.intValue);
+    case bitset:
+//      return bsSelect(x);
+    case array:
+      return selectItem(x).value;
     default:
       return x.value;
     }
@@ -532,11 +547,11 @@ public class ScriptVariable extends Token {
   }
 
   public static String sValue(Token x) {
-    StringBuffer sb;
-    Map map;
     if (x == null)
       return "";
     int i;
+    StringBuffer sb;
+    Map<Object, Boolean> map;
     switch (x.tok) {
     case on:
       return "true";
@@ -544,28 +559,20 @@ public class ScriptVariable extends Token {
       return "false";
     case integer:
       return "" + x.intValue;
-    case point3f:
-      return Escape.escape((Point3f) x.value);
-    case point4f:
-      return Escape.escape((Point4f) x.value);
-    case matrix3f:
-      return Escape.escape((Matrix3f) x.value);
-    case matrix4f:
-      return Escape.escape((Matrix4f) x.value);
     case bitset:
       return Escape.escape(bsSelect(x), !(x.value instanceof BondSet));
     case varray:
     case hash:
       if (x.tok == Token.varray) {
-        List sv = ((ScriptVariable) x).getList();
+        List<ScriptVariable> sv = ((ScriptVariable) x).getList();
         i = x.intValue;
         if (i <= 0)
           i = sv.size() - i;
         if (i != Integer.MAX_VALUE)
-          return (i < 1 || i > sv.size() ? "" : sValue((ScriptVariable)  sv.get(i - 1)));
+          return (i < 1 || i > sv.size() ? "" : sValue(sv.get(i - 1)));
       }
       sb = new StringBuffer();
-      map = new Hashtable();
+      map = new Hashtable<Object, Boolean>();
       sValueArray(sb, (ScriptVariable) x, map, 0, false);
       return sb.toString();
     case string:
@@ -578,24 +585,29 @@ public class ScriptVariable extends Token {
       if (i < 1 || i > s.length())
         return "";
       return "" + s.charAt(i - 1);
-    case decimal:
+    case point3f:
+    case point4f:
+    case matrix3f:
+    case matrix4f:
+      return Escape.escape(x.value);
     default:
-      return "" + x.value;
+      return x.value.toString();
     }
   }
 
+  @SuppressWarnings("unchecked")
   private static void sValueArray(StringBuffer sb, ScriptVariable vx,
-                                  Map map, int level,
+                                  Map<Object, Boolean> map, int level,
                                   boolean isEscaped) {
     switch (vx.tok) {
     case hash:
       if (map.containsKey(vx)) {
-        sb.append(isEscaped ? "{}" : vx.name == null ? "<circular reference>"
-            : "<" + vx.name + ">");
+        sb.append(isEscaped ? "{}" : vx.myName == null ? "<circular reference>"
+            : "<" + vx.myName + ">");
         break;
       }
       map.put(vx, Boolean.TRUE);
-      Map ht = (Map) vx.value;
+      Map<String, ScriptVariable> ht = (Map<String, ScriptVariable>) vx.value;
       Object[] keys = ht.keySet().toArray();
       Arrays.sort(keys);
 
@@ -605,7 +617,7 @@ public class ScriptVariable extends Token {
         for (int i = 0; i < keys.length; i++) {
           String key = (String) keys[i];
           sb.append(sep).append(Escape.escape(key)).append(':');
-          sValueArray(sb, (ScriptVariable) ht.get(key), map, level + 1, true);
+          sValueArray(sb, ht.get(key), map, level + 1, true);
           sep = ", ";
         }
         sb.append(" }");
@@ -623,18 +635,18 @@ public class ScriptVariable extends Token {
       break;
     case varray:
       if (map.containsKey(vx)) {
-        sb.append(isEscaped ? "[]" : vx.name == null ? "<circular reference>"
-            : "<" + vx.name + ">");
+        sb.append(isEscaped ? "[]" : vx.myName == null ? "<circular reference>"
+            : "<" + vx.myName + ">");
         break;
       }
       map.put(vx, Boolean.TRUE);
       if (isEscaped)
         sb.append("[");
-      List sx = vx.getList();
+      List<ScriptVariable> sx = vx.getList();
       for (int i = 0; i < sx.size(); i++) {
         if (isEscaped && i > 0)
           sb.append(",");
-        ScriptVariable sv = (ScriptVariable) sx.get(i);
+        ScriptVariable sv = sx.get(i);
         sValueArray(sb, sv, map, level + 1, isEscaped);
         if (!isEscaped)
           sb.append("\n");
@@ -685,8 +697,8 @@ public class ScriptVariable extends Token {
 
   public static ScriptVariable concatList(ScriptVariable x1, ScriptVariable x2,
                                           boolean asNew) {
-    List v1 = x1.getList();
-    List v2 = x2.getList();
+    List<ScriptVariable> v1 = x1.getList();
+    List<ScriptVariable> v2 = x2.getList();
     if (!asNew) {
       if (v2 == null)
         v1.add(new ScriptVariable(x2));
@@ -695,7 +707,7 @@ public class ScriptVariable extends Token {
           v1.add(v2.get(i));
       return x1;
     }
-    List vlist = new ArrayList(
+    List<ScriptVariable> vlist = new ArrayList<ScriptVariable>(
         (v1 == null ? 1 : v1.size()) + (v2 == null ? 1 : v2.size()));
 
     if (v1 == null)
@@ -883,12 +895,12 @@ public class ScriptVariable extends Token {
       if (i1 < 1 || i1 > len || i2 > len)
         return new ScriptVariable(string, "");
       if (i2 == i1)
-        return (ScriptVariable) ((ScriptVariable) tokenIn).getList().get(i1 - 1);
-      List o2 = new ArrayList();
-      List o1 = ((ScriptVariable) tokenIn).getList();
+        return ((ScriptVariable) tokenIn).getList().get(i1 - 1);
+      List<ScriptVariable> o2 = new ArrayList<ScriptVariable>();
+      List<ScriptVariable> o1 = ((ScriptVariable) tokenIn).getList();
       n = i2 - i1 + 1;
       for (int i = 0; i < n; i++)
-        o2.add(new ScriptVariable((ScriptVariable) o1.get(i + i1 - 1)));
+        o2.add(new ScriptVariable(o1.get(i + i1 - 1)));
       tokenOut.value = o2;
       break;
     }
@@ -916,11 +928,11 @@ public class ScriptVariable extends Token {
       }
       if (selector != 0 && Math.abs(selector) <= len
           && var.tok == varray) {
-        List sv = var.getList();
+        List<ScriptVariable> sv = var.getList();
         if (sv.size() == len) {
           float[] data = new float[len];
           for (int i = 0; i < len; i++)
-            data[i] = fValue((ScriptVariable)  sv.get(i));
+            data[i] = fValue(sv.get(i));
           if (selector > 0) {
             if (tok == matrix3f)
               ((Matrix3f) value).setRow(selector - 1, data);
@@ -966,30 +978,16 @@ public class ScriptVariable extends Token {
 
   public String escape() {
     switch (tok) {
-    case on:
-      return "true";
-    case off:
-      return "false";
-    case integer:
-      return "" + intValue;
-    case bitset:
-      return Escape.escape((BitSet)value, !(value instanceof BondSet));
+    case string:
+      return Escape.escape(value);
     case varray:
     case hash:
       StringBuffer sb = new StringBuffer();
-      Map map = new Hashtable();
+      Map<Object,Boolean>map = new Hashtable<Object,Boolean>();
       sValueArray(sb, this, map, 0, true);
-      return sb.toString();      
-    case point3f:
-      return Escape.escape((Point3f)value);
-    case point4f:
-      return Escape.escape((Point4f)value);
-    case matrix3f:
-      return Escape.escape((Matrix3f)value);
-    case matrix4f:
-      return Escape.escape((Matrix4f)value);
+      return sb.toString();
     default:
-      return Escape.escape(value);
+      return sValue(this);
     }
   }
 
@@ -1055,10 +1053,10 @@ public class ScriptVariable extends Token {
     Object[] of = new Object[] { vd, vf, ve, null, null};
     if (var.tok != varray)
       return sprintf(strFormat, var, of, vd, vf, ve, getS, getP);
-    List sv = var.getList();
+    List<ScriptVariable> sv = var.getList();
     String[] list2 = new String[sv.size()];
     for (int i = 0; i < list2.length; i++)
-      list2[i] = sprintf(strFormat, (ScriptVariable)  sv.get(i), of, vd, vf, ve, getS, getP);
+      list2[i] = sprintf(strFormat, sv.get(i), of, vd, vf, ve, getS, getP);
     return list2;
   }
 
@@ -1106,19 +1104,21 @@ public class ScriptVariable extends Token {
     return sb.toString();
   }
   
+  @Override
   public String toString() {
-    return super.toString() + "[" + name + "] index =" + index + " hashcode=" + hashCode();
+    return super.toString() + "[" + myName + " index =" + index + " intValue=" + intValue + "]";
   }
 
+  @SuppressWarnings("unchecked")
   public static BitSet getBitSet(ScriptVariable x, boolean allowNull) {
     switch (x.tok) {
     case bitset:
       return bsSelect(x);
     case varray:
       BitSet bs = new BitSet();
-      List sv = (List) x.value;
+      List<ScriptVariable> sv = (ArrayList<ScriptVariable>) x.value;
       for (int i = 0; i < sv.size(); i++)
-        if (!((ScriptVariable) sv.get(i)).unEscapeBitSetArray(bs) && allowNull)
+        if (!sv.get(i).unEscapeBitSetArray(bs) && allowNull)
           return null;
       return bs;
     }
@@ -1126,6 +1126,8 @@ public class ScriptVariable extends Token {
   }
 
   public static boolean areEqual(ScriptVariable x1, ScriptVariable x2) {
+    if (x1 == null || x2 == null)
+      return false;
     if (x1.tok == string && x2.tok == string)
       return sValue(x1).equalsIgnoreCase(
           sValue(x2));
@@ -1137,16 +1139,14 @@ public class ScriptVariable extends Token {
         - fValue(x2)) < 0.000001);
   }
 
-  protected class Sort implements Comparator {
+  protected class Sort implements Comparator<ScriptVariable> {
     private int arrayPt;
     
     protected Sort(int arrayPt) {
       this.arrayPt = arrayPt;
     }
     
-    public int compare(Object ox, Object oy) {
-      ScriptVariable x = (ScriptVariable) ox;
-      ScriptVariable y = (ScriptVariable) oy;
+    public int compare(ScriptVariable x, ScriptVariable y) {
       if (x.tok != y.tok) {
         if (x.tok == Token.decimal || x.tok == Token.integer
             || y.tok == Token.decimal || y.tok == Token.integer) {
@@ -1161,8 +1161,8 @@ public class ScriptVariable extends Token {
       case string:
         return sValue(x).compareTo(sValue(y));
       case varray:
-        List sx = x.getList();
-        List sy = y.getList();
+        List<ScriptVariable> sx = x.getList();
+        List<ScriptVariable> sy = y.getList();
         if (sx.size() != sy.size())
           return (sx.size() < sy.size() ? -1 : 1);
         int iPt = arrayPt;
@@ -1185,14 +1185,14 @@ public class ScriptVariable extends Token {
    * @return sorted or reversed array
    */
   public ScriptVariable sortOrReverse(int arrayPt) {
-    List x = getList();
+    List<ScriptVariable> x = getList();
     if (x == null || x.size() < 2) 
       return this;
     if (arrayPt == Integer.MIN_VALUE) {
       // reverse
       int n = x.size();
       for (int i = 0; i < n; i++) {
-        ScriptVariable v = (ScriptVariable) x.get(i);
+        ScriptVariable v = x.get(i);
         x.set(i, x.get(--n));
         x.set(n, v);
       }
@@ -1217,10 +1217,10 @@ public class ScriptVariable extends Token {
     return false;   
   }
 
-  public static BitSet unEscapeBitSetArray(List x, boolean allowNull) {
+  public static BitSet unEscapeBitSetArray(ArrayList<ScriptVariable> x, boolean allowNull) {
     BitSet bs = new BitSet();
     for (int i = 0; i < x.size(); i++)
-      if (!((ScriptVariable) x.get(i)).unEscapeBitSetArray(bs) && allowNull)
+      if (!x.get(i).unEscapeBitSetArray(bs) && allowNull)
         return null;
     return bs;
   }
@@ -1228,23 +1228,23 @@ public class ScriptVariable extends Token {
   public static String[] listValue(Token x) {
     if (x.tok != varray)
       return new String[] { sValue(x) };
-    List sv = ((ScriptVariable) x).getList();
+    List<ScriptVariable> sv = ((ScriptVariable) x).getList();
     String[] list = new String[sv.size()];
     for (int i = sv.size(); --i >= 0;)
-      list[i] = sValue((ScriptVariable) sv.get(i));
+      list[i] = sValue(sv.get(i));
     return list;
   }
 
   public static float[] flistValue(Token x, int nMin) {
     if (x.tok != varray)
       return new float[] { fValue(x) };
-    List sv = ((ScriptVariable) x).getList();
+    List<ScriptVariable> sv = ((ScriptVariable) x).getList();
     float[] list;
     list = new float[Math.max(nMin, sv.size())];
     if (nMin == 0)
       nMin = list.length;
     for (int i = Math.min(sv.size(), nMin); --i >= 0;)
-      list[i] = fValue((ScriptVariable) sv.get(i));
+      list[i] = fValue(sv.get(i));
     return list;
   }
 
@@ -1265,7 +1265,7 @@ public class ScriptVariable extends Token {
       return;
     }
     tok = varray;
-    List o2 = new ArrayList(dim);
+    List<ScriptVariable> o2 = new ArrayList<ScriptVariable>(dim);
     for (int i = 0; i < dim; i++) {
       float[] a = new float[dim];
       if (m3 == null)
@@ -1277,11 +1277,13 @@ public class ScriptVariable extends Token {
     value = o2;
   }
 
+  @SuppressWarnings("unchecked")
   public ScriptVariable mapValue(String key) {
-    return (tok == hash ? (ScriptVariable) ((Map) value).get(key) : null);
+    return (tok == hash ? ((Map<String, ScriptVariable>) value).get(key) : null);
   }
 
-  public List getList() {
-    return (tok == varray ? (List) value : null);
+  @SuppressWarnings("unchecked")
+  public List<ScriptVariable> getList() {
+    return (tok == varray ? (ArrayList<ScriptVariable>) value : null);
   }
 }
