@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2011-02-13 06:52:46 +0100 (dim., 13 févr. 2011) $
- * $Revision: 15159 $
+ * $Date: 2011-09-19 23:17:35 +0200 (lun., 19 sept. 2011) $
+ * $Revision: 16082 $
  *
  * Copyright (C) 2005  The Jmol Development Team
  *
@@ -24,18 +24,19 @@
 
 package org.jmol.smiles;
 
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
 import java.util.Hashtable;
 import java.util.List;
-import java.util.ArrayList;
+import java.util.Map;
 
 import javax.vecmath.Point3f;
 import javax.vecmath.Vector3f;
 
-import org.jmol.api.JmolEdge;
-import org.jmol.api.JmolMolecule;
-import org.jmol.api.JmolNode;
+import org.jmol.util.JmolEdge;
+import org.jmol.util.JmolMolecule;
+import org.jmol.util.JmolNode;
 import org.jmol.util.Logger;
 
 /**
@@ -46,8 +47,9 @@ import org.jmol.util.Logger;
  */
 public class SmilesSearch extends JmolMolecule {
 
+  @Override
   public String toString() {
-    StringBuffer sb = new StringBuffer(pattern);
+    StringBuilder sb = new StringBuilder(pattern);
     sb.append("\nmolecular formula: " + getMolecularFormula(true));
     return sb.toString();    
   }
@@ -80,10 +82,10 @@ public class SmilesSearch extends JmolMolecule {
   boolean haveBondStereochemistry;
   boolean haveAtomStereochemistry;
   boolean needRingData;
-  boolean needAromatic;
+  boolean needAromatic = true; // we just have to always consider aromatic, except in the case of bioSequences.
   boolean needRingMemberships;
   int ringDataMax = Integer.MIN_VALUE;
-  List measures = new ArrayList();
+  List<SmilesMeasure> measures = new ArrayList<SmilesMeasure>();
   boolean noAromatic;
   boolean ignoreStereochemistry;
   StringBuffer ringSets;
@@ -103,11 +105,11 @@ public class SmilesSearch extends JmolMolecule {
   private int[] ringCounts;
   private int[] ringConnections;
   private BitSet bsFound = new BitSet(); 
-  private Hashtable htNested;
+  private Map<String, Object> htNested;
   private int nNested;
   private SmilesBond nestedBond;
 
-  private List vReturn;
+  private List<Object> vReturn;
   private BitSet bsReturn = new BitSet();
     
 
@@ -134,7 +136,7 @@ public class SmilesSearch extends JmolMolecule {
 
   int addNested(String pattern) {
     if (parent.htNested == null)
-      parent.htNested = new Hashtable();
+      parent.htNested = new Hashtable<String, Object>();
     setNested(++parent.nNested, pattern);
     return parent.nNested;
   }
@@ -156,7 +158,10 @@ public class SmilesSearch extends JmolMolecule {
     return n;
   }
 
+  @SuppressWarnings("unchecked")
   void setRingData(BitSet bsA) throws InvalidSmilesException {
+    if (needAromatic)
+      needRingData = true;
     needAromatic &= (bsA == null) & !noAromatic;
     // when using "xxx".find("search","....")
     // or $(...), the aromatic set has already been determined
@@ -185,13 +190,12 @@ public class SmilesSearch extends JmolMolecule {
         continue;
       String smarts = "*1" + s.substring(0, i - 2) + "*1";
       SmilesSearch search = SmilesParser.getMolecule(smarts, true);
-      List v = (List) getBitSets(search, false, true);
+      List<Object> v = (List<Object>) getBitSets(search, false, true);
       if (needAromatic)
         for (int r = v.size(); --r >= 0;) {
           BitSet bs = (BitSet) v.get(r);
-          if (SmilesAromatic.isFlatSp2Ring(
-              jmolAtoms, bsSelected, bs, 0.01f))
-          for (int j = bs.nextSetBit(0); j >= 0; j = bs.nextSetBit(j + 1)) 
+          if (SmilesAromatic.isFlatSp2Ring(jmolAtoms, bsSelected, bs, 0.01f))
+            for (int j = bs.nextSetBit(0); j >= 0; j = bs.nextSetBit(j + 1))
               bsAromatic.set(j);
         }
       if (needRingData) {
@@ -205,7 +209,8 @@ public class SmilesSearch extends JmolMolecule {
       }
     }
     if (needRingData) {
-      for (int i = bsSelected.nextSetBit(0); i >= 0; i = bsSelected.nextSetBit(i + 1)) {
+      for (int i = bsSelected.nextSetBit(0); i >= 0; i = bsSelected
+          .nextSetBit(i + 1)) {
         JmolNode atom = jmolAtoms[i];
         JmolEdge[] bonds = atom.getEdges();
         if (bonds != null)
@@ -261,7 +266,7 @@ public class SmilesSearch extends JmolMolecule {
   /* ============================================================= */
 
   /** 
-   * the start of the search. ret will be either a List or a BitSet
+   * the start of the search. ret will be either a Vector or a BitSet
    * @param firstAtomOnly TODO
    * @return BitSet or Vector
    * @throws InvalidSmilesException 
@@ -300,7 +305,7 @@ public class SmilesSearch extends JmolMolecule {
       Logger.debug("SmilesSearch processing " + pattern);
 
     if (vReturn == null && (asVector || getMaps))
-      vReturn = new ArrayList();
+      vReturn = new ArrayList<Object>();
     if (bsSelected == null) {
       bsSelected = new BitSet(jmolAtomCount);
       bsSelected.set(0, jmolAtomCount);
@@ -376,6 +381,7 @@ public class SmilesSearch extends JmolMolecule {
               return true;
         }
       }
+      
       // Check bonds
 
       jmolBonds = jmolAtom.getEdges();
@@ -394,7 +400,7 @@ public class SmilesSearch extends JmolMolecule {
 
         // BIOSMILES/BIOSMARTS check is by group
 
-        switch (patternBond.bondType) {
+        switch (patternBond.order) {
         case SmilesBond.TYPE_BIO_SEQUENCE:
         case SmilesBond.TYPE_BIO_PAIR:
           if (!checkMatchBond(patternAtom, atom1, patternBond, iAtom,
@@ -451,6 +457,13 @@ public class SmilesSearch extends JmolMolecule {
 
       // so far, so good... not done yet... on to the next position...
 
+      /*
+      if (patternAtoms[atomNum].toString().indexOf("*") < 0) {
+        for (int i = 0; i < atomNum; i++)System.out.println(i + " " + patternAtoms[i]);
+      System.out.println("");
+      }
+      */
+      
       SmilesAtom newPatternAtom = patternAtoms[atomNum];
       // For all the pattern bonds for this atom...
       // find the bond to atoms already assigned.
@@ -485,7 +498,8 @@ public class SmilesSearch extends JmolMolecule {
               bs.set(ii);
           }
         }
-        boolean skipGroup = (iAtom >= 0 && newPatternAtom.isBioAtom && (newPatternAtom.atomName == null || newPatternAtom.residueChar != null));
+        boolean skipGroup = (iAtom >= 0 && newPatternAtom.isBioAtom 
+            && (newPatternAtom.atomName == null || newPatternAtom.residueChar != null));
         for (int j = bsSelected.nextSetBit(0); j >= 0; j = bsSelected
             .nextSetBit(j + 1)) {
           if (!bs.get(j)
@@ -515,13 +529,13 @@ public class SmilesSearch extends JmolMolecule {
       // For sequences, we go to the next GROUP, either via
       // the standard sequence or via basepair/cysteine pairing. 
 
-      switch (newPatternBond.bondType) {
+      switch (newPatternBond.order) {
       case SmilesBond.TYPE_BIO_SEQUENCE:
         int nextGroupAtom = jmolAtom.getOffsetResidueAtom(
             newPatternAtom.atomName, 1);
         if (nextGroupAtom >= 0) {
           BitSet bs = (BitSet) bsFound.clone();
-          jmolAtom.setGroupBits(bsFound);
+          jmolAtom.getGroupBits(bsFound);
 
           // working here
 
@@ -531,13 +545,12 @@ public class SmilesSearch extends JmolMolecule {
         }
         return true;
       case SmilesBond.TYPE_BIO_PAIR:
-        List vLinks = new ArrayList();
+        List<Integer> vLinks = new ArrayList<Integer>();
         jmolAtom.getCrossLinkLeadAtomIndexes(vLinks);
         BitSet bs = (BitSet) bsFound.clone();
-        jmolAtom.setGroupBits(bsFound);
+        jmolAtom.getGroupBits(bsFound);
         for (int j = 0; j < vLinks.size(); j++)
-          if (!checkMatch(newPatternAtom, atomNum, ((Integer) vLinks.get(j))
-              .intValue(), firstAtomOnly))
+          if (!checkMatch(newPatternAtom, atomNum, vLinks.get(j).intValue(), firstAtomOnly))
             return false;
         bsFound = bs;
         return true;
@@ -591,7 +604,7 @@ public class SmilesSearch extends JmolMolecule {
       nMatch++;
       bs.set(i);
       if (patternAtoms[j].isBioAtom && patternAtoms[j].atomName == null)
-        jmolAtoms[i].setGroupBits(bs);
+        jmolAtoms[i].getGroupBits(bs);
       if (firstAtomOnly)
         break;
       if (!isSmarts && patternAtoms[j].missingHydrogenCount > 0)
@@ -663,6 +676,10 @@ public class SmilesSearch extends JmolMolecule {
   private boolean checkPrimitiveAtom(SmilesAtom patternAtom, int iAtom) throws InvalidSmilesException {
     JmolNode atom = jmolAtoms[iAtom];
     boolean foundAtom = patternAtom.not;
+    
+    //if (pattern.indexOf("*") < 0)
+      //System.out.println("testing smilessearch");
+
     while (true) {
 
       int n;
@@ -743,7 +760,7 @@ public class SmilesSearch extends JmolMolecule {
 
         // Check aromatic
         boolean isAromatic = patternAtom.isAromatic();
-        if (!noAromatic && patternAtom.elementNumber != -2
+        if (!noAromatic && !patternAtom.aromaticAmbiguous
             && isAromatic != bsAromatic.get(iAtom))
           break;
 
@@ -763,7 +780,6 @@ public class SmilesSearch extends JmolMolecule {
           break;
 
         // H explicit H count
-        //n = patternAtom.missingHydrogenCount;
         //problem here is that you can have C[H]
         n = patternAtom.getCovalentHydrogenCount() + patternAtom.missingHydrogenCount;
         if (n >= 0 && n != atom.getCovalentHydrogenCount())
@@ -847,45 +863,53 @@ public class SmilesSearch extends JmolMolecule {
     }
 
     if (patternBond.primitives == null) {
-      if (!checkPrimitiveBond(patternAtom, atom1, patternBond, iAtom, matchingAtom, bond))
+      if (!checkPrimitiveBond(patternBond, iAtom, matchingAtom, bond))
         return false;
     } else {
       for (int i = 0; i < patternBond.nPrimitives; i++)
-        if (!checkPrimitiveBond(patternAtom, atom1, patternBond.primitives[i], iAtom, matchingAtom, bond))
+        if (!checkPrimitiveBond(patternBond.primitives[i], iAtom, matchingAtom, bond))
           return false;
     }
     patternBond.matchingBond = bond;
     return true;
   }
 
-  private boolean checkPrimitiveBond(SmilesAtom patternAtom, SmilesAtom atom1,
-                                     SmilesBond patternBond, int iAtom,
-                                     int matchingAtom, JmolEdge bond) {
+  private boolean checkPrimitiveBond(SmilesBond patternBond, int iAtom1,
+                                     int iAtom2, JmolEdge bond) {
     boolean bondFound = false;
     
-    switch (patternBond.bondType) {
+    switch (patternBond.order) {
     case SmilesBond.TYPE_BIO_SEQUENCE:
-      return (patternBond.isNot != (jmolAtoms[matchingAtom].getOffsetResidueAtom("0", 1)
-          == jmolAtoms[iAtom].getOffsetResidueAtom("0", 0)));
+      return (patternBond.isNot != (jmolAtoms[iAtom2].getOffsetResidueAtom("0", 1)
+          == jmolAtoms[iAtom1].getOffsetResidueAtom("0", 0)));
     case SmilesBond.TYPE_BIO_PAIR:
-      return (patternBond.isNot != jmolAtoms[iAtom].isCrossLinked(jmolAtoms[matchingAtom]));
+      return (patternBond.isNot != jmolAtoms[iAtom1].isCrossLinked(jmolAtoms[iAtom2]));
     }
     
-    boolean isAromatic = !noAromatic && patternAtom.isAromatic();
-
-    
+    boolean isAromatic1 = (!noAromatic && bsAromatic.get(iAtom1));
+    boolean isAromatic2 = (!noAromatic && bsAromatic.get(iAtom2));
+    //if (pattern.indexOf("*") < 0)
+      //System.out.println("testing smilessearch");
     int order = bond.getCovalentOrder();
-    if (isAromatic && atom1.isAromatic()) {
-      switch (patternBond.bondType) {
+    if (isAromatic1 && isAromatic2) {
+      switch (patternBond.order) {
       case SmilesBond.TYPE_AROMATIC: // :
-      case SmilesBond.TYPE_DOUBLE:
       case SmilesBond.TYPE_RING:
-        bondFound = isRingBond(ringSets, iAtom, matchingAtom);
+        bondFound = isRingBond(ringSets, iAtom1, iAtom2);
         break;
       case SmilesBond.TYPE_SINGLE:
-        // for SMARTS, single bond in aromatic means 
-        // TO ANOTHER RING
-        bondFound = !isSmarts || !isRingBond(ringSets, iAtom, matchingAtom);
+        // for SMARTS, single bond in aromatic means TO ANOTHER RING;
+        // for SMILES, we don't care
+        bondFound = !isSmarts || !isRingBond(ringSets, iAtom1, iAtom2);
+        break;
+      case SmilesBond.TYPE_DOUBLE:
+        // note: Freiburg considers TYPE_DOUBLE to be NOT aromatic
+        // changed for Jmol 12.2.RC8
+        // but this is ambiguous at http://www.daylight.com/dayhtml/doc/theory/theory.smarts.html
+        // see, for example: http://opentox.informatik.uni-freiburg.de/depict?data=[H]C%3D1C%28[H]%29%3DC%28[H]%29C%28%3DC%28C%3D1%28[H]%29%29C%28F%29%28F%29F%29S[H]&smarts=[%236]=[%236]
+        // however, if it is not SMARTS, then we consider this fine -- it does
+        // not matter what the order is for double/single bonds around the ring
+        bondFound = !isSmarts;
         break;
       case SmilesBond.TYPE_ATROPISOMER_1:
       case SmilesBond.TYPE_ATROPISOMER_2:
@@ -895,7 +919,7 @@ public class SmilesSearch extends JmolMolecule {
         break;
       }
     } else {
-      switch (patternBond.bondType) {
+      switch (patternBond.order) {
       case SmilesBond.TYPE_ANY:
       case SmilesBond.TYPE_UNKNOWN:
         bondFound = true;
@@ -921,7 +945,7 @@ public class SmilesSearch extends JmolMolecule {
         bondFound = (order == JmolEdge.BOND_COVALENT_TRIPLE);
         break;
       case SmilesBond.TYPE_RING:
-        bondFound = isRingBond(ringSets, iAtom, matchingAtom);
+        bondFound = isRingBond(ringSets, iAtom1, iAtom2);
         break;
       }
     }
@@ -941,7 +965,7 @@ public class SmilesSearch extends JmolMolecule {
     // first, @ stereochemistry
 
     for (int i = 0; i < measures.size(); i++)
-      if (!((SmilesMeasure) measures.get(i)).check())
+      if (!measures.get(i).check())
         return false;
     if (haveAtomStereochemistry) {
 
@@ -951,6 +975,18 @@ public class SmilesSearch extends JmolMolecule {
       JmolNode atom1 = null, atom2 = null, atom3 = null, atom4 = null, atom5 = null, atom6 = null;
       SmilesAtom sAtom1 = null, sAtom2 = null;
       JmolNode[] jn;
+      //for debugging, first try SET DEBUG
+      //for (int i = 0; i < atomCount; i++) {
+      //  SmilesAtom sAtom = patternAtoms[i];
+      //  System.out.print(sAtom + "=");
+      //}
+      //System.out.println("");
+      //for (int i = 0; i < atomCount; i++) {
+      //  SmilesAtom sAtom = patternAtoms[i];
+      //  JmolNode atom0 = jmolAtoms[sAtom.getMatchingAtom()];
+      //  System.out.print(atom0.getIndex() + "-");
+      //}
+      //System.out.println("");
       for (int i = 0; i < atomCount; i++) {
         SmilesAtom sAtom = patternAtoms[i];
         JmolNode atom0 = jmolAtoms[sAtom.getMatchingAtom()];
@@ -965,7 +1001,7 @@ public class SmilesSearch extends JmolMolecule {
           return false;
         atom4 = null;
         if (Logger.debugging)
-          Logger.debug("...type " + chiralClass);
+          Logger.debug("...type " + chiralClass + " for pattern atom " + sAtom + " " + atom0);
         switch (chiralClass) {
         //        case SmilesAtom.STEREOCHEMISTRY_DOUBLE_BOND:
         case SmilesAtom.STEREOCHEMISTRY_ALLENE:
@@ -1021,16 +1057,16 @@ public class SmilesSearch extends JmolMolecule {
 
           if (isSmilesFind) {
             if (jn[1] == null)
-              getX(sAtom, sAtom2, jn, 1, false, isAllene);
+              getX(sAtom, jn, 1, false, isAllene);
             if (jn[3] == null)
-              getX(sAtom2, sAtom, jn, 3, false, false);
+              getX(sAtom2, jn, 3, false, false);
             if (!setSmilesCoordinates(atom0, sAtom, sAtom2, jn))
               return false;
           }
           if (jn[1] == null)
-            getX(sAtom, sAtom2, jn, 1, true, false);
+            getX(sAtom, jn, 1, true, false);
           if (jn[3] == null)
-            getX(sAtom2, sAtom, jn, 3, true, false);
+            getX(sAtom2, jn, 3, true, false);
           if (!checkStereochemistry(sAtom.not, atom0, chiralClass, order,
               jn[0], jn[1], jn[2], jn[3], null, null, v))
             return false;
@@ -1072,10 +1108,11 @@ public class SmilesSearch extends JmolMolecule {
               && !setSmilesCoordinates(atom0, sAtom, sAtom2, new JmolNode[] {
                   atom1, atom2, atom3, atom4, atom5, atom6 }))
             return false;
-
+            //System.out.println(order + ":" + sAtom + "/" + atom0 + ":" + atom1 + "," + atom2 + "," + atom3 + "," + atom4);
           if (!checkStereochemistry(sAtom.not, atom0, chiralClass, order,
               atom1, atom2, atom3, atom4, atom5, atom6, v))
             return false;
+            //System.out.println("OK");
           continue;
         }
       }
@@ -1097,7 +1134,7 @@ public class SmilesSearch extends JmolMolecule {
         for (int j = 0; j < nBonds; j++) {
           b = sAtom1.getBond(j);
           boolean isAtom2 = (b.getAtom2() == sAtom1);
-          int type = b.bondType;
+          int type = b.order;
           switch (type) {
           case SmilesBond.TYPE_ATROPISOMER_1:
           case SmilesBond.TYPE_ATROPISOMER_2:
@@ -1134,7 +1171,7 @@ public class SmilesSearch extends JmolMolecule {
           for (int j = 0; j < nBonds && dir2 == 0; j++) {
             b = sAtom2.getBond(j);
             boolean isAtom2 = (b.getAtom2() == sAtom2);
-            int type = b.bondType;
+            int type = b.order;
             switch (type) {
             case SmilesBond.TYPE_DIRECTIONAL_1:
             case SmilesBond.TYPE_DIRECTIONAL_2:
@@ -1180,7 +1217,7 @@ public class SmilesSearch extends JmolMolecule {
 
   }
 
-  private void getX(SmilesAtom sAtom, SmilesAtom sAtom2, JmolNode[] jn, int pt,
+  private void getX(SmilesAtom sAtom, JmolNode[] jn, int pt,
                     boolean haveCoordinates, boolean needHSwitch) {
     JmolNode atom = getJmolAtom(sAtom.getMatchingAtom());
     boolean doSwitch = sAtom.isFirst || pt == 3;
@@ -1342,7 +1379,7 @@ public class SmilesSearch extends JmolMolecule {
         atom.set(-1, (nBonds++ == 0) ? -1 : 1, 0);
         int mode = (bond.getAtomIndex2() == dbAtom1.getIndex() ? nBonds
             : -nBonds);
-        switch (bond.getOrder()) {
+        switch (bond.order) {
         case JmolEdge.BOND_STEREO_NEAR:
           dir1 = mode;
           break;
@@ -1363,7 +1400,7 @@ public class SmilesSearch extends JmolMolecule {
         atom.set(1, (nBonds++ == 0) ? 1 : -1, 0);
         int mode = (bond.getAtomIndex2() == dbAtom2.getIndex() ? nBonds
             : -nBonds);
-        switch (bond.getOrder()) {
+        switch (bond.order) {
         case JmolEdge.BOND_STEREO_NEAR:
           dir2 = mode;
           break;
@@ -1393,7 +1430,7 @@ public class SmilesSearch extends JmolMolecule {
       for (int k = bonds.length; --k >= 0;) {
         JmolEdge bond = bonds[k];
         if (bond.getOtherAtom(dbAtom1) == dbAtom2) {
-          dir = (bond.getOrder() == JmolEdge.BOND_PARTIAL01 ? 1 : -1);
+          dir = (bond.order == JmolEdge.BOND_PARTIAL01 ? 1 : -1);
           break;
         }
       }
@@ -1561,6 +1598,8 @@ public class SmilesSearch extends JmolMolecule {
     v.vB.set((Point3f) atomB);
     v.vA.sub((Point3f) atom1);
     v.vB.sub((Point3f) atom2);
+    v.vA.normalize();
+    v.vB.normalize();
     // -0.95f about 172 degrees
     return (v.vA.dot(v.vB) < f);
   }
@@ -1576,11 +1615,12 @@ public class SmilesSearch extends JmolMolecule {
    */
   private static int getHandedness(JmolNode a, JmolNode b, JmolNode c, JmolNode pt, VTemp v) {
     float d = SmilesAromatic.getNormalThroughPoints(a, b, c, v.vTemp, v.vA, v.vB);
+    //int atat = (distanceToPlane(v.vTemp, d, (Point3f) pt) > 0 ? 1 : 2);
     //System.out.println("draw p1 @{point" + new Point3f((Point3f)a) +"} color red");
     //System.out.println("draw p2 @{point" + new Point3f((Point3f)b)+"} color green");
     //System.out.println("draw p3 @{point" + new Point3f((Point3f)c)+"} color blue");
-    //System.out.println("draw p plane @{point" + new Point3f((Point3f)a) +"} @{point" + new Point3f((Point3f)b)+"} @{point" + new Point3f((Point3f)c)+"}");
-    //System.out.println("draw v vector {0 0 0} @{point" + v.vTemp+"}");
+    //System.out.println("draw p @{point" + new Point3f((Point3f)a) +"} @{point" + new Point3f((Point3f)b)+"} @{point" + new Point3f((Point3f)c)+"}");
+    //System.out.println("draw v vector @{point" + new Point3f((Point3f)pt) + "} @{point" + v.vTemp+"} \""+ (atat==2 ? "@@" : "@")+"\" color " + (atat == 2 ? "white" : "yellow"));
     return (distanceToPlane(v.vTemp, d, (Point3f) pt) > 0 ? 1 : 2);
   }
 
@@ -1680,7 +1720,7 @@ public class SmilesSearch extends JmolMolecule {
         //SmilesBond b;
         if (firstAtom) {
           int order = 1;
-          switch (sBond.bondType) {
+          switch (sBond.order) {
           // these first two are for cis/trans alkene
           // stereochemistry; we co-opt stereo near/far here
           case SmilesBond.TYPE_ATROPISOMER_1:
@@ -1697,7 +1737,7 @@ public class SmilesSearch extends JmolMolecule {
             break;
           case SmilesBond.TYPE_BIO_PAIR:
           case SmilesBond.TYPE_BIO_SEQUENCE:
-            order = sBond.bondType;
+            order = sBond.order;
             break;
           case SmilesBond.TYPE_SINGLE:
             order = JmolEdge.BOND_COVALENT_SINGLE;
