@@ -27,10 +27,9 @@ package org.jmol.util;
 import java.io.DataInputStream;
 import java.io.BufferedInputStream;
 import java.io.InputStream;
-//import java.io.RandomAccessFile;
-import java.util.Hashtable;
-import java.util.List;
 import java.util.ArrayList;
+import java.util.List;
+import java.util.Map;
 
 
 /* a simple compound document reader. 
@@ -51,7 +50,7 @@ public class CompoundDocument extends BinaryDocument {
 
 //  RandomAccessFile file;
   CmpDocHeader header = new CmpDocHeader();
-  List directory = new ArrayList(); //i.e.  CmpDocDirectoryEntry[]
+  List<CmpDocDirectoryEntry> directory = new ArrayList<CmpDocDirectoryEntry>();
   CmpDocDirectoryEntry rootEntry;
 
   int[] SAT;
@@ -103,14 +102,14 @@ public class CompoundDocument extends BinaryDocument {
   }
   
 
-  public List getDirectory() {
+  public List<CmpDocDirectoryEntry> getDirectory() {
     return directory;
   }
 
   public String getDirectoryListing(String separator) {
     String str = "";
     for (int i = 0; i < directory.size(); i++) {
-      CmpDocDirectoryEntry thisEntry = (CmpDocDirectoryEntry) directory.get(i);
+      CmpDocDirectoryEntry thisEntry = directory.get(i);
       if (!thisEntry.isEmpty)
         str += separator
             + thisEntry.entryName
@@ -145,18 +144,18 @@ public class CompoundDocument extends BinaryDocument {
    * @param fileData
    */
   public void getAllData(String prefix, 
-                         String binaryFileList, Hashtable fileData) {
+                         String binaryFileList, Map<String, String> fileData) {
     fileData.put("#Directory_Listing", getDirectoryListing("|"));
     binaryFileList = "|" + binaryFileList + "|";
     for (int i = 0; i < directory.size(); i++) {
-      CmpDocDirectoryEntry thisEntry = (CmpDocDirectoryEntry) directory.get(i);
+      CmpDocDirectoryEntry thisEntry = directory.get(i);
       String name = thisEntry.entryName;
       Logger.info("reading " + name);
       if (!thisEntry.isEmpty && thisEntry.entryType != 5) {
         boolean isBinary = (binaryFileList != null && binaryFileList.indexOf("|" + thisEntry.entryName + "|") >= 0);
         if (isBinary)
           name += ":asBinaryString";
-        StringBuffer data = new StringBuffer();
+        StringBuilder data = new StringBuilder();
         data.append("BEGIN Directory Entry ").append(name).append("\n"); 
         data.append(getFileAsString(thisEntry, isBinary));
         data.append("\nEND Directory Entry ").append(name).append("\n");
@@ -169,7 +168,7 @@ public class CompoundDocument extends BinaryDocument {
   public StringBuffer getAllData(String binaryFileList, String firstFile) {
     if (firstFile != null) {
       for (int i = 0; i < directory.size(); i++) {
-        CmpDocDirectoryEntry thisEntry = (CmpDocDirectoryEntry) directory.get(i);
+        CmpDocDirectoryEntry thisEntry = directory.get(i);
         if (thisEntry.entryName.equals(firstFile)) {
           directory.remove(i);
           directory.add(1, thisEntry); // after ROOT_ENTRY
@@ -183,7 +182,7 @@ public class CompoundDocument extends BinaryDocument {
     data.append("\n");
     binaryFileList = "|" + binaryFileList + "|";
     for (int i = 0; i < directory.size(); i++) {
-      CmpDocDirectoryEntry thisEntry = (CmpDocDirectoryEntry) directory.get(i);
+      CmpDocDirectoryEntry thisEntry = directory.get(i);
       Logger.info("reading " + thisEntry.entryName);
       if (!thisEntry.isEmpty && thisEntry.entryType != 5) {
         data.append("BEGIN Directory Entry ").append(thisEntry.entryName).append("\n");            
@@ -198,7 +197,7 @@ public class CompoundDocument extends BinaryDocument {
 
   public StringBuffer getFileAsString(String entryName) {
     for (int i = 0; i < directory.size(); i++) {
-      CmpDocDirectoryEntry thisEntry = (CmpDocDirectoryEntry) directory.get(i);
+      CmpDocDirectoryEntry thisEntry = directory.get(i);
       if (thisEntry.entryName.equals(entryName))
         return getFileAsString(thisEntry, false);
     }
@@ -480,7 +479,7 @@ public class CompoundDocument extends BinaryDocument {
     readByteArray(byteBuf);
     if (asBinaryString) {
       for (int i = 0; i < nSectorBytes; i++) {
-        data.append(Integer.toHexString(((int)byteBuf[i]) & 0xFF)).append(' ');
+        data.append(Integer.toHexString(byteBuf[i] & 0xFF)).append(' ');
         if (--nBytes < 1)
           break;
       }
