@@ -23,15 +23,18 @@
  */
 package org.openscience.jmol.app;
 
-import org.jmol.api.*;
+import java.awt.Dimension;
+import java.awt.Point;
+import java.io.File;
+import java.util.Scanner;
+
 import org.jmol.export.history.HistoryFile;
 import org.jmol.i18n.GT;
+import org.jmol.util.Escape;
+import org.jmol.util.Logger;
+import org.jmol.util.Parser;
+import org.jmol.api.JmolAdapter;
 import org.jmol.api.JmolViewer;
-import org.jmol.util.*;
-
-import java.awt.*;
-import java.io.*;
-import java.util.*;
 
 import org.apache.commons.cli.Options;
 import org.apache.commons.cli.CommandLine;
@@ -47,6 +50,7 @@ public class JmolApp {
    * The data model.
    */
 
+  public int port;
   public int startupWidth, startupHeight;
   public Point border;
   public boolean haveBorder;
@@ -66,6 +70,7 @@ public class JmolApp {
   public boolean haveConsole = true;
   public boolean haveDisplay = true;
   public boolean isDataOnly;
+  public boolean isKiosk;
   public boolean isPrintOnly;
   public boolean isSilent;
   public boolean listCommands;
@@ -146,6 +151,8 @@ public class JmolApp {
         ._("check script syntax only - with file loading"));
     options.addOption("d", "debug", false, GT._("debug"));
     options.addOption("i", "silent", false, GT._("silent startup operation"));
+    options.addOption("k", "kiosk", false, GT
+        ._("kiosk mode -- no frame"));
     options.addOption("l", "list", false, GT
         ._("list commands during script execution"));
     options.addOption("L", "nosplash", false, GT
@@ -159,6 +166,11 @@ public class JmolApp {
     options.addOption("x", "exit", false, GT
         ._("exit after script (implicit with -n)"));
 
+    OptionBuilder.withLongOpt("port");
+    OptionBuilder.withDescription(GT._("port for JSON/MolecularPlayground-style communication"));
+    OptionBuilder.hasArg();
+    options.addOption(OptionBuilder.create("P"));
+    
     OptionBuilder.withLongOpt("script");
     OptionBuilder.withDescription(GT
         ._("script file to execute or '-' for System.in"));
@@ -263,9 +275,27 @@ public class JmolApp {
     // invoked with just new JmolApp(), we can 
     // set options ourselves. 
     
+    commandOptions = (isDataOnly ? "JmolData " : "Jmol ");
+
+    // kiosk mode -- no frame
+    
+    if (line.hasOption("k"))
+      isKiosk = true;
+    if (isKiosk) {
+      commandOptions += "-k";
+    }
+
+    // port for JSON mode communication
+    
+    if (line.hasOption("P"))
+      port =  Parser.parseInt(line.getOptionValue("P"));
+    if (port > 0) {
+      commandOptions += "-P";
+    }
+
+
     // print command output only (implies silent)
 
-    commandOptions = (isDataOnly ? "JmolData " : "Jmol ");
     if (line.hasOption("p"))
       isPrintOnly = true;
     if (isPrintOnly) {
@@ -357,7 +387,8 @@ public class JmolApp {
             + "1.1.2 or higher version VM!!!");
       }
 
-      size = historyFile.getWindowSize("Jmol");
+      if (!isKiosk) {
+        size = historyFile.getWindowSize("Jmol");
       if (size != null) {
         startupWidth = size.width;
         startupHeight = size.height;
@@ -370,12 +401,14 @@ public class JmolApp {
         border = new Point(12, 116);
       else
         border = new Point(b.x, b.y);
-      // note -- the first time this is run after changes it will not work
+        // note -- the first time this is run after changes it will not work
       // because there is a bootstrap problem.
+      }
     }
-    int width = 500;
-    int height = 500;
     // INNER frame dimensions
+    int width = (isKiosk ? 0 : 500);
+    int height = 500;
+
     if (line.hasOption("g")) {
       String geometry = line.getOptionValue("g");
       int indexX = geometry.indexOf('x');
@@ -389,7 +422,7 @@ public class JmolApp {
     }
 
     if (startupWidth <= 0 || startupHeight <= 0) {
-      if (haveDisplay) {
+      if (haveDisplay && !isKiosk) {
         startupWidth = width + border.x;
         startupHeight = height + border.y;
       } else {
