@@ -1,7 +1,7 @@
 /* $RCSfile$
  * $Author: hansonr $
- * $Date: 2010-01-17 00:53:08 +0100 (dim., 17 janv. 2010) $
- * $Revision: 12151 $
+ * $Date: 2011-12-22 14:32:44 +0100 (jeu., 22 déc. 2011) $
+ * $Revision: 16625 $
  *
  * Copyright (C) 2002-2005  The Jmol Development Team
  *
@@ -24,6 +24,7 @@
 package org.openscience.jmol.app.jmolpanel;
 
 import org.jmol.api.*;
+import org.jmol.console.KeyJMenuItem;
 import org.jmol.i18n.GT;
 import org.jmol.util.Logger;
 import org.openscience.jmol.app.jmolpanel.GuiMap;
@@ -39,6 +40,8 @@ import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.util.Dictionary;
+import java.util.Map;
 import java.util.Properties;
 import java.util.Hashtable;
 import java.io.FileInputStream;
@@ -73,7 +76,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
   boolean showBoundingBox;
   boolean axesOrientationRasmol;
   boolean openFilePreview;
-  boolean clearConsoleButton;
+  boolean clearHistory;
   float minBondDistance;
   float bondTolerance;
   short marBond;
@@ -87,7 +90,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
   private JCheckBox cbShowAxes, cbShowBoundingBox;
   private JCheckBox cbAxesOrientationRasmol;
   private JCheckBox cbOpenFilePreview;
-  private JCheckBox cbClearConsoleButton;
+  private JCheckBox cbClearHistory;
   private Properties originalSystemProperties;
   private Properties jmolDefaultProperties;
   Properties currentProperties;
@@ -95,7 +98,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
   // The actions:
 
   private PrefsAction prefsAction = new PrefsAction();
-  private Hashtable commands;
+  private Map<String, Action> commands;
 
   final static String[] jmolDefaults  = {
     "jmolDefaults",                   "true",
@@ -105,7 +108,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     "showAxes",                       "false",
     "showBoundingBox",                "false",
     "axesOrientationRasmol",          "false",
-	"openFilePreview",                "true",
+    "openFilePreview",                "true",
     "percentVdwAtom",                 "23",
     "autoBond",                       "true",
     "marBond",                        "150",
@@ -137,11 +140,12 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     this.setTitle(GT._("Preferences"));
 
     initVariables();
-    commands = new Hashtable();
+    commands = new Hashtable<String, Action>();
     Action[] actions = getActions();
     for (int i = 0; i < actions.length; i++) {
       Action a = actions[i];
-      commands.put(a.getValue(Action.NAME), a);
+      Object name = a.getValue(Action.NAME);
+      commands.put((name != null) ? name.toString() : null, a);
     }
     JPanel container = new JPanel();
     container.setLayout(new BorderLayout());
@@ -166,9 +170,9 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     rasmolDefaultsButton.addActionListener(this);
     buttonPanel.add(rasmolDefaultsButton);
 
-    cancelButton = new JButton(GT._("Cancel"));
-    cancelButton.addActionListener(this);
-    buttonPanel.add(cancelButton);
+    //cancelButton = new JButton(GT._("Cancel"));
+    //cancelButton.addActionListener(this);
+    //buttonPanel.add(cancelButton);
 
     applyButton = new JButton(GT._("Apply"));
     applyButton.addActionListener(this);
@@ -266,10 +270,10 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     cbOpenFilePreview.addItemListener(checkBoxListener);
     otherPanel.add(cbOpenFilePreview);
     
-    cbClearConsoleButton =
-        guimap.newJCheckBox("Prefs.clearConsoleButton", clearConsoleButton);
-    cbClearConsoleButton.addItemListener(checkBoxListener);
-    otherPanel.add(cbClearConsoleButton);
+    cbClearHistory =
+        guimap.newJCheckBox("Prefs.clearHistory", clearHistory);
+    cbClearHistory.addItemListener(checkBoxListener);
+    otherPanel.add(cbClearHistory);
     
     constraints = new GridBagConstraints();
     constraints.gridwidth = GridBagConstraints.REMAINDER;
@@ -311,11 +315,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     vdwPercentSlider.addChangeListener(new ChangeListener() {
 
       public void stateChanged(ChangeEvent e) {
-
-        JSlider source = (JSlider) e.getSource();
-        percentVdwAtom = source.getValue();
-        viewer.setIntProperty("PercentVdwAtom", percentVdwAtom);
-        currentProperties.put("percentVdwAtom", "" + percentVdwAtom);
+        rebond();
       }
     });
     sfPanel.add(vdwPercentSlider, BorderLayout.CENTER);
@@ -338,6 +338,11 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     return atomPanel;
   }
 
+  @SuppressWarnings("unchecked")
+  private Dictionary<Object, Object> getJSliderLabelTable(JSlider slider) {
+    return slider.getLabelTable();
+  }
+
   public JPanel buildBondPanel() {
 
     JPanel bondPanel = new JPanel();
@@ -397,18 +402,15 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     for (int i = 0; i <= 250; i += 50) {
       String label = "" + (1000 + i);
       label = "0." + label.substring(1);
-      bwSlider.getLabelTable().put(new Integer(i),
-                                   new JLabel(label, SwingConstants.CENTER));
-      bwSlider.setLabelTable(bwSlider.getLabelTable());
+      Dictionary<Object, Object> labelTable = getJSliderLabelTable(bwSlider);
+      labelTable.put(Integer.valueOf(i),
+          new JLabel(label, SwingConstants.CENTER));
+      bwSlider.setLabelTable(labelTable);
     }
     bwSlider.addChangeListener(new ChangeListener() {
 
       public void stateChanged(ChangeEvent e) {
-
-        JSlider source = (JSlider) e.getSource();
-        marBond = (short)source.getValue();
-        viewer.setIntProperty("bondRadiusMilliAngstroms", marBond);
-        currentProperties.put("marBond", "" + marBond);
+        rebond();
       }
     });
 
@@ -433,34 +435,24 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     btSlider.setMajorTickSpacing(20);
     btSlider.setMinorTickSpacing(10);
     btSlider.setPaintLabels(true);
-    btSlider.getLabelTable().put(new Integer(0),
-        new JLabel("0.0", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
-    btSlider.getLabelTable().put(new Integer(20),
-        new JLabel("0.2", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
-    btSlider.getLabelTable().put(new Integer(40),
-        new JLabel("0.4", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
-    btSlider.getLabelTable().put(new Integer(60),
-        new JLabel("0.6", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
-    btSlider.getLabelTable().put(new Integer(80),
-        new JLabel("0.8", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
-    btSlider.getLabelTable().put(new Integer(100),
-        new JLabel("1.0", SwingConstants.CENTER));
-    btSlider.setLabelTable(btSlider.getLabelTable());
+    Dictionary<Object, Object> labelTable = getJSliderLabelTable(btSlider);
+    labelTable.put(Integer.valueOf(0), new JLabel("0.0", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(20), new JLabel("0.2", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(40), new JLabel("0.4", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(60), new JLabel("0.6", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(80), new JLabel("0.8", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(100), new JLabel("1.0", SwingConstants.CENTER));
+    btSlider.setLabelTable(labelTable);
 
     btSlider.addChangeListener(new ChangeListener() {
 
       public void stateChanged(ChangeEvent e) {
-
-        JSlider source = (JSlider) e.getSource();
-        bondTolerance = source.getValue() / 100f;
-        viewer.setFloatProperty("bondTolerance", bondTolerance);
-        currentProperties.put("bondTolerance", "" + bondTolerance);
-        viewer.rebond();
+        rebond();
       }
     });
     btPanel.add(btSlider);
@@ -484,34 +476,23 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     bdSlider.setMajorTickSpacing(20);
     bdSlider.setMinorTickSpacing(10);
     bdSlider.setPaintLabels(true);
-    bdSlider.getLabelTable().put(new Integer(0),
-        new JLabel("0.0", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
-    bdSlider.getLabelTable().put(new Integer(20),
-        new JLabel("0.2", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
-    bdSlider.getLabelTable().put(new Integer(40),
-        new JLabel("0.4", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
-    bdSlider.getLabelTable().put(new Integer(60),
-        new JLabel("0.6", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
-    bdSlider.getLabelTable().put(new Integer(80),
-        new JLabel("0.8", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
-    bdSlider.getLabelTable().put(new Integer(100),
-        new JLabel("1.0", SwingConstants.CENTER));
-    bdSlider.setLabelTable(bdSlider.getLabelTable());
+    labelTable = getJSliderLabelTable(bdSlider);
+    labelTable.put(Integer.valueOf(0), new JLabel("0.0", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(20), new JLabel("0.2", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(40), new JLabel("0.4", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(60), new JLabel("0.6", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(80), new JLabel("0.8", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
+    labelTable.put(Integer.valueOf(100), new JLabel("1.0", SwingConstants.CENTER));
+    bdSlider.setLabelTable(labelTable);
 
     bdSlider.addChangeListener(new ChangeListener() {
-
       public void stateChanged(ChangeEvent e) {
-
-        JSlider source = (JSlider) e.getSource();
-        minBondDistance = source.getValue() / 100f;
-        viewer.setFloatProperty("minBondDistance", minBondDistance);
-        currentProperties.put("minBondDistance", "" + minBondDistance);
-        viewer.rebond();
+        rebond();
       }
     });
     bdPanel.add(bdSlider);
@@ -558,7 +539,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     cbAxesOrientationRasmol.setSelected(viewer.getAxesOrientationRasmol());
     
     cbOpenFilePreview.setSelected(openFilePreview);
-    cbClearConsoleButton.setSelected(clearConsoleButton);
+    cbClearHistory.setSelected(clearHistory);
 
     // Atom panel controls: 
     vdwPercentSlider.setValue(viewer.getPercentVdwAtom());
@@ -571,6 +552,12 @@ public class PreferencesDialog extends JDialog implements ActionListener {
 
   }
 
+  private void apply() {
+    rebond();
+    save();
+    viewer.refresh(3, "PreferencesDialog:apply()");
+  }
+
   private void save() {
     try {
       FileOutputStream fileOutputStream =
@@ -580,7 +567,6 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     } catch (Exception e) {
       Logger.error("Error saving preferences", e);
     }
-    viewer.refresh(3, "PreferencesDialog:save()");
   }
 
   void initializeProperties() {
@@ -610,6 +596,27 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     updateComponents();
   }
 
+  void rebond() {
+    percentVdwAtom = vdwPercentSlider.getValue();
+    viewer.setIntProperty("PercentVdwAtom", percentVdwAtom);
+    currentProperties.put("percentVdwAtom", "" + percentVdwAtom);
+
+    bondTolerance = btSlider.getValue() / 100f;
+    viewer.setFloatProperty("bondTolerance", bondTolerance);
+    currentProperties.put("bondTolerance", "" + bondTolerance);
+    
+    minBondDistance = bdSlider.getValue() / 100f;
+    viewer.setFloatProperty("minBondDistance", minBondDistance);
+    currentProperties.put("minBondDistance", "" + minBondDistance);
+    
+    marBond = (short)bwSlider.getValue();
+    viewer.setIntProperty("bondRadiusMilliAngstroms", marBond);
+    currentProperties.put("marBond", "" + marBond);
+    
+    viewer.rebond();
+    viewer.refresh(3, "PreferencesDialog:rebond()");
+  }
+  
   void initVariables() {
 
     autoBond = Boolean.getBoolean("autoBond");
@@ -621,7 +628,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     showBoundingBox = Boolean.getBoolean("showBoundingBox");
     axesOrientationRasmol = Boolean.getBoolean("axesOrientationRasmol");
     openFilePreview = Boolean.valueOf(System.getProperty("openFilePreview", "true")).booleanValue();
-    clearConsoleButton = Boolean.getBoolean("clearConsoleButton");
+    clearHistory = Boolean.getBoolean("clearHistory");
 
     minBondDistance =
       new Float(currentProperties.getProperty("minBondDistance")).floatValue();
@@ -657,7 +664,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     }
 
     public void actionPerformed(ActionEvent e) {
-      show();
+      setVisible(true);
     }
   }
 
@@ -669,7 +676,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
   }
 
   protected Action getAction(String cmd) {
-    return (Action) commands.get(cmd);
+    return commands.get(cmd);
   }
 
   ItemListener checkBoxListener = new ItemListener() {
@@ -680,7 +687,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
     public void itemStateChanged(ItemEvent e) {
 
       JCheckBox cb = (JCheckBox) e.getSource();
-      String key = GuiMap.getKey(cb);
+      String key = KeyJMenuItem.getKey(cb);
       boolean isSelected = cb.isSelected();
       String strSelected = isSelected ? "true" : "false";
       if (key.equals("Prefs.showHydrogens")) {
@@ -710,11 +717,13 @@ public class PreferencesDialog extends JDialog implements ActionListener {
         viewer.setBooleanProperty("axesOrientationRasmol", isSelected);
         currentProperties.put("axesOrientationRasmol", strSelected);
       } else if (key.equals("Prefs.openFilePreview")) {
-      	openFilePreview = isSelected;
-      	currentProperties.put("openFilePreview", strSelected);
-      } else if (key.equals("Prefs.clearConsoleButton")) {
-        clearConsoleButton = isSelected;
-        currentProperties.put("clearConsoleButton", strSelected);
+        openFilePreview = isSelected;
+        currentProperties.put("openFilePreview", strSelected);
+      } else if (key.equals("Prefs.clearHistory")) {
+        clearHistory = isSelected;
+        currentProperties.put("clearHistory", strSelected);
+        if (JmolPanel.historyFile != null)
+          JmolPanel.historyFile.addProperty("clearHistory", strSelected);
       }
     }
   };
@@ -727,7 +736,7 @@ public class PreferencesDialog extends JDialog implements ActionListener {
   
   public void actionPerformed(ActionEvent event) {
     if (event.getSource() == applyButton) {
-      save();
+      apply();
     } else if (event.getSource() == jmolDefaultsButton) {
       resetDefaults(null);
     } else if (event.getSource() == rasmolDefaultsButton) {
